# Mitarbeit bei Subato
Bevor mit der Entwicklung begonnen wird, sollten die folgenden Hinweise gelesen werden, damit alles richtig eingerichtet ist und wir bei der Entwicklung von Subato einheitliche Standards haben.

## IDE/Editor
- Im Frontend sollte VSCode (oder alternativ WebStorm) verwendet werden.
- Bei der Verwendung einer anderen IDE/Editor im Frontend muss darauf geachtet werden, dass die Einstellungen in den Dateien `.editorconfig` und `.prettierrc.json` verwendet werden.
- Im Backend sollte IntelliJ verwendet werden.

Für VSCode, WebStorm und IntelliJ befinden sich Konfigurationsdateien direkt im Repository, sodass _vieles_ bereits automatisch eingerichtet sein sollte. 

## Code Style

### Frontend
- Für einen einheitlichen Code Style im Frontend muss [Prettier](https://prettier.io/) für `*.{ts|js|scss}`-Dateien verwendet werden
- Für die Formatierung von anderen Sprachen (wie HTML) sollte der Formatter in VSCode/WebStorm verwendet werden. 
- In TypeScript/JavaScript-Imports müssen absolute Pfade verwendet werden, um Refactorings zu erleichtern. 
- Es sollte die automatische Formatierung/Einrückung beim Speichern eingerichtet werden. (für JetBrains IDEs folgende Zeile in `.idea/workspace.xml` ergänzen: `<component name="FormatOnSaveOptions"><option name="myRunOnSave" value="true" /></component>`)
- Der [Angular Styleguide](https://angular.io/guide/styleguide) sollte beachtet werden.

### Backend
- Es muss der [Google Styleguide zu Java](https://google.github.io/styleguide/javaguide.html) eingehalten werden. Wir weichen geringfügig davon ab, indem wir 4 (statt 2) Spaces zur Einrückung verwenden.

## Sprache
- Wir nutzen die Terminologie der Konzepte im Domänenmodell (siehe `docs.subato.cs.hs-rm.de` -> Spezifikation -> Domäne) zur Kommunikation (Lösung, Aufgabe, Aufgabeninstanz, ...)
- Sämtliche Kommentare, Klassennamen, Methodennamen usw. im Code sind im Gegensatz dazu auf Englisch zu verfassen. Das hat den Grund, dass das System international vorgestellt wird und der Code für Personen verstehbar/erweiterbar sein soll, die kein Deutsch können. In Hinsicht auf die Konzepte im Domänenmodell gibt es daher leider eine Diskrepanz (Lösung <-> Solution), wobei das aufgrund der geringen Komplexität der Begriffe eher unproblematisch ist.
- Die API Dokumentation muss ebenfalls in Englisch verfasst werden. Aktuell wird Deutsch genutzt, wobei das zukünftig umgestellt wird.
- Benutzerhandbuch und Spezifikation unter `docs.subato.cs.hs-rm.de` sind auf Deutsch zu verfassen.

## Dokumentation
- Dokumentiert wird das, was ausreichende Komplexität aufweist. Es werden keine Getter/Setter oder CRUD Operationen kommentiert. Genauso wenig sollen Parameter/Rückgabewerte von Methoden nicht nach dem Stil `@param solution the solution` kommentiert werden. Das kann man sich sparen.
- Erste "Anlaufstelle" für technische Dokumentation sollte immer das JavaDoc direkt im Code sein. Wenn die Dokumentation a) in keine Klasse passt (übergreifende Konzepte) und das auch nicht mit Verlinkungen gelöst werden kann oder b) aufgrund der Komplexität eine Übersicht notwendig ist oder c) es um größere Architekturentscheidungen geht, soll die Dokumentation im `docs`-Ordner des Repositories untergebracht werden. Diese wird dann unter `docs.subato.cs.hs-rm.de` angezeigt.

## Sonstige Konventionen
Wir nutzen eine vereinfachte Version des [Git Workflows](https://nvie.com/posts/a-successful-git-branching-model/). Weiterhin sollten die üblichen Best-Practices (Single Responsibility, DRY usw.) so gut es geht (sofern es verhältnismäßig ist) eingehalten werden. Zudem verwenden wir [Semantic Versioning](https://semver.org/lang/de/). Inkompatible Änderungen an der Subato API müssen zu einer Inkrementierung der Major-Version führen damit eine passende Meldung in den IDE-Plugins angezeigt wird.

## Tests
Wir beschränken uns (aufgrund des hohen Aufwands) primär auf Tests zu Authentifizierung/Autorisierung. Vereinzelt werden auch Aspekte der Geschäftslogik getestet, sofern diese kritisch für Subato sind und manuelle Tests zeitaufwändig sind (z.B. der AoR-Import). Siehe [Dokumentation zu Tests im Backend](docs/backend/test) für weitere Infos zur Testsuite. 

## Release-Workflow
Der Ablauf für einen neuen Release ist:

- Mit `update_version.sh <version>` an sämtlichen Stellen in der Codebase die neue Version setzen und die Änderungen comitten.
- `develop` in `master` mergen
- In GitLab einen neuen Tag `<version>` erstellen