/**
 * @license
 * Copyright Akveo. All Rights Reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */
import { Component } from '@angular/core';
import { NavigationStart, Router } from '@angular/router';
import { NbIconLibraries } from '@nebular/theme';
import { BaseComponent } from './shared/misc/base.component';
import { Observable } from 'rxjs';
import { filter, map, takeUntil, withLatestFrom } from 'rxjs/operators';
import { AuthService } from './auth/auth.service';

import { Store } from '@ngrx/store';
import { SubatoState } from './store/app.reducers';
import { LoadPrincipal } from './auth/store/auth.actions';
import { LoadTerms } from './modules/store/actions';
import { LoadConfig } from './shared/config/store/config.actions';

@Component({
    selector: 'sb-app',
    styleUrls: ['./app.component.scss'],
    template: `
        <div id="root">
            <div class="content">
                <sb-layout-one-column *ngIf="initialized$ | async">
                    <router-outlet appRouteReuseLifecycle></router-outlet>
                </sb-layout-one-column>
            </div>
            <div class="overlay"></div>
        </div>
    `,
})
export class AppComponent extends BaseComponent {
    initialized$: Observable<boolean> = this.store.select(
        (s) => s.app.initialized
    );

    constructor(
        private authService: AuthService,
        private iconLibraries: NbIconLibraries,
        private store: Store<SubatoState>,
        private router: Router
    ) {
        super();

        this.iconLibraries.registerFontPack('fas', {
            packClass: 'fas',
            iconClassPrefix: 'fa',
        });

        router.events
            .pipe(
                takeUntil(this.destroy$),
                filter((routerEvent) => routerEvent instanceof NavigationStart),
                map((routerEvent) => routerEvent as NavigationStart),
                withLatestFrom(this.initialized$),
                filter(([routerEvent, initialized]) => !initialized)
            )
            .subscribe(([routerEvent, _]) => {
                this.initApp(routerEvent.url);
            });
    }

    private initApp(url: string): void {
        this.store.dispatch(new LoadPrincipal());
        this.store.dispatch(new LoadTerms());
        this.store.dispatch(new LoadConfig());
    }
}
