import { User } from 'app/shared/user/user';
import { Solution } from 'app/solutions/models/solution';
import { Task } from 'app/tasks/models/task';

import { Exercise } from 'app/exercises/models/exercise';
import { SubmissionPointRating } from 'app/assessment/models/submission-point-rating';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { TaskInstanceAttempts } from 'app/assessment/models/task-instance-attempts';

export class Submission {
    id: number;
    passed?: boolean;
    submittedBy: User;
    checkedBy: User;
    exercise: Exercise;
    solution: Solution[];
    totalPoints: number;
    pointRatings: SubmissionPointRating[];
    attempts: TaskInstanceAttempts[];
    pointRatingCount: number;

    constructor(submission: any) {
        this.id = submission.id;
        this.submittedBy = new User(submission.submittedBy);
        this.checkedBy = submission.checkedBy
            ? new User(submission.checkedBy)
            : null;
        this.passed = submission.passed;
        this.totalPoints = submission.totalPoints;
        this.attempts = submission.attempts;
        this.pointRatings = submission.pointRatings
            ? submission.pointRatings.map((r) => new SubmissionPointRating(r))
            : [];
        this.exercise = submission.exercise
            ? new Exercise(submission.exercise)
            : null;
        this.solution = submission.solution
            ? submission.solution.map((s) => new Solution(s))
            : [];
        this.attempts = submission.attempts
            ? submission.attempts.map((a) => new TaskInstanceAttempts(a))
            : [];
        this.pointRatingCount = submission.pointRatingCount;
    }

    reachedPointsFor(taskInstance: TaskInstance): number | null {
        return this.pointRatings.find(
            (r) => r.taskInstance.task.id == taskInstance.task.id
        )?.points;
    }

    getNextSubmission(submissions: Submission[]): Submission | null {
        let idx = submissions.map((s) => s.id).indexOf(this.id);
        return idx + 1 < submissions.length ? submissions[idx + 1] : null;
    }

    attemptsExceeded(task: Task, exercise: Exercise): boolean {
        if (task.attempts == null) {
            return false;
        }

        let currentAttempts =
            this.attempts.find(
                (a) => a.exerciseId == exercise.id && a.taskId == task.id
            )?.attempts ?? 0;
        return currentAttempts >= task.attempts;
    }
}
