import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    ExerciseState,
    loadExercise,
} from 'app/exercises/shared/operators/exercise.operators';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

interface ExerciseAssessmentPageState {
    exerciseState: ExerciseState;
}

export const initialState: ExerciseAssessmentPageState = {
    exerciseState: { exercise: null, error: null, loading: true },
};

@Injectable()
export class ExerciseAssessmentPageStore extends BaseComponentStore<ExerciseAssessmentPageState> {
    public readonly exercise$ = this.select((s) => s.exerciseState.exercise);
    public readonly exerciseState$ = this.select((s) => s.exerciseState);

    constructor() {
        super(initialState);
    }

    readonly patchExercise = this.updaterFor('exerciseState');

    readonly loadExercise = this.effect((exerciseId$: Observable<number>) =>
        exerciseId$.pipe(loadExercise((s) => this.patchExercise(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.exercise$.pipe(
            ifAttached(this),
            filter((exercise) => exercise != null),
            map((exercise) => exercise?.course),
            dispatchCourseChange
        )
    );
}
