import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Submission } from 'app/assessment/models/submission';
import { filter, takeUntil } from 'rxjs/operators';
import { DecisionDialogComponent } from 'app/assessment/pages/submission/components/decision-dialog/decision-dialog.component';
import { BaseComponent } from 'app/shared/misc/base.component';
import { NbDialogService } from '@nebular/theme';

@Component({
    selector: 'sb-submission-passed-input',
    templateUrl: './passed-input.component.html',
    styleUrls: ['./passed-input.component.scss'],
})
export class PassedInputComponent extends BaseComponent implements OnInit {
    @Input() submission: Submission;
    @Input() saving: boolean;
    @Output() onPassedChange: EventEmitter<boolean | null> = new EventEmitter<
        boolean | null
    >();

    constructor(private dialogService: NbDialogService) {
        super();
    }

    ngOnInit(): void {}

    openDecision() {
        this.dialogService
            .open(DecisionDialogComponent, {
                context: {
                    submission: this.submission,
                },
            })
            .onClose.pipe(
                takeUntil(this.destroy$),
                filter((result) => result && !result.cancel)
            )
            .subscribe((result) => {
                this.onPassedChange.emit(result.passed);
            });
    }
}
