import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    Output,
    SimpleChanges,
    ViewChild,
} from '@angular/core';
import { Submission } from 'app/assessment/models/submission';
import { TaskInstance } from 'app/exercises/models/task-instance';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { NbPopoverDirective } from '@nebular/theme';

@Component({
    selector: 'sb-submission-points-input',
    templateUrl: './points-input.component.html',
    styleUrls: ['./points-input.component.scss'],
})
export class PointsInputComponent implements OnChanges {
    @ViewChild(NbPopoverDirective) pointsFormPopover: NbPopoverDirective;

    @Input() taskInstance: TaskInstance;
    @Input() submission: Submission;
    @Input() saving: boolean;
    @Output() onSave: EventEmitter<number> = new EventEmitter<number>();

    pointsForm: UntypedFormGroup;

    constructor(private fb: UntypedFormBuilder) {}

    ngOnChanges(changes: SimpleChanges) {
        let shouldRefresh =
            changes.hasOwnProperty('taskInstance') ||
            changes.hasOwnProperty('submission');

        if (shouldRefresh) {
            this.createPointsForm(this.taskInstance, this.submission);
        }
    }

    createPointsForm(taskInstance: TaskInstance, submission: Submission) {
        var points =
            submission.pointRatings.find(
                (r) => r.taskInstance.task.id == taskInstance.task.id
            )?.points ?? '';

        this.pointsForm = this.fb.group({
            points: [
                points,
                [
                    Validators.required,
                    Validators.pattern('^(-[0-9]+)|([0-9]+)$'),
                ],
            ],
        });
    }

    private getPoints(): number | null {
        let pointsInput = this.pointsForm?.get('points');
        if (pointsInput == null || !pointsInput.valid) {
            return null;
        }

        let parsed = parseInt(pointsInput.value);
        return isNaN(parsed) ? null : parsed;
    }

    requestSave() {
        let points = this.getPoints();
        this.onSave.emit(points);
    }

    hide() {
        this.pointsFormPopover.hide();
    }
}
