import { Component, OnInit, ViewChild } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { SubmissionPageStore } from 'app/assessment/pages/submission/submission.page.store';
import { PointsInputComponent } from 'app/assessment/pages/submission/components/points-input/points-input.component';

@Component({
    selector: 'sb-submission-nav',
    templateUrl: './submission-nav.component.html',
    styleUrls: ['./submission-nav.component.scss'],
})
export class SubmissionNavComponent extends BaseComponent implements OnInit {
    @ViewChild(PointsInputComponent) pointsInput: PointsInputComponent;

    isLecturerOrAdmin$ = this.submissionStore.isLecturerOrAdmin$;
    currentTaskInstance$ = this.submissionStore.currentTaskInstance$;
    prevTaskInstance$ = this.submissionStore.prevTaskInstance$;
    nextTaskInstance$ = this.submissionStore.nextTaskInstance$;
    submission$ = this.submissionStore.submission$;
    nextSubmission$ = this.submissionStore.nextSubmission$;
    savingPoints$ = this.submissionStore.savingPoints$;

    constructor(private submissionStore: SubmissionPageStore) {
        super();
    }

    ngOnInit(): void {}

    next() {
        this.submissionStore.navigateToNextTaskInstance();
    }

    prev() {
        this.submissionStore.navigateToPrevTaskInstance();
    }

    navigateToNextSubmission() {
        this.submissionStore.navigateToNextSubmission();
    }

    savePoints(points: number): void {
        this.submissionStore.savePoints({
            points: points,
            done: () => this.pointsInput.hide(),
        });
    }
}
