import { Component, Input, OnChanges, SimpleChanges } from '@angular/core';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Breadcrumb } from 'app/shared/breadcrumbs/breadcrumb';

@Component({
    selector: 'sb-task-instance-assessment-breadcrumbs',
    templateUrl: './breadcrumbs.component.html',
    styleUrls: ['./breadcrumbs.component.scss'],
})
export class BreadcrumbsComponent implements OnChanges {
    @Input() taskInstance: TaskInstance;

    breadcrumbs: Breadcrumb[] = [];

    constructor() {}

    ngOnChanges(changes: SimpleChanges) {
        let shouldRefresh = changes.hasOwnProperty('taskInstance');

        if (shouldRefresh) {
            this.buildBreadcrumbs(this.taskInstance);
        }
    }

    buildBreadcrumbs(taskInstance: TaskInstance) {
        this.breadcrumbs = [
            {
                id: 'exercises',
                params: {
                    course_id: taskInstance.exercise.course.id,
                },
            },
            {
                id: 'exercise',
                params: {
                    exercise_id: taskInstance.exercise.id,
                },
                ctx: taskInstance.exercise,
            },
            {
                id: 'submitSolution',
                params: {
                    exercise_id: taskInstance.exercise.id,
                    task_id: taskInstance.task.id,
                },
                ctx: taskInstance,
            },
            {
                id: 'taskInstanceAssessment',
                params: {
                    exercise_id: taskInstance.exercise.id,
                    task_id: taskInstance.task.id,
                },
            },
        ];
    }
}
