import { Injectable } from '@angular/core';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';
import {
    loadTaskInstance,
    LoadTaskInstanceParams,
    TaskInstanceState,
} from 'app/exercises/shared/operators/task-instance.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

interface TaskInstanceAssessmentPageState {
    taskInstanceState: TaskInstanceState;
}

export const initialState: TaskInstanceAssessmentPageState = {
    taskInstanceState: { taskInstance: null, error: null, loading: true },
};

@Injectable()
export class TaskInstanceAssessmentPageStore extends BaseComponentStore<TaskInstanceAssessmentPageState> {
    public readonly taskInstance$ = this.select(
        (s) => s.taskInstanceState.taskInstance
    );
    public readonly taskInstanceState$ = this.select(
        (s) => s.taskInstanceState
    );

    constructor() {
        super(initialState);
    }

    readonly patchTaskInstance = this.updaterFor('taskInstanceState');

    readonly loadTaskInstance = this.effect(
        (params$: Observable<LoadTaskInstanceParams>) =>
            params$.pipe(loadTaskInstance((s) => this.patchTaskInstance(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.taskInstance$.pipe(
            ifAttached(this),
            map((taskInstance) => taskInstance?.exercise?.course),
            dispatchCourseChange
        )
    );
}
