import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { Submission } from 'app/assessment/models/submission';
import { SubmissionPageComponent } from 'app/assessment/pages/submission/submission.page.component';
import { TaskInstanceAssessmentPageComponent } from 'app/assessment/pages/task-instance-assessment/task-instance-assessment.page.component';
import { NameFormat } from 'app/shared/user/user';
import { ExerciseAssessmentPageComponent } from 'app/assessment/pages/exercise-assessment/exercise-assessment.page.component';
import { SubmissionDetailLinkDirective } from 'app/assessment/routing/directives/submission-link-directives';

export const routes: Routes = [
    {
        path: 'exercises/:exercise_id/assessment',
        component: ExerciseAssessmentPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'exerciseAssessment',
                label: `Bewertung`,
            },
            courseMenuEnabled: true,
            reuseRoute: true,
        },
    },
    {
        path: 'submissions/:submission_id',
        component: SubmissionPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'submission',
                label: (submission: Submission) =>
                    `${submission.submittedBy.getName(NameFormat.Personal)}`,
            },
            courseMenuEnabled: true,
        },
    },
    {
        path: 'exercises/:exercise_id/tasks/:task_id/assessment',
        component: TaskInstanceAssessmentPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'taskInstanceAssessment',
                label: `Bewertung`,
            },
            courseMenuEnabled: true,
            reuseRoute: true,
        },
    },
];

@NgModule({
    declarations: [SubmissionDetailLinkDirective],
    exports: [RouterModule, SubmissionDetailLinkDirective],
})
export class AssessmentRoutingModule {}
