import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable, of } from 'rxjs';
import { map, switchMap } from 'rxjs/operators';
import { UpdatePassedDto } from 'app/assessment/dto/update-passed.dto';
import { UpdatePointsDto } from 'app/assessment/dto/update-points.dto';
import { Submission } from 'app/assessment/models/submission';
import { SubmissionFilter } from 'app/assessment/dto/submission-filter';

@Injectable({
    providedIn: 'root',
})
export class SubmissionService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<Submission> {
        return this.http
            .get<Submission>(`${environment.backendUrl}/submission/${id}`)
            .pipe(map((solution) => new Submission(solution)));
    }

    getSubmissions(filter: SubmissionFilter): Observable<Submission[]> {
        var params = new HttpParams();
        if (filter.exerciseId != null) {
            params = params.set('exerciseId', filter.exerciseId);
        }

        if (filter.unitId != null) {
            params = params.set('unitId', filter.unitId);
        }

        if (filter.userId != null) {
            params = params.set('userId', filter.userId);
        }

        return this.http
            .get<Submission[]>(`${environment.backendUrl}/submission`, {
                params: params,
            })
            .pipe(
                map((submissions) =>
                    submissions.map((submission) => new Submission(submission))
                )
            );
    }

    getLatestSubmission(filter: SubmissionFilter) {
        return this.getSubmissions(filter).pipe(
            map((submissions) => submissions[0]),
            switchMap((submission) =>
                submission ? this.getById(submission.id) : of(<Submission>null)
            )
        );
    }

    updatePoints(
        submissionId: number,
        dto: UpdatePointsDto
    ): Observable<Submission> {
        return this.http
            .post<Submission>(
                `${environment.backendUrl}/submission/${submissionId}/points`,
                dto
            )
            .pipe(map((submission) => new Submission(submission)));
    }

    updatePassed(
        submissionId: number,
        dto: UpdatePassedDto
    ): Observable<Submission> {
        return this.http
            .post<Submission>(
                `${environment.backendUrl}/submission/${submissionId}/passed`,
                dto
            )
            .pipe(map((submission) => new Submission(submission)));
    }
}
