import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Exercise } from 'app/exercises/models/exercise';
import { Origin } from 'app/assessment/models/origin';
import { SubmissionListStore } from 'app/assessment/shared/components/submission-list/submission-list.store';

@Component({
    selector: 'sb-submission-list',
    templateUrl: './submission-list.component.html',
    styleUrls: ['./submission-list.component.scss'],
    providers: [SubmissionListStore],
})
export class SubmissionListComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() exercise: Exercise;
    @Input() taskId: number;
    @Input() origin: Origin;

    filteredProgresses$ = this.submissionListStore.filteredProgresses$;
    progressesState$ = this.submissionListStore.progressesState$;
    onlyOwnUnits$ = this.submissionListStore.onlyOwnUnits$;

    constructor(private submissionListStore: SubmissionListStore) {
        super();
    }

    ngOnInit(): void {}

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('exercise')) {
            this.submissionListStore.patchState({ exercise: this.exercise });
        }
    }

    onlyOwnUnitsChanged(onlyOwnUnits: boolean) {
        this.submissionListStore.patchState({ onlyOwnUnits: onlyOwnUnits });
    }
}
