import { Injectable } from '@angular/core';
import {
    ActivatedRouteSnapshot,
    CanActivateChild,
    Router,
    RouterStateSnapshot,
    UrlTree,
} from '@angular/router';
import { AuthService } from 'app/auth/auth.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Observable } from 'rxjs';
import { filter, switchMap } from 'rxjs/operators';
import { KeycloakService } from 'keycloak-angular';
import { CourseService } from 'app/courses/services/course.service';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { ChangeCourse } from 'app/courses/store/courses.actions';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';

@Injectable({
    providedIn: 'root',
})
export class AuthGuard implements CanActivateChild {
    initialized$: Observable<boolean> = this.store.select(
        (s) => s.app.initialized
    );
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    constructor(
        private authService: AuthService,
        private router: Router,
        private notificationService: NotificationService,
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private keycloakService: KeycloakService
    ) {}

    canActivateChild(
        next: ActivatedRouteSnapshot,
        state: RouterStateSnapshot
    ):
        | Observable<boolean | UrlTree>
        | Promise<boolean | UrlTree>
        | boolean
        | UrlTree {
        return this.initialized$.pipe(
            filter((initialized) => initialized),
            switchMap((init) => this.principal$),
            switchMap(async (principal) => {
                if (!next.data.courseMenuEnabled) {
                    this.store.dispatch(new ChangeCourse({ course: null }));
                }

                if (next.data.authRequired && !principal) {
                    await this.keycloakService.login({
                        redirectUri: window.location.origin + state.url,
                    });
                    return;
                }

                return true;
            })
        );
    }
}
