import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { KeycloakEventType, KeycloakService } from 'keycloak-angular';
import { from, Observable } from 'rxjs';
import { map, switchMap } from 'rxjs/operators';
import { SubatoPrincipal } from './models/principal/subato-principal';
import { Store } from '@ngrx/store';
import { SubatoState } from '../store/app.reducers';
import { PrincipalChange } from './store/auth.actions';

@Injectable({
    providedIn: 'root',
})
export class AuthService {
    constructor(
        private http: HttpClient,
        private keycloakService: KeycloakService,
        private store: Store<SubatoState>
    ) {
        this.keycloakService.keycloakEvents$.subscribe((evt) => {
            if (evt.type == KeycloakEventType.OnAuthLogout) {
                this.store.dispatch(new PrincipalChange({ principal: null }));
                this.notifyLogout().pipe(
                    map(() => {
                        this.keycloakService.login({
                            redirectUri: window.location.href,
                        });
                    })
                );
            }
        });
    }

    public getPrincipal(): Observable<SubatoPrincipal> {
        return this.http
            .get<SubatoPrincipal>(`${environment.backendUrl}/auth/principal`)
            .pipe(map((principal) => new SubatoPrincipal(principal)));
    }

    public login(): Observable<void> {
        return from(
            this.keycloakService.login({
                redirectUri: window.location.href,
            })
        );
    }

    public logout(): Observable<void> {
        return this.notifyLogout().pipe(
            switchMap(() =>
                from(this.keycloakService.logout(window.location.origin))
            ),
            map(() => {
                this.store.dispatch(new PrincipalChange({ principal: null }));
            })
        );
    }

    notifyLogout(): Observable<void> {
        return this.http.post<void>(
            `${environment.backendUrl}/auth/logout`,
            {}
        );
    }
}
