import { Role } from '../role';
import { CourseStaffMembership } from './course-membership';
import { CourseUnit } from './course-unit';
import { DegreeProgram } from './degree-program';
import { CourseParticipantProfile } from 'app/auth/models/principal/course-participant-profile';

export class SubatoPrincipal {
    id: number;
    loginName: string;
    firstName: string;
    lastName: string;
    name: string;
    mail: string;
    consentedToCollection: boolean;
    program: DegreeProgram;
    staffMemberships: CourseStaffMembership[];
    participatingUnits: CourseUnit[];
    participantProfiles: CourseParticipantProfile[];
    roles: Role[];

    constructor(principal: any) {
        this.id = principal.id;
        this.loginName = principal.loginName;
        this.firstName = principal.firstName;
        this.lastName = principal.lastName;
        this.consentedToCollection = principal.consentedToCollection;
        this.name = principal.name;
        this.mail = principal.mail;
        this.program = principal.program;
        this.staffMemberships = principal.staffMemberships;
        this.roles = principal.roles;
        this.participatingUnits = principal.participatingUnits;
        this.participantProfiles = principal.participantProfiles;
    }

    isAdmin(courseId: number): boolean {
        var mi = this.staffMemberships.find(
            (m) => m.courseId === courseId && m.group == 0
        );
        return mi != null || this.hasSuperAdminRole();
    }

    isLecturer(courseId: number): boolean {
        var mi = this.staffMemberships.find(
            (m) => m.courseId === courseId && m.group == 1
        );
        return mi != null || this.hasSuperAdminRole();
    }

    isTutor(courseId: number): boolean {
        var mi = this.staffMemberships.find(
            (m) => m.courseId === courseId && m.group == 2
        );
        return mi != null || this.hasSuperAdminRole();
    }

    isLecturerOrAdmin(courseId: number): boolean {
        return this.isAdmin(courseId) || this.isLecturer(courseId);
    }

    hasAnyGroup(courseId: number): boolean {
        return (
            this.isAdmin(courseId) ||
            this.isLecturer(courseId) ||
            this.isTutor(courseId)
        );
    }

    isParticipant(courseId: number): boolean {
        return this.getUnitForCourse(courseId) != null;
    }

    getProfile(courseId: number): CourseParticipantProfile | null {
        return this.participantProfiles.find((p) => p.courseId == courseId);
    }

    isParticipantOfUnit(unitId: number): boolean {
        return this.participatingUnits.find((u) => u.id === unitId) != null;
    }

    getUnitForCourse(courseId: number): CourseUnit {
        var mi = this.participatingUnits.find((c) => c.courseId === courseId);
        return mi;
    }

    hasSuperAdminRole(): boolean {
        return this.hasRole(Role.SuperAdmin);
    }

    hasStaffRole(): boolean {
        return this.hasRole(Role.Staff) || this.hasSuperAdminRole();
    }

    hasStudentRole(): boolean {
        return this.hasRole(Role.Student) || this.hasSuperAdminRole();
    }

    private hasRole(role: Role): boolean {
        return this.roles.find((r) => r == role) != null;
    }

    public inProgram(program: DegreeProgram) {
        return (
            this.program != null &&
            program != null &&
            this.program.id == program.id
        );
    }
}
