import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';
import { catchError, map, switchMap } from 'rxjs/operators';
import { AuthActions, LoadPrincipal, PrincipalChange } from './auth.actions';
import { AuthService } from 'app/auth/auth.service';
import { from, of } from 'rxjs';
import { KeycloakService } from 'keycloak-angular';

@Injectable()
export class AuthEffects {
    loadPrincipal$ = createEffect(() => {
        return this.actions$.pipe(
            ofType(AuthActions.LoadPrincipal),
            switchMap((a: LoadPrincipal) => {
                return from(this.keycloakService.getToken()).pipe(
                    switchMap((token) => {
                        if (!token) {
                            throw 'Not signed in';
                        }
                        return this.authService.getPrincipal();
                    }),
                    catchError(() => of(null))
                );
            }),
            map((principal) => {
                return new PrincipalChange({ principal: principal });
            })
        );
    });

    constructor(
        private actions$: Actions,
        private keycloakService: KeycloakService,
        private authService: AuthService
    ) {}
}
