import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { CourseUnit } from './models/course-unit';
import { CourseUnitsPageComponent } from 'app/courses/pages/course-units/course-units.page.component';
import { CoursePageComponent } from 'app/courses/pages/course/course.page.component';
import { CourseFilesPageComponent } from 'app/courses/pages/course-files/course-files.page.component';
import { CoursesPageComponent } from 'app/courses/pages/courses/courses.page.component';
import { CoursePermissionsPageComponent } from 'app/courses/pages/course-permissions/course-permissions.page.component';
import { ParticipantProgressPageComponent } from 'app/courses/pages/participant-progress/participant-progress.page.component';
import { CourseSettingsPageComponent } from 'app/courses/pages/course-settings/course-settings.page.component';
import { CourseParticipantsPageComponent } from 'app/courses/pages/course-participants/course-participants.page.component';
import { CourseUnitAdminPageComponent } from 'app/courses/pages/course-unit-admin/course-unit-admin.page.component';
import { NewCoursePageComponent } from 'app/courses/pages/new-course/new-course.page.component';
import { CourseUnitPageComponent } from 'app/courses/pages/course-unit/course-unit.page.component';

export const routes: Routes = [
    {
        path: 'course',
        component: CoursesPageComponent,
        data: {
            reuseRoute: true,
        },
    },
    {
        path: 'course/new',
        component: NewCoursePageComponent,
    },
    {
        path: 'course/:course_id',
        component: CoursePageComponent,
        data: {
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/permissions',
        component: CoursePermissionsPageComponent,
        data: {
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/settings',
        component: CourseSettingsPageComponent,
        data: {
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/participants',
        component: CourseParticipantsPageComponent,
        data: {
            courseMenuEnabled: true,
            reuseRoute: true,
        },
    },
    {
        path: 'course/:course_id/files',
        component: CourseFilesPageComponent,
        data: {
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/unit',
        component: CourseUnitsPageComponent,
        data: {
            breadcrumb: {
                alias: 'courseUnits',
                label: 'Veranstaltungen',
            },
            courseMenuEnabled: true,
            reuseRoute: true,
        },
    },
    {
        path: 'unit/:unit_id/admin',
        component: CourseUnitAdminPageComponent,
        data: {
            breadcrumb: {
                alias: 'courseUnitAdmin',
                label: 'Administration',
            },
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/participants/:user_id/progress',
        component: ParticipantProgressPageComponent,
        data: {
            reuseRoute: true,
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/myprogress',
        component: ParticipantProgressPageComponent,
        data: {
            reuseRoute: true,
            courseMenuEnabled: true,
        },
    },
    {
        path: 'unit/:unit_id',
        component: CourseUnitPageComponent,
        data: {
            reuseRoute: true,
            breadcrumb: {
                alias: 'courseUnit',
                label: (unit: CourseUnit) => unit.shortName,
            },
            courseMenuEnabled: true,
        },
    },
];

@NgModule({
    exports: [RouterModule],
})
export class CoursesRoutingModule {}
