import { User } from 'app/shared/user/user';
import { DegreeProgram } from 'app/modules/models/degree-program';
import { ProgramModule } from 'app/modules/models/program-module';
import { Term } from 'app/modules/models/term';
import { CourseVisibility } from './course-visibility';
import { SubmitPolicy } from './submit-policy';

export class Course {
    id: number;
    name: string;
    shortName: string;
    motd: string;
    visibility: CourseVisibility;
    submitPolicy: SubmitPolicy;
    lecturers: User[];
    admins: User[];
    tutors: User[];
    modules: ProgramModule[];
    term: Term;
    showEmails: boolean;
    allowJoiningUnits: boolean;
    disableEditor: boolean;
    finished: boolean;

    constructor(course: any) {
        this.id = course.id;
        this.name = course.name;
        this.shortName = course.shortName;
        this.motd = course.motd;
        this.showEmails = course.showEmails;
        this.allowJoiningUnits = course.allowJoiningUnits;
        this.disableEditor = course.disableEditor;
        this.visibility = course.visibility;
        this.submitPolicy = course.submitPolicy;
        this.finished = course.finished;
        this.modules = course.modules
            ? course.modules.map((mod) => new ProgramModule(mod))
            : [];
        this.lecturers = course.lecturers
            ? course.lecturers.map((lecturer) => new User(lecturer))
            : [];
        this.admins = course.admins
            ? course.admins.map((admin) => new User(admin))
            : [];
        this.tutors = course.tutors
            ? course.tutors.map((tutor) => new User(tutor))
            : [];
        this.term = course.term ? new Term(course.term) : null;
    }

    get adminNames(): string {
        return this.admins.map((u) => u.name).join(', ');
    }

    moduleForProgram(program: DegreeProgram) {
        return this.modules.find((m) => m.program.id == program.id);
    }

    get public() {
        return this.visibility == CourseVisibility.Public;
    }

    get internal() {
        return this.visibility == CourseVisibility.Internal;
    }

    get participantsOnly() {
        return this.visibility == CourseVisibility.Participants;
    }

    get private() {
        return this.visibility == CourseVisibility.Private;
    }

    isAdmin(user: User) {
        return this.admins.map((u) => u.id).indexOf(user.id) != -1;
    }

    isTutor(user: User) {
        return this.tutors.map((u) => u.id).indexOf(user.id) != -1;
    }

    isLecturer(user: User) {
        return this.lecturers.map((u) => u.id).indexOf(user.id) != -1;
    }
}
