import { Exercise } from 'app/exercises/models/exercise';
import { Submission } from 'app/assessment/models/submission';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Solution } from 'app/solutions/models/solution';

export class ExerciseResult {
    exercise: Exercise;
    taskInstances: TaskInstance[];
    testsPassed: number;
    testsTotal: number;
    styleErrorsTotal: number;
    leaksTotal: number;
    submission: Submission;
    solutions: Solution[];

    constructor(dto: any) {
        this.testsPassed = dto.testsPassed;
        this.testsTotal = dto.testsTotal;
        this.styleErrorsTotal = dto.styleErrorsTotal;
        this.leaksTotal = dto.leaksTotal;
        this.exercise = new Exercise(dto.exercise);
        this.taskInstances = dto.taskInstances.map(
            (ti) => new TaskInstance(ti)
        );
        this.submission = dto.submission
            ? new Submission(dto.submission)
            : null;
        this.solutions = dto.solutions.map((r) => new Solution(r));
    }

    solutionCompiles(taskInstance: TaskInstance) {
        return (
            this.hasSolution(taskInstance) &&
            this.getSolution(taskInstance).hasResult &&
            !this.getSolution(taskInstance).evalFailed &&
            this.getSolution(taskInstance).evalState.result.compilation &&
            this.getSolution(taskInstance).evalState.result.compilation
                .compiling
        );
    }

    solutionDoesNotCompile(taskInstance: TaskInstance) {
        return (
            this.hasSolution(taskInstance) &&
            this.getSolution(taskInstance).hasResult &&
            !this.getSolution(taskInstance).evalFailed &&
            this.getSolution(taskInstance).evalState.result.compilation &&
            !this.getSolution(taskInstance).evalState.result.compilation
                .compiling
        );
    }

    solutionPresentWithResult(taskInstance: TaskInstance) {
        return (
            this.hasSolution(taskInstance) &&
            this.getSolution(taskInstance).hasResult &&
            !this.getSolution(taskInstance).evalFailed
        );
    }

    exercisePassed() {
        return (
            this.exercise.usingBinary &&
            this.submission &&
            this.submission.passed == true
        );
    }

    exerciseNotPassed() {
        return (
            this.exercise.usingBinary &&
            this.submission &&
            this.submission.passed == false
        );
    }

    exerciseNoDecision() {
        return (
            this.exercise.usingBinary &&
            this.submission &&
            this.submission.passed == null
        );
    }

    hasSolution(taskInstance: TaskInstance) {
        return this.getSolution(taskInstance) != null;
    }

    getSolution(taskInstance: TaskInstance) {
        return this.solutions.find((s) => s.task.id == taskInstance.task.id);
    }

    hasPointRating(taskInstance: TaskInstance) {
        return this.getPointRating(taskInstance) != null;
    }

    getPointRating(taskInstance: TaskInstance) {
        return this.submission?.pointRatings.find(
            (r) => r.taskInstance.task.id == taskInstance.task.id
        );
    }
}
