import { Component, Input, OnInit } from '@angular/core';
import { NbDialogService } from '@nebular/theme';
import { AuthService } from 'app/auth/auth.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ConfirmationDialogComponent } from 'app/shared/dialogs/confirmation-dialog/confirmation-dialog.component';
import { filter, takeUntil } from 'rxjs/operators';
import { CourseFile } from '../../../../models/course-file';
import { CourseFileService } from '../../../../services/course-file.service';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { CourseFilesPageStore } from 'app/courses/pages/course-files/course-files.page.store';

@Component({
    selector: 'sb-cfs-course-file-list',
    templateUrl: './course-file-list.component.html',
    styleUrls: ['./course-file-list.component.scss'],
})
export class CourseFileListComponent extends BaseComponent implements OnInit {
    @Input() showDelete: boolean;

    filesState$ = this.courseFilesPageStore.filesState$;
    principal$ = this.store.select((s) => s.auth.principal);
    deletingFile$ = this.courseFilesPageStore.deletingFile$;

    constructor(
        private courseFileService: CourseFileService,
        private courseFilesPageStore: CourseFilesPageStore,
        private authService: AuthService,
        private store: Store<SubatoState>,
        private dialogService: NbDialogService
    ) {
        super();
    }

    ngOnInit(): void {}

    getDownloadLink(file: CourseFile) {
        return this.courseFileService.getFileUrl(file);
    }

    deleteFile(file: CourseFile) {
        this.dialogService
            .open(ConfirmationDialogComponent, {
                context: {
                    title: `Löschung von Datei: ${file.name}`,
                    text: `Bist du wirklich sicher, dass du die Datei '${file.name}' löschen willst?`,
                    actionText: 'Löschen',
                },
            })
            .onClose.pipe(
                takeUntil(this.destroy$),
                filter((result) => result && result.confirmed)
            )
            .subscribe(() => this.courseFilesPageStore.deleteFile(file));
    }
}
