import { Component, OnInit } from '@angular/core';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CourseFilesPageStore } from 'app/courses/pages/course-files/course-files.page.store';

@Component({
    selector: 'sb-cfs-upload',
    templateUrl: './upload.component.html',
    styleUrls: ['./upload.component.scss'],
})
export class UploadComponent extends BaseComponent implements OnInit {
    course$ = this.courseFileListStore.course$;
    uploading$ = this.courseFileListStore.uploading$;
    form: UntypedFormGroup;

    constructor(
        private courseFileListStore: CourseFilesPageStore,
        private router: Router,
        private route: ActivatedRoute,
        private globalErrorHandler: GlobalErrorHandler,
        private store: Store<SubatoState>,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    ngOnInit(): void {
        this.createForm();
    }

    createForm() {
        this.form = this.fb.group({
            file: ['', Validators.required],
            description: [''],
        });
    }

    save() {
        this.courseFileListStore.uploadFile({
            description: this.form.get('description').value,
            file: this.form.get('file').value,
        });
    }
}
