import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { AuthService } from 'app/auth/auth.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EMPTY } from 'rxjs';
import { CourseFilesPageStore } from 'app/courses/pages/course-files/course-files.page.store';

@Component({
    templateUrl: './course-files.page.component.html',
    styleUrls: ['./course-files.page.component.scss'],
    providers: [CourseFilesPageStore],
})
export class CourseFilesPageComponent extends BaseComponent implements OnInit {
    courseState$ = this.courseFilesStore.courseState$;
    isAdmin$ = this.courseFilesStore.isAdmin$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    constructor(
        private courseService: CourseService,
        private courseFilesStore: CourseFilesPageStore,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private store: Store<SubatoState>,
        private authService: AuthService,
        private dialogService: NbDialogService,
        private notificationService: NotificationService,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseFilesStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.courseFilesStore.loadCourse(courseId)
            );
    }
}
