import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/auth/auth.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EMPTY } from 'rxjs';
import { CourseParticipantsPageStore } from 'app/courses/pages/course-participants/course-participants.page.store';

@Component({
    templateUrl: './course-participants.page.component.html',
    styleUrls: ['./course-participants.page.component.scss'],
    providers: [CourseParticipantsPageStore],
})
export class CourseParticipantsPageComponent
    extends BaseComponent
    implements OnInit
{
    courseState$ = this.courseParticipantsPageStore.courseState$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    constructor(
        private courseService: CourseService,
        private store: Store<SubatoState>,
        private courseParticipantsPageStore: CourseParticipantsPageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private authService: AuthService,
        private route: ActivatedRoute
    ) {
        super();
    }

    onAttach() {
        super.onAttach();
        this.courseParticipantsPageStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.courseParticipantsPageStore.onDetach();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseParticipantsPageStore.patchCourse({
                        error: err,
                    });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.courseParticipantsPageStore.loadCourse(courseId)
            );
    }
}
