import { Component, OnInit } from '@angular/core';
import { FormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { NbDialogRef } from '@nebular/theme';
import { User } from 'app/shared/user/user';
import { environment } from 'environments/environment';
import { StaffGroup } from '../../../../models/staff-group';
import { EnumHelper } from 'app/shared/misc/helper/enum.helper';
import { DocsService } from 'app/shared/misc/docs.service';

@Component({
    templateUrl: './add-staff-dialog.component.html',
    styleUrls: ['./add-staff-dialog.component.scss'],
})
export class AddStaffDialogComponent implements OnInit {
    membershipForm: UntypedFormGroup;
    user: User;
    docsUrls = this.docsService.urls;
    staffGroups = EnumHelper.getAll(StaffGroup);

    constructor(
        protected ref: NbDialogRef<AddStaffDialogComponent>,
        private fb: FormBuilder,
        private docsService: DocsService
    ) {}

    createForm() {
        this.membershipForm = this.fb.group({
            user: [null, [Validators.required]],
            group: [null, [Validators.required]],
        });
    }

    ngOnInit(): void {
        this.createForm();
    }

    onUserSelected(user: User) {
        this.user = user;
        this.membershipForm.patchValue({
            user: user,
        });
    }

    cancel() {
        this.ref.close();
    }

    add() {
        this.ref.close({
            user: this.membershipForm.value.user,
            group: this.membershipForm.value.group,
        });
    }
}
