import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { filter, takeUntil } from 'rxjs/operators';
import { NbDialogService } from '@nebular/theme';
import { CourseStaffMembership } from '../../../../models/course-staff-membership';
import { CourseService } from '../../../../services/course.service';
import { StaffGroup } from 'app/courses/models/staff-group';
import { AddStaffDialogComponent } from 'app/courses/pages/course-permissions/components/add-staff-dialog/add-staff-dialog.component';
import { EnumHelper } from 'app/shared/misc/helper/enum.helper';
import { CoursePermissionsPageStore } from 'app/courses/pages/course-permissions/course-permissions.page.store';

@Component({
    selector: 'sb-course-permissions-staff-form',
    templateUrl: './staff-form.component.html',
    styleUrls: ['./staff-form.component.scss'],
})
export class StaffFormComponent extends BaseComponent implements OnInit {
    course$ = this.coursePermissionsStore.course$;
    staffMembershipState$ = this.coursePermissionsStore.staffMembershipState$;

    staffGroups = EnumHelper.getAll(StaffGroup);

    constructor(
        private courseService: CourseService,
        private coursePermissionsStore: CoursePermissionsPageStore,
        private router: Router,
        private dialogService: NbDialogService
    ) {
        super();
    }

    ngOnInit(): void {
        this.coursePermissionsStore.loadStaff();
    }

    remove(membership: CourseStaffMembership) {
        this.coursePermissionsStore.removeStaff(membership);
    }

    add() {
        this.dialogService
            .open(AddStaffDialogComponent, {
                context: {},
            })
            .onClose.pipe(
                takeUntil(this.destroy$),
                filter(
                    (result) => result && result.user && result.group != null
                )
            )
            .subscribe((result) => {
                this.coursePermissionsStore.addStaff({
                    userId: result.user.id,
                    group: result.group,
                });
            });
    }
}
