import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/auth/auth.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { environment } from 'environments/environment';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CoursePermissionsPageStore } from 'app/courses/pages/course-permissions/course-permissions.page.store';
import { CourseStaffMembershipStore } from 'app/courses/pages/course-permissions/course-staff-membership.store';
import { EMPTY } from 'rxjs';
import { DocsService } from 'app/shared/misc/docs.service';

@Component({
    templateUrl: './course-permissions.page.component.html',
    styleUrls: ['./course-permissions.page.component.scss'],
    providers: [CourseStaffMembershipStore, CoursePermissionsPageStore],
})
export class CoursePermissionsPageComponent
    extends BaseComponent
    implements OnInit
{
    docsUrls = this.docsService.urls;
    courseState$ = this.coursePermissionsStore.courseState$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    constructor(
        private store: Store<SubatoState>,
        private coursePermissionsStore: CoursePermissionsPageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private authService: AuthService,
        private route: ActivatedRoute,
        private docsService: DocsService
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.coursePermissionsStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.coursePermissionsStore.loadCourse(courseId)
            );
    }
}
