import { Injectable } from '@angular/core';
import { filter, tap, withLatestFrom } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseStaffMembership } from 'app/courses/models/course-staff-membership';
import { StaffGroup } from 'app/courses/models/staff-group';
import { CourseStaffMembershipStore } from 'app/courses/pages/course-permissions/course-staff-membership.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';

interface CoursePermissionsPageState {
    courseState: CourseState;
}

export const initialState: CoursePermissionsPageState = {
    courseState: { course: null, error: null, loading: true },
};

@Injectable()
export class CoursePermissionsPageStore extends BaseComponentStore<CoursePermissionsPageState> {
    public readonly course$ = this.select((s) => s.courseState.course);
    public readonly courseState$ = this.select((s) => s.courseState);
    public readonly staffMembershipState$ =
        this.courseStaffMembershipStore.select((s) => s);

    constructor(
        private store: Store<SubatoState>,
        private courseStaffMembershipStore: CourseStaffMembershipStore
    ) {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly loadStaff = this.effect(() =>
        this.course$.pipe(
            filter((course) => course != null),
            tap((course) => this.courseStaffMembershipStore.loadStaff(course))
        )
    );

    readonly removeStaff = this.effect(
        (membership: Observable<CourseStaffMembership>) =>
            membership.pipe(
                withLatestFrom(this.course$),
                tap(([membership, course]) =>
                    this.courseStaffMembershipStore.removeStaff(membership)
                )
            )
    );

    readonly addStaff = this.effect(
        (params: Observable<{ userId: number; group: StaffGroup }>) =>
            params.pipe(
                withLatestFrom(this.course$),
                tap(([params, course]) =>
                    this.courseStaffMembershipStore.addStaff({
                        courseId: course.id,
                        userId: params.userId,
                        group: params.group,
                    })
                )
            )
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.course$.pipe(dispatchCourseChange)
    );
}
