import { Component, Input, OnInit } from '@angular/core';
import { Course } from '../../../../models/course';
import { CourseSettingsPageStore } from 'app/courses/pages/course-settings/course-settings.page.store';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ConfirmationDialogComponent } from 'app/shared/dialogs/confirmation-dialog/confirmation-dialog.component';
import { filter, takeUntil } from 'rxjs/operators';
import { NbDialogService } from '@nebular/theme';

@Component({
    selector: 'sb-course-settings-modules',
    templateUrl: './modules.component.html',
    styleUrls: ['./modules.component.scss'],
})
export class ModulesComponent extends BaseComponent implements OnInit {
    @Input() course: Course;
    canReimport$ = this.courseSettingsStore.canReimport$;
    reimportingCourse$ = this.courseSettingsStore.reimportingCourse$;

    constructor(
        private courseSettingsStore: CourseSettingsPageStore,
        private dialogService: NbDialogService
    ) {
        super();
    }

    ngOnInit(): void {}

    reimport() {
        this.dialogService
            .open(ConfirmationDialogComponent, {
                context: {
                    title: `Reimport von Kurs ${this.course.name}`,
                    text: `Bist du wirklich sicher, dass du den Import für den Kurs '${this.course.name}' erneut durchführen willst? Dabei gehen erfasste Anwesenheiten und manuelle Veranstaltungsänderungen (z.B. Teilnehmeränderungen) verloren.`,
                    actionText: 'Kurs zurücksetzen',
                },
            })
            .onClose.pipe(
                takeUntil(this.destroy$),
                filter((result) => result && result.confirmed)
            )
            .subscribe(() => this.courseSettingsStore.reimportCourse());
    }
}
