import { Component, OnInit } from '@angular/core';
import { UntypedFormBuilder } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { AuthService } from 'app/auth/auth.service';
import { FormComponent } from 'app/shared/validation/form.component';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CourseSettingsPageStore } from 'app/courses/pages/course-settings/course-settings.page.store';
import { EMPTY } from 'rxjs';

@Component({
    templateUrl: './course-settings.page.component.html',
    styleUrls: ['./course-settings.page.component.scss'],
    providers: [CourseSettingsPageStore],
})
export class CourseSettingsPageComponent
    extends FormComponent
    implements OnInit
{
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    courseState$ = this.courseSettingsStore.courseState$;
    descriptionState$ = this.courseSettingsStore.descriptionState$;

    constructor(
        private courseService: CourseService,
        private courseSettingsStore: CourseSettingsPageStore,
        private store: Store<SubatoState>,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private authService: AuthService,
        private dialogService: NbDialogService,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseSettingsStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.courseSettingsStore.loadCourse(courseId)
            );
    }
}
