import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import { CourseUnitEvent } from '../../../../models/course-unit-event';
import { BaseComponent } from 'app/shared/misc/base.component';
import { CourseUnitService } from '../../../../services/course-unit.service';
import { CourseUnitEventsStore } from 'app/courses/pages/course-unit-admin/components/course-unit-event-list/course-unit-events.store';
import { CourseUnit } from 'app/courses/models/course-unit';

@Component({
    selector: 'sb-course-unit-admin-course-unit-event-list',
    templateUrl: './course-unit-event-list.component.html',
    styleUrls: ['./course-unit-event-list.component.scss'],
    providers: [CourseUnitEventsStore],
})
export class CourseUnitEventListComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() unit: CourseUnit;
    @Input() changesAllowed: boolean;

    error$ = this.courseUnitEventsStore.error$;
    loading$ = this.courseUnitEventsStore.loading$;
    events$ = this.courseUnitEventsStore.events$;
    processingEvent$ = this.courseUnitEventsStore.processingEvent$;

    constructor(
        private courseUnitService: CourseUnitService,
        private courseUnitEventsStore: CourseUnitEventsStore
    ) {
        super();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('unit')) {
            this.courseUnitEventsStore.setUnit(this.unit);
        }
    }

    ngOnInit(): void {}

    isCancelled(event: CourseUnitEvent): boolean {
        return event.holiday;
    }

    updateCancellation(event: CourseUnitEvent, cancel: boolean) {
        this.courseUnitEventsStore.updateCancellation({
            event: event,
            cancel: cancel,
        });
    }
}
