import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import { filter, map, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/misc/base.component';
import { User } from 'app/shared/user/user';
import { AddParticipantDialogComponent } from 'app/courses/pages/course-unit-admin/components/add-participant-dialog/add-participant-dialog.component';
import { NbDialogService } from '@nebular/theme';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseUnit } from 'app/courses/models/course-unit';
import { ParticipantListStore } from 'app/courses/pages/course-unit-admin/components/participant-list/participant-list.store';

@Component({
    selector: 'sb-course-unit-admin-participant-list',
    templateUrl: './participant-list.component.html',
    styleUrls: ['./participant-list.component.scss'],
    providers: [ParticipantListStore],
})
export class ParticipantListComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() unit: CourseUnit;
    @Input() addEnabled: boolean;
    @Input() removeEnabled: boolean;

    participantsState$ = this.courseUnitParticipantListStore.participantsState$;
    query$ = this.courseUnitParticipantListStore.query$;
    removingParticipant$ =
        this.courseUnitParticipantListStore.removingParticipant$;
    addingParticipant$ = this.courseUnitParticipantListStore.addingParticipant$;

    constructor(
        private courseUnitParticipantListStore: ParticipantListStore,
        private dialogService: NbDialogService,
        private store: Store<SubatoState>
    ) {
        super();
    }

    ngOnInit(): void {}

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('unit')) {
            this.courseUnitParticipantListStore.setUnit(this.unit);
        }
    }

    removeParticipant(participant: User) {
        this.courseUnitParticipantListStore.removeParticipant(participant);
    }

    addParticipant() {
        this.dialogService
            .open(AddParticipantDialogComponent, {
                context: {},
            })
            .onClose.pipe(
                takeUntil(this.destroy$),
                filter((result) => result && result.user),
                map((result) => result.user)
            )
            .subscribe((user) =>
                this.courseUnitParticipantListStore.addParticipant(user)
            );
    }

    onChange(query) {
        this.courseUnitParticipantListStore.patchState({ query: query });
    }

    cancel() {}
}
