import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EMPTY } from 'rxjs';
import { CourseUnitAdminPageStore } from 'app/courses/pages/course-unit-admin/course-unit-admin.page.store';

@Component({
    templateUrl: './course-unit-admin.page.component.html',
    styleUrls: ['./course-unit-admin.page.component.scss'],
    providers: [CourseUnitAdminPageStore],
})
export class CourseUnitAdminPageComponent
    extends BaseComponent
    implements OnInit
{
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Veranstaltung konnten wir leider nicht finden.',
        },
    };

    unitState$ = this.courseUnitAdminPageStore.unitState$;
    isLecturerOrAdmin$ = this.courseUnitAdminPageStore.isLecturerOrAdmin$;

    constructor(
        private courseUnitAdminPageStore: CourseUnitAdminPageStore,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) => ParamHelper.parseIdOrThrow(params, 'unit_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseUnitAdminPageStore.patchUnit({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((unitId) =>
                this.courseUnitAdminPageStore.loadUnit(unitId)
            );
    }
}
