import { Injectable } from '@angular/core';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import {
    CourseUnitState,
    loadUnit,
} from 'app/courses/shared/operators/course-unit.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';

export interface CourseUnitAdminPageState {
    unitState: CourseUnitState;
}

export const initialState: CourseUnitAdminPageState = {
    unitState: { unit: null, error: null, loading: true },
};

@Injectable()
export class CourseUnitAdminPageStore extends BaseComponentStore<CourseUnitAdminPageState> {
    public readonly unit$ = this.select((state) => state.unitState.unit);
    public readonly unitState$ = this.select((state) => state.unitState);

    principal$ = this.store.select((s) => s.auth.principal);

    public readonly isLecturerOrAdmin$ = this.select(
        this.unit$,
        this.principal$,
        (unit, principal) =>
            unit != null &&
            principal != null &&
            principal.isLecturerOrAdmin(unit.course.id)
    );

    constructor(private store: Store<SubatoState>) {
        super(initialState);
    }

    readonly patchUnit = this.updaterFor('unitState');

    readonly loadUnit = this.effect((unitId: Observable<number>) =>
        unitId.pipe(loadUnit((s) => this.patchUnit(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.unit$.pipe(
            map((unit) => unit?.course),
            dispatchCourseChange
        )
    );
}
