import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import { User } from 'app/shared/user/user';
import { CourseUnit } from 'app/courses/models/course-unit';
import { BaseComponent } from 'app/shared/misc/base.component';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { takeUntil } from 'rxjs/operators';
import { ParticipantListStore } from 'app/courses/pages/course-unit/components/participant-list/participant-list.store';

@Component({
    selector: 'sb-course-unit-participant-list',
    templateUrl: './participant-list.component.html',
    styleUrls: ['./participant-list.component.scss'],
    providers: [ParticipantListStore],
})
export class ParticipantListComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() courseUnit: CourseUnit;
    @Output() participantSelected: EventEmitter<User> =
        new EventEmitter<User>();

    participantsState$ =
        this.courseUnitParticipantSelectionStore.participantsState$;
    selectedUser$ = this.courseUnitParticipantSelectionStore.selectedUser$;
    query$ = this.courseUnitParticipantSelectionStore.query$;

    constructor(
        private courseUnitParticipantSelectionStore: ParticipantListStore,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('courseUnit')) {
            this.courseUnitParticipantSelectionStore.setUnit(this.courseUnit);
        }
    }

    ngOnInit(): void {
        this.selectedUser$
            .pipe(takeUntil(this.destroy$))
            .subscribe((u) => this.participantSelected.emit(u));
    }

    onChange(query) {
        this.courseUnitParticipantSelectionStore.patchState({ query: query });
    }

    selectUser(user: User) {
        this.courseUnitParticipantSelectionStore.patchState({
            selectedUser: user,
        });
    }
}
