import { Injectable } from '@angular/core';
import { filter, map, tap } from 'rxjs/operators';
import { CourseUnit } from 'app/courses/models/course-unit';
import {
    CourseUnitParticipantsState,
    loadParticipants,
} from 'app/courses/shared/operators/course-unit-participant.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { User } from 'app/shared/user/user';
import { combineLatest } from 'rxjs';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';

interface ParticipantListState {
    unit: CourseUnit;
    participantsState: CourseUnitParticipantsState;
    query: string;
    selectedUser: User;
}

export const initialState: ParticipantListState = {
    unit: null,
    participantsState: { participants: [], error: null, loading: true },
    query: '',
    selectedUser: null,
};

@Injectable()
export class ParticipantListStore extends BaseComponentStore<ParticipantListState> {
    public readonly unit$ = this.select((state) => state.unit);
    public readonly participants$ = this.select(
        (state) => state.participantsState.participants
    );
    public readonly participantsState$ = this.select(
        (state) => state.participantsState
    );

    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );
    public readonly selectedUser$ = this.select((state) => state.selectedUser);

    constructor() {
        super(initialState);
    }

    readonly patchParticipants = this.updaterFor('participantsState');

    readonly setUnit = this.updater((state, unit: CourseUnit) => ({
        ...state,
        unit: unit,
    }));

    readonly loadParticipants$ = this.effect(() =>
        combineLatest([this.unit$, this.debouncedQuery$]).pipe(
            filter(([unit]) => unit != null),
            map(([unit, query]) => ({ unitId: unit.id, query: query })),
            loadParticipants((s) => this.patchParticipants(s))
        )
    );

    readonly setSelectedUser$ = this.effect(() =>
        this.participants$.pipe(
            tap((participants) =>
                this.patchState({ selectedUser: participants[0] ?? null })
            )
        )
    );
}
