import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Course } from 'app/courses/models/course';
import {
    ExercisesState,
    loadExercises,
} from 'app/exercises/shared/operators/exercise.operators';

interface LastExercisesListState {
    exercisesState: ExercisesState;
    course: Course;
}

export const initialState: LastExercisesListState = {
    exercisesState: { exercises: [], error: null, loading: true },
    course: null,
};

@Injectable()
export class LastExercisesListStore extends BaseComponentStore<LastExercisesListState> {
    public readonly course$ = this.select((state) => state.course);
    public readonly exercisesState$ = this.select(
        (state) => state.exercisesState
    );
    public readonly exercises$ = this.select(
        (state) => state.exercisesState.exercises
    );

    public filteredExercises$ = this.select(this.exercises$, (exercises) => {
        let visibleExercises = exercises.filter((e) => e.visible);
        visibleExercises.sort(
            (a, b) => b.orderNumber - a.orderNumber || b.id - a.id
        );
        return visibleExercises.slice(0, Math.min(3, visibleExercises.length));
    });

    constructor() {
        super(initialState);
    }

    readonly patchExercises = this.updaterFor('exercisesState');

    readonly loadExercises$ = this.effect(() =>
        this.course$.pipe(
            filter((course) => course != null),
            map((course) => ({ courseId: course.id, query: null })),
            loadExercises((s) => this.patchExercises(s))
        )
    );
}
