import { Component, Input, OnInit } from '@angular/core';
import { User } from 'app/shared/user/user';
import { DistinctHelper } from 'app/shared/misc/helper/distinct.helper';
import { Course } from '../../../../models/course';
import { StaffGroup } from '../../../../models/staff-group';

interface StaffMember {
    user: User;
    groups: StaffGroup[];
}

@Component({
    selector: 'sb-course-staff',
    templateUrl: './staff.component.html',
    styleUrls: ['./staff.component.scss'],
})
export class StaffComponent implements OnInit {
    @Input() course: Course;
    @Input() compactMode: boolean;

    members: StaffMember[];

    constructor() {}

    ngOnInit(): void {
        this.computeMembers();
    }

    computeMembers() {
        let users = DistinctHelper.distinct(
            this.course.admins
                .concat(this.course.lecturers)
                .concat(this.course.tutors),
            (u) => u.id
        );

        this.members = users.map((u) => ({
            user: u,
            groups: this.getGroups(u),
        }));
    }

    getGroups(user: User) {
        let groups: StaffGroup[] = [];

        if (this.course.isAdmin(user)) {
            groups.push(StaffGroup.Admin);
        }
        if (this.course.isLecturer(user)) {
            groups.push(StaffGroup.Lecturer);
        }
        if (this.course.isTutor(user)) {
            groups.push(StaffGroup.Tutor);
        }

        return groups;
    }
}
