import { Component, Input, OnInit } from '@angular/core';
import { Course } from '../../../../models/course';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { BaseComponent } from 'app/shared/misc/base.component';
import { map, takeUntil } from 'rxjs/operators';

@Component({
    selector: 'sb-course-visibility-hint',
    templateUrl: './visibility-hint.component.html',
    styleUrls: ['./visibility-hint.component.scss'],
})
export class VisibilityHintComponent extends BaseComponent implements OnInit {
    @Input() course: Course;
    principal$ = this.store.select((s) => s.auth.principal);
    isVisible$: Observable<boolean>;

    constructor(private store: Store<SubatoState>) {
        super();
    }

    ngOnInit(): void {
        this.isVisible$ = this.principal$.pipe(
            takeUntil(this.destroy$),
            map((principal) => {
                let isAdmin = principal?.isAdmin(this.course.id) ?? false;
                return !this.course.public && isAdmin;
            })
        );
    }
}
