import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/auth/auth.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CoursePageStore } from 'app/courses/pages/course/course.page.store';
import { EMPTY } from 'rxjs';

@Component({
    templateUrl: './course.page.component.html',
    styleUrls: ['./course.page.component.scss'],
    providers: [CoursePageStore],
})
export class CoursePageComponent extends BaseComponent implements OnInit {
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    courseState$ = this.courseDetailStore.courseState$;
    descriptionState$ = this.courseDetailStore.descriptionState$;
    programs$ = this.courseDetailStore.programs$;

    constructor(
        private courseService: CourseService,
        private courseDetailStore: CoursePageStore,
        private router: Router,
        private store: Store<SubatoState>,
        private authService: AuthService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseDetailStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.courseDetailStore.loadCourse(courseId)
            );
    }
}
