import { Injectable } from '@angular/core';
import { map, tap } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { DegreeProgram } from 'app/modules/models/degree-program';
import { DistinctHelper } from 'app/shared/misc/helper/distinct.helper';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import {
    DescriptionState,
    loadDescription,
} from 'app/courses/shared/operators/description.operators';

interface CoursePageState {
    courseState: CourseState;
    descriptionState: DescriptionState;
    programs: DegreeProgram[];
}

export const initialState: CoursePageState = {
    courseState: { course: null, error: null, loading: true },
    descriptionState: { description: null, error: null, loading: true },
    programs: [],
};

@Injectable()
export class CoursePageStore extends BaseComponentStore<CoursePageState> {
    public readonly programs$ = this.select((state) => state.programs);
    public readonly courseState$ = this.select((state) => state.courseState);
    public readonly course$ = this.select((state) => state.courseState.course);
    public readonly descriptionState$ = this.select(
        (state) => state.descriptionState
    );

    constructor() {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');
    readonly patchDescription = this.updaterFor('descriptionState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.course$.pipe(dispatchCourseChange)
    );

    readonly loadDescription$ = this.effect(() =>
        this.course$.pipe(
            loadDescription(false, (s) => this.patchDescription(s))
        )
    );

    readonly loadPrograms$ = this.effect(() =>
        this.course$.pipe(
            map((course) => course?.modules ?? []),
            map((modules) =>
                DistinctHelper.distinct(
                    modules.map((m) => m.program),
                    (program) => program.id
                )
            ),
            tap((programs) => this.patchState({ programs: programs }))
        )
    );
}
