import { Component, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import {
    ComponentWithOptionalLifecycleHooks,
    ifAttached,
} from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { CoursesPageStore } from 'app/courses/pages/courses/courses.page.store';
import { combineLatest } from 'rxjs';
import { map, takeUntil } from 'rxjs/operators';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';

@Component({
    templateUrl: './courses.page.component.html',
    styleUrls: ['./courses.page.component.scss'],
    providers: [CoursesPageStore],
})
export class CoursesPageComponent
    extends BaseComponent
    implements OnInit, ComponentWithOptionalLifecycleHooks
{
    programGroupsState$ = this.coursesStore.programGroupsState$;
    query$ = this.coursesStore.query$;
    expand$ = this.coursesStore.expand$;
    term$ = this.coursesStore.term$;
    canCreateCourse$ = this.coursesStore.canCreateCourse$;

    actions$ = combineLatest([this.canCreateCourse$]).pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map(([canCreateCourse]) => this.buildActions(canCreateCourse))
    );

    constructor(private coursesStore: CoursesPageStore) {
        super();
    }

    onAttach() {
        this.coursesStore.onAttach();
    }

    onDetach() {
        this.coursesStore.onDetach();
    }

    ngOnInit(): void {}

    filterCourses(query) {
        this.coursesStore.patchState({ query: query });
    }

    buildActions(canCreateCourse: boolean): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (canCreateCourse) {
            actions.push({
                name: 'Neuer Kurs',
                icon: 'plus-outline',
                link: {
                    commands: ['new'],
                },
                status: 'primary',
            });
        }

        return actions;
    }
}
