import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { Observable } from 'rxjs';
import { Course } from '../../../models/course';
import { CourseFilter, CourseService } from '../../../services/course.service';
import { DistinctHelper } from 'app/shared/misc/helper/distinct.helper';
import { map } from 'rxjs/operators';
import { Injectable } from '@angular/core';
import { ProgramGroup } from '../models/program-group';

@Injectable({
    providedIn: 'root',
})
export class ProgramGroupService {
    constructor(private courseService: CourseService) {}

    public getAll(
        filter?: CourseFilter,
        projection?: DataQueryParams
    ): Observable<ProgramGroup[]> {
        return this.courseService
            .getAll(filter, projection)
            .pipe(map((courses) => this.createProgramGroups(courses)));
    }

    private createProgramGroups(courses: Course[]) {
        let programs = courses
            .map((course) => course.modules)
            .flat()
            .map((m) => m.program);
        let distinctPrograms = DistinctHelper.distinct(
            programs,
            (program) => program.id
        );
        let programGroups: ProgramGroup[] = distinctPrograms
            .map((program) => {
                return {
                    program: program,
                    description: `PO ${program.poYear}`,
                    courses: courses
                        .filter((course) =>
                            course.modules.some(
                                (m) => m.program.id == program.id
                            )
                        )
                        .sort(
                            (a, b) =>
                                a.moduleForProgram(program).scheduledTerm -
                                b.moduleForProgram(program).scheduledTerm
                        ),
                };
            })
            .sort((a, b) => a.program.name.localeCompare(b.program.name));

        var coursesWithoutModules = courses.filter(
            (c) => c.modules.length == 0
        );
        if (coursesWithoutModules.length != 0) {
            programGroups.splice(0, 0, {
                program: null,
                courses: coursesWithoutModules,
                description: 'Kurse ohne Bezug zu einem Studiengang',
            });
        }

        return programGroups;
    }
}
