import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import { ProgramModule } from 'app/modules/models/program-module';
import { ProgramModuleService } from 'app/modules/services/program-module.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { ModuleMultiselectStore } from 'app/courses/pages/new-course/components/module-multiselect/module-multiselect.store';
import { take } from 'rxjs/operators';

@Component({
    selector: 'sb-new-course-module-multiselect',
    templateUrl: './module-multiselect.component.html',
    styleUrls: ['./module-multiselect.component.scss'],
    providers: [ModuleMultiselectStore],
})
export class ModuleMultiselectComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() selectedModules: ProgramModule[] = [];
    @Output() modulesSelected: EventEmitter<ProgramModule[]> = new EventEmitter<
        ProgramModule[]
    >();

    selectedModules$ = this.moduleMultiselectStore.selectedModules$;
    modules$ = this.moduleMultiselectStore.modules$;
    loading$ = this.moduleMultiselectStore.loading$;
    error$ = this.moduleMultiselectStore.error$;
    query$ = this.moduleMultiselectStore.query$;
    term$ = this.moduleMultiselectStore.term$;

    constructor(
        private programModuleService: ProgramModuleService,
        private store: Store<SubatoState>,
        private moduleMultiselectStore: ModuleMultiselectStore
    ) {
        super();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('selectedModules')) {
            this.moduleMultiselectStore.addModules(this.selectedModules);
        }
    }

    ngOnInit(): void {}

    filterModules(query) {
        this.moduleMultiselectStore.patchState({ query: query });
    }

    private emitSelectedModules() {
        this.moduleMultiselectStore.selectedModules$
            .pipe(take(1))
            .subscribe((modules) => this.modulesSelected.emit(modules));
    }

    add(module: ProgramModule) {
        this.moduleMultiselectStore.addModules([module]);
        this.emitSelectedModules();
    }

    remove(module: ProgramModule) {
        this.moduleMultiselectStore.removeModule(module);
        this.emitSelectedModules();
    }

    canAddModule(module: ProgramModule) {
        return !this.moduleMultiselectStore.containsModule(
            this.selectedModules,
            module
        );
    }
}
