import { SubatoError } from 'app/shared/error/error';
import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseService } from 'app/courses/services/course.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { CreateCourseDto } from 'app/courses/dto/create-course.dto';

export interface NewCoursePageState {
    error: SubatoError;
    saving: boolean;
}

export const initialState: NewCoursePageState = {
    error: null,
    saving: false,
};

@Injectable()
export class NewCoursePageStore extends ComponentStore<NewCoursePageState> {
    public readonly saving$ = this.select((s) => s.saving);
    public readonly error$ = this.select((s) => s.error);

    constructor(
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly createCourse = this.effect((params: Observable<CreateCourseDto>) =>
        params.pipe(
            tap(() => this.patchState({ saving: true })),
            switchMap((dto) =>
                this.courseService.create(dto).pipe(
                    tap((course) => {
                        this.patchState({ saving: false });
                        this.notificationService.showSuccess(
                            'Kurs erfolgreich erstellt'
                        );
                        this.router.navigate(['course', course.id]);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ saving: false });
                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Erstellung des Kurses',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
