import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Course } from '../models/course';
import { HttpCacheManager } from '@ngneat/cashew';
import { CourseFile } from '../models/course-file';
import { CreateCourseFileDto } from '../dto/create-course-file.dto';
import { DownloadService } from 'app/shared/download/download.service';

@Injectable({
    providedIn: 'root',
})
export class CourseFileService extends BaseService {
    constructor(
        private http: HttpClient,
        private downloadService: DownloadService,
        private cacheManager: HttpCacheManager
    ) {
        super();
    }

    getById(id: number): Observable<Course> {
        return this.http
            .get<Course>(`${environment.backendUrl}/course/${id}`)
            .pipe(map((course) => new Course(course)));
    }

    getFileUrl(file: CourseFile): string {
        return `${environment.backendUrl}/course_file/${file.id}/download`;
    }

    create(dto: CreateCourseFileDto, file: File): Observable<CourseFile> {
        var formData = new FormData();
        if (file) {
            formData.append('file', file, file.name);
        }

        formData.append(
            'metadata',
            new Blob([JSON.stringify(dto)], { type: 'application/json' })
        );

        return this.http
            .post<CourseFile>(`${environment.backendUrl}/course_file`, formData)
            .pipe(map((response) => new CourseFile(response)));
    }

    delete(id: number): Observable<void> {
        return this.http
            .delete<void>(`${environment.backendUrl}/course_file/${id}`)
            .pipe(map(() => this.cacheManager.clear()));
    }
}
