import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { BehaviorSubject, combineLatest, Subject } from 'rxjs';
import { debounceTime, filter, map, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/misc/base.component';
import { CourseResultFilter } from 'app/courses/services/course.service';
import { environment } from 'environments/environment';

@Component({
    selector: 'sb-course-result-filter',
    templateUrl: './course-result-filter.component.html',
    styleUrls: ['./course-result-filter.component.scss'],
})
export class CourseResultFilterComponent
    extends BaseComponent
    implements OnInit
{
    @Input() filter: CourseResultFilter = {
        query: '',
        showInvisible: false,
    };

    @Input() showInvisibleFilter: boolean = false;

    @Output() filterChanged: EventEmitter<CourseResultFilter> =
        new EventEmitter<CourseResultFilter>();

    _query: Subject<string> = new BehaviorSubject<string>(null);
    query$ = this._query.asObservable();

    _hideInvisible: Subject<boolean> = new BehaviorSubject<boolean>(null);
    hideInvisible$ = this._hideInvisible.asObservable();

    constructor() {
        super();
    }

    ngOnInit(): void {
        combineLatest(
            this.hideInvisible$,
            this.query$.pipe(debounceTime(environment.debounceTimeMs))
        )
            .pipe(
                takeUntil(this.destroy$),
                map(([showInvisible, query]) => ({
                    query: query ?? this.filter.query,
                    showInvisible: showInvisible ?? this.filter.showInvisible,
                })),
                filter((f) => this.hasFilterChanged(f))
            )
            .subscribe((f) => {
                this.filter = f;
                this.filterChanged.emit(f);
            });
    }

    private hasFilterChanged(newFilter: CourseResultFilter) {
        return (
            newFilter?.showInvisible != this.filter?.showInvisible ||
            newFilter?.query != this.filter?.query
        );
    }

    onHideInvisibleChanged(val: boolean) {
        this._hideInvisible.next(val);
    }

    onQueryChange(query) {
        this._query.next(query);
    }
}
