import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import { Course } from 'app/courses/models/course';
import { User } from 'app/shared/user/user';
import { CourseResultStore } from 'app/courses/shared/course-result/course-result.store';
import { CourseResultFilter } from 'app/courses/services/course.service';

@Component({
    selector: 'sb-course-result-tab',
    templateUrl: './course-result-tab.component.html',
    styleUrls: ['./course-result-tab.component.scss'],
    providers: [CourseResultStore],
})
export class CourseResultTabComponent implements OnInit, OnChanges {
    @Input() course: Course;
    @Input() user: User;
    @Input() showInvisibleFilter: boolean;

    loadingResults$ = this.courseResultStore.loadingResults$;
    courseResult$ = this.courseResultStore.courseResult$;
    courseResultFilter$ = this.courseResultStore.courseResultFilter$;

    constructor(private courseResultStore: CourseResultStore) {}

    ngOnInit(): void {}

    onFilterChanged(filter: CourseResultFilter) {
        this.courseResultStore.patchState({
            courseResultFilter: filter,
        });
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('course')) {
            this.courseResultStore.setCourse(this.course);
        }

        if (changes.hasOwnProperty('user')) {
            this.courseResultStore.setUser(this.user);
        }

        if (changes.hasOwnProperty('showInvisibleFilter')) {
            this.courseResultStore.setFilter({ showInvisible: false });
        }
    }
}
