import { Course } from 'app/courses/models/course';
import { EMPTY, Observable } from 'rxjs';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';

export interface DescriptionState {
    description: string | null;
    error: SubatoError;
    loading: boolean;
}

export function loadDescription(
    raw: boolean,
    updaterFn: (state: Partial<DescriptionState>) => void
) {
    let op = inject(DescriptionOperators);
    return (course: Observable<Course>) =>
        op.loadDescription(course, raw, updaterFn);
}

@Injectable({ providedIn: 'root' })
class DescriptionOperators {
    constructor(
        private courseService: CourseService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadDescription(
        course$: Observable<Course>,
        raw: boolean,
        updaterFn: (state: Partial<DescriptionState>) => void
    ) {
        return course$.pipe(
            filter((course) => course != null),
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((course) =>
                this.courseService.getDescription(course.id, raw).pipe(
                    tap((description) =>
                        updaterFn({
                            description: description,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            description: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
