import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { Exercise } from './models/exercise';
import { TaskInstance } from './models/task-instance';
import { NewExercisePageComponent } from 'app/exercises/pages/exercise-admin/new-exercise/new-exercise.page.component';
import { EditExercisePageComponent } from 'app/exercises/pages/exercise-admin/edit-exercise/edit-exercise.page.component';
import { ExercisesPageComponent } from 'app/exercises/pages/exercises/exercises.page.component';
import { ExercisePageComponent } from 'app/exercises/pages/exercise/exercise.page.component';
import { SubmitSolutionPageComponent } from 'app/exercises/pages/submit-solution/submit-solution.page.component';

export const routes: Routes = [
    {
        path: 'course/:course_id/exercises',
        component: ExercisesPageComponent,
        data: {
            breadcrumb: {
                alias: 'exercises',
                label: 'Übungen',
            },
            reuseRoute: true,
            courseMenuEnabled: true,
        },
    },
    {
        path: 'course/:course_id/exercises/new',
        component: NewExercisePageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'newExercise',
                label: 'Erstellen',
            },
            courseMenuEnabled: true,
        },
    },
    {
        path: 'exercises/:exercise_id',
        component: ExercisePageComponent,
        data: {
            breadcrumb: {
                alias: 'exercise',
                label: (exercise: Exercise) => `${exercise.name}`,
            },
            courseMenuEnabled: true,
        },
    },
    {
        path: 'exercises/:exercise_id/edit',
        component: EditExercisePageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'editExercise',
                label: `Bearbeiten`,
            },
            courseMenuEnabled: true,
        },
    },

    {
        path: 'exercises/:exercise_id/tasks/:task_id',
        component: SubmitSolutionPageComponent,
        data: {
            breadcrumb: {
                alias: 'submitSolution',
                label: (ti: TaskInstance) => `${ti.task.name}`,
            },
            courseMenuEnabled: true,
        },
    },
];

@NgModule({
    exports: [RouterModule],
})
export class ExercisesRoutingModule {}
