import { Task } from 'app/tasks/models/task';
import { Exercise } from './exercise';

export class TaskInstance {
    task: Task;
    orderNumber: number;
    reachablePoints?: number;
    exercise: Exercise;
    solutionPublic: boolean;
    downloadBundleUrl: string;

    constructor(taskInstance: any) {
        this.orderNumber = taskInstance.orderNumber;
        this.reachablePoints = taskInstance.reachablePoints;
        this.task = taskInstance.task ? new Task(taskInstance.task) : null;
        this.exercise = taskInstance.exercise
            ? new Exercise(taskInstance.exercise)
            : null;
        this.solutionPublic = taskInstance.solutionPublic;
        this.downloadBundleUrl = taskInstance.downloadBundleUrl;
    }

    hasPoints() {
        return this.reachablePoints != null;
    }

    getNextTaskInstance(taskInstances: TaskInstance[]): TaskInstance | null {
        let idx =
            taskInstances.map((ti) => ti.task.id).indexOf(this.task.id) + 1;
        return idx < taskInstances.length ? taskInstances[idx] : null;
    }

    getPrevTaskInstance(taskInstances: TaskInstance[]): TaskInstance | null {
        let idx =
            taskInstances.map((ti) => ti.task.id).indexOf(this.task.id) - 1;
        return idx >= 0 ? taskInstances[idx] : null;
    }

    equals(other: TaskInstance) {
        return (
            this.task?.id == other?.task?.id &&
            this.exercise?.id == other?.exercise?.id
        );
    }
}
