import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Task } from 'app/tasks/models/task';
import { TaskPool } from 'app/tasks/models/task-pool';
import { TaskSelectStore } from 'app/exercises/pages/exercise-admin/exercise-form/task-select/task-select.store';

@Component({
    selector: 'sb-exercise-form-task-select',
    templateUrl: './task-select.component.html',
    styleUrls: ['./task-select.component.scss'],
    providers: [TaskSelectStore],
})
export class TaskSelectComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() selectedTasks: Task[] = [];
    @Output() taskSelected: EventEmitter<Task> = new EventEmitter<Task>();

    query$ = this.taskSelectStore.query$;
    showArchived$ = this.taskSelectStore.showArchived$;
    pools$ = this.taskSelectStore.pools$;
    selectedPool$ = this.taskSelectStore.selectedPool$;

    constructor(public taskSelectStore: TaskSelectStore) {
        super();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('selectedTasks')) {
            this.taskSelectStore.selectTasks(this.selectedTasks);
        }
    }

    ngOnInit(): void {
        this.taskSelectStore.loadPools();
    }

    selectPool(pool: TaskPool) {
        this.taskSelectStore.setPool(pool);
    }

    filterTasks(query) {
        this.taskSelectStore.setQuery(query);
    }

    onShowArchivedChanged(showArchived) {
        this.taskSelectStore.setShowArchived(showArchived);
    }

    selectTask(task: Task) {
        this.taskSelectStore.selectTasks([task]);
        this.taskSelected.emit(task);
    }

    canAddTask(task: Task) {
        return !this.taskSelectStore.containsTask(this.selectedTasks, task);
    }
}
