import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CreateExerciseDto } from 'app/exercises/dto/create-exercise.dto';
import { EMPTY } from 'rxjs';
import { NewExercisePageStore } from 'app/exercises/pages/exercise-admin/new-exercise/new-exercise.page.store';

@Component({
    templateUrl: './new-exercise.page.component.html',
    styleUrls: ['./new-exercise.page.component.scss'],
    providers: [NewExercisePageStore],
})
export class NewExercisePageComponent extends BaseComponent implements OnInit {
    courseState$ = this.newExerciseStore.courseState$;

    validationError$ = this.newExerciseStore.validationError$;
    saving$ = this.newExerciseStore.saving$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    constructor(
        private store: Store<SubatoState>,
        private newExerciseStore: NewExercisePageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.newExerciseStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.newExerciseStore.loadCourse(courseId)
            );
    }

    createExercise(dto: CreateExerciseDto) {
        this.newExerciseStore.createExercise(dto);
    }
}
