import { Component, Input, OnInit } from '@angular/core';
import { Exercise } from '../../../../models/exercise';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Observable } from 'rxjs';
import { map, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/misc/base.component';

@Component({
    selector: 'sb-exercise-visibility-hint',
    templateUrl: './visibility-hint.component.html',
    styleUrls: ['./visibility-hint.component.scss'],
})
export class VisibilityHintComponent extends BaseComponent implements OnInit {
    @Input() exercise: Exercise;
    principal$ = this.store.select((s) => s.auth.principal);
    show$: Observable<boolean>;

    constructor(private store: Store<SubatoState>) {
        super();
    }

    ngOnInit(): void {
        this.show$ = this.principal$.pipe(
            takeUntil(this.destroy$),
            map((principal) => {
                let isLecturerOrAdmin =
                    principal?.isLecturerOrAdmin(this.exercise.course.id) ??
                    false;
                return isLecturerOrAdmin && !this.exercise.visible;
            })
        );
    }
}
