import { Component, Input, OnInit } from '@angular/core';
import { environment } from 'environments/environment';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { DocsService } from 'app/shared/misc/docs.service';

@Component({
    selector: 'sb-submit-solution-intellij-import',
    templateUrl: './intellij-import.component.html',
    styleUrls: ['./intellij-import.component.scss'],
})
export class IntellijImportComponent implements OnInit {
    @Input() taskInstance: TaskInstance | null;

    docsUrls = this.docsService.urls;
    importUrl: string | null;
    copied = false;

    constructor(private docsService: DocsService) {}

    ngOnInit(): void {
        if (this.taskInstance != null) {
            this.generateImportUrl();
        }
    }

    generateImportUrl() {
        let rootUrl = window.location.protocol + '//' + window.location.host;
        this.importUrl = `${rootUrl}/exercises/${this.taskInstance.exercise.id}/tasks/${this.taskInstance.task.id}`;
    }

    copyUrl() {
        this.copyToClipboard(this.importUrl);
        this.copied = true;
    }

    copyToClipboard(val: string) {
        const selBox = document.createElement('textarea');
        selBox.style.position = 'fixed';
        selBox.style.left = '0';
        selBox.style.top = '0';
        selBox.style.opacity = '0';
        selBox.value = val;
        document.body.appendChild(selBox);
        selBox.focus();
        selBox.select();
        document.execCommand('copy');
        document.body.removeChild(selBox);
    }
}
