import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { takeUntil } from 'rxjs/operators';
import { Exercise } from 'app/exercises/models/exercise';
import { Submission } from 'app/assessment/models/submission';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { TaskInstanceListStore } from 'app/exercises/shared/components/task-instance-list/task-instance-list.store';

@Component({
    selector: 'sb-task-instance-list',
    templateUrl: './task-instance-list.component.html',
    styleUrls: ['./task-instance-list.component.scss'],
    providers: [TaskInstanceListStore],
})
export class TaskInstanceListComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() exercise: Exercise;
    @Input() submission: Submission | null;
    @Input() selectedTaskInstance: TaskInstance | null;

    @Input() showAttempts: boolean = true;
    @Input() compactMode: boolean = false;
    @Input() showDescription: boolean = false;
    @Input() showLanguage: boolean = true;
    @Input() showPoints: boolean = false;
    @Input() detailLinkSupplier: (ti: TaskInstance) => any = (ti) => null;
    @Input() detailLinkParamsSupplier: (ti: TaskInstance) => any = (ti) => null;

    @Output() taskSelected: EventEmitter<TaskInstance> =
        new EventEmitter<TaskInstance>();
    @Output() tasksLoaded: EventEmitter<TaskInstance[]> = new EventEmitter<
        TaskInstance[]
    >();

    taskInstancesState$ = this.exerciseTasksListStore.taskInstancesState$;
    selectedTaskInstance$ = this.exerciseTasksListStore.selectedTaskInstance$;

    constructor(private exerciseTasksListStore: TaskInstanceListStore) {
        super();
    }

    ngOnInit(): void {
        this.exerciseTasksListStore.taskInstances$
            .pipe(takeUntil(this.destroy$))
            .subscribe((taskInstances) => this.tasksLoaded.emit(taskInstances));
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('exercise')) {
            this.exerciseTasksListStore.patchState({ exercise: this.exercise });
        }

        if (changes.hasOwnProperty('selectedTaskInstance')) {
            this.exerciseTasksListStore.patchState({
                selectedTaskInstance: this.selectedTaskInstance,
            });
        }
    }

    selectTask(taskInstance: TaskInstance) {
        this.exerciseTasksListStore.patchState({
            selectedTaskInstance: taskInstance,
        });
        this.taskSelected.emit(taskInstance);
    }
}
