import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadTaskInstances,
    TaskInstancesState,
} from 'app/exercises/shared/operators/task-instance.operators';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Exercise } from 'app/exercises/models/exercise';

interface TaskInstanceListState {
    exercise: Exercise;
    taskInstancesState: TaskInstancesState;
    selectedTaskInstance: TaskInstance;
}

export const initialState: TaskInstanceListState = {
    exercise: null,
    taskInstancesState: { taskInstances: [], error: null, loading: true },
    selectedTaskInstance: null,
};

@Injectable()
export class TaskInstanceListStore extends BaseComponentStore<TaskInstanceListState> {
    public readonly exercise$ = this.select((state) => state.exercise);
    public readonly taskInstances$ = this.select(
        (state) => state.taskInstancesState.taskInstances
    );
    public readonly taskInstancesState$ = this.select(
        (state) => state.taskInstancesState
    );

    public readonly selectedTaskInstance$ = this.select(
        (state) => state.selectedTaskInstance
    );

    constructor() {
        super(initialState);
    }

    readonly patchTaskInstances = this.updaterFor('taskInstancesState');

    readonly setExercise = this.updater((state, exercise: Exercise) => ({
        ...state,
        exercise: exercise,
    }));

    readonly loadTaskInstances$ = this.effect(() =>
        this.exercise$.pipe(
            filter((exercise) => exercise != null),
            map((exercise) => exercise.id),
            loadTaskInstances((s) => this.patchTaskInstances(s))
        )
    );
}
