import { EMPTY, Observable } from 'rxjs';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { Exercise } from 'app/exercises/models/exercise';

export interface DescriptionState {
    description: string | null;
    error: SubatoError;
    loading: boolean;
}

export function loadDescription(
    raw: boolean,
    updaterFn: (state: Partial<DescriptionState>) => void
) {
    let op = inject(DescriptionOperators);
    return (exercise: Observable<Exercise>) =>
        op.loadDescription(exercise, raw, updaterFn);
}

@Injectable({ providedIn: 'root' })
class DescriptionOperators {
    constructor(
        private exerciseService: ExerciseService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadDescription(
        exercise$: Observable<Exercise>,
        raw: boolean,
        updaterFn: (state: Partial<DescriptionState>) => void
    ) {
        return exercise$.pipe(
            filter((course) => course != null),
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((course) =>
                this.exerciseService.getDescription(course.id, raw).pipe(
                    tap((description) =>
                        updaterFn({
                            description: description,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            description: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
