import { Pipe, PipeTransform } from '@angular/core';
import { LruCache } from 'app/shared/misc/helper/lru-cache';
import { Exercise } from 'app/exercises/models/exercise';
import { Submission } from 'app/assessment/models/submission';
import { TaskInstance } from 'app/exercises/models/task-instance';

const isExercise = (b: Exercise | TaskInstance): b is Exercise => {
    return (b as Exercise).name !== undefined;
};

@Pipe({
    name: 'points',
    pure: false,
})
export class PointsPipe implements PipeTransform {
    cache: LruCache<string> = new LruCache<string>();

    transform(val: TaskInstance | Exercise, submission: Submission): any {
        let formatted = '';

        let key = `${isExercise(val) ? 'e' : 'ti'}-${
            isExercise(val) ? val.id : val.exercise.id + ' ' + val.task.id
        }-${submission?.totalPoints}`;
        let cached = this.cache.get(key);
        if (cached) {
            return cached;
        }

        if (isExercise(val)) {
            if (submission == null) {
                formatted = `${val.reachablePoints} Punkte`;
            } else {
                var reachedPoints = submission.totalPoints ?? '-';
                formatted = `${reachedPoints} / ${val.reachablePoints} Punkte`;
            }
        } else {
            if (submission == null) {
                formatted = `${val.reachablePoints} Punkte`;
            } else {
                var reachedPoints = submission.reachedPointsFor(val) ?? '-';
                formatted = `${reachedPoints} / ${val.reachablePoints} Punkte`;
            }
        }
        this.cache.put(key, formatted);
        return formatted;
    }
}
