import { Component, OnDestroy, OnInit } from '@angular/core';
import { NbMenuItem, NbMenuService, NbSidebarService } from '@nebular/theme';

import { filter, takeUntil } from 'rxjs/operators';
import { HeaderStore } from 'app/layout/components/header/header.store';
import { BaseComponent } from 'app/shared/misc/base.component';
import { environment } from 'environments/environment';

@Component({
    selector: 'sb-layout-header',
    styleUrls: ['./header.component.scss'],
    templateUrl: './header.component.html',
    providers: [HeaderStore],
})
export class HeaderComponent
    extends BaseComponent
    implements OnInit, OnDestroy
{
    userMenu: NbMenuItem[] = [
        {
            title: 'Profil',
            link: `/profile`,
            icon: 'person-outline',
        },
        { title: 'Abmelden', icon: 'log-out-outline' },
    ];

    userPictureOnly$ = this.headerStore.userPictureOnly$;
    showMenu$ = this.headerStore.showMenu$;
    darkMode$ = this.headerStore.darkMode$;
    principal$ = this.headerStore.principal$;
    course$ = this.headerStore.course$;
    mainMenu$ = this.headerStore.mainMenu$;

    imagesConfig = environment.images;

    public constructor(
        private sidebarService: NbSidebarService,
        private menuService: NbMenuService,
        private headerStore: HeaderStore
    ) {
        super();
    }

    ngOnInit() {
        this.menuService
            .onItemClick()
            .pipe(
                takeUntil(this.destroy$),
                filter(({ tag }) => tag === 'user-menu'),
                filter(({ item: { link } }) => link == null)
            )
            .subscribe(() => this.headerStore.logout());
    }

    toggleTheme() {
        this.headerStore.toggleTheme();
    }

    toggleSidebar(): boolean {
        this.sidebarService.toggle(true, 'menu-sidebar');
        return false;
    }

    navigateHome() {
        this.headerStore.navigateHome();
        return false;
    }

    login() {
        this.headerStore.login();
    }
}
