import { Component, ElementRef, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { map, switchMap, take, takeUntil } from 'rxjs/operators';
import { NbMenuItem, NbMenuService, NbSidebarService } from '@nebular/theme';
import { merge, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Term } from 'app/modules/models/term';
import { cloneDeep } from 'lodash';

@Component({
    selector: 'sb-layout-sidebar',
    templateUrl: './sidebar.component.html',
    styleUrls: ['./sidebar.component.scss'],
})
export class SidebarComponent extends BaseComponent implements OnInit {
    term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );
    items$: Observable<NbMenuItem[]> = this.store
        .select((s) => s.layout.sideNav.items)
        .pipe(map((items) => cloneDeep(items))); // nebular may change the individual items but the store is immutable
    showTermSelection$: Observable<boolean> = this.store.select(
        (s) => s.layout.sideNav.showTermSelection
    );

    constructor(
        private el: ElementRef,
        private store: Store<SubatoState>,
        private menuService: NbMenuService,
        private sidebarService: NbSidebarService
    ) {
        super();
    }

    ngOnInit(): void {
        merge(this.menuService.onItemClick(), this.term$)
            .pipe(
                takeUntil(this.destroy$),
                switchMap(() =>
                    this.sidebarService
                        .getSidebarResponsiveState()
                        .pipe(take(1))
                )
            )
            .subscribe((state) => {
                if (state != 'pc') {
                    this.sidebarService.collapse();
                }
            });
    }
}
