import { ModuleWithProviders, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { FooterComponent } from './components/footer/footer.component';
import { HeaderComponent } from './components/header/header.component';
import { SharedModule } from 'app/shared/shared.module';
import { NbThemeModule } from '@nebular/theme';
import { OneColumnLayoutComponent } from './layouts/one-column/one-column.layout';
import { ThreeColumnsLayoutComponent } from './layouts/three-columns/three-columns.layout';
import { TwoColumnsLayoutComponent } from './layouts/two-columns/two-columns.layout';
import { SidebarComponent } from 'app/layout/components/sidebar/sidebar.component';
import { HorizontalMenuComponent } from './components/horizontal-menu/horizontal-menu.component';
import { StoreModule } from '@ngrx/store';
import { layoutReducer } from './store/layout.reducers';
import { EffectsModule } from '@ngrx/effects';
import { LayoutEffects } from './store/layout.effects';
import { ModulesSharedModule } from 'app/modules/shared/modules-shared.module';
import { SearchSharedModule } from 'app/search/shared/search-shared.module';
import { environment } from 'environments/environment';

export const SUBATO_THEME = 'subato';
export const SUBATO_DARK_THEME = 'subatoDark';
export const SUBATO_TEST_THEME = 'subatoTest';
export const SUBATO_TEST_DARK_THEME = 'subatoTestDark';

@NgModule({
    declarations: [
        HeaderComponent,
        FooterComponent,
        OneColumnLayoutComponent,
        ThreeColumnsLayoutComponent,
        TwoColumnsLayoutComponent,
        SidebarComponent,
        HorizontalMenuComponent,
    ],
    imports: [
        CommonModule,
        FormsModule,
        RouterModule,
        SharedModule,
        StoreModule.forFeature('layout', layoutReducer),
        EffectsModule.forFeature([LayoutEffects]),
        SearchSharedModule,
        ModulesSharedModule,
    ],
    exports: [
        CommonModule,
        OneColumnLayoutComponent,
        ThreeColumnsLayoutComponent,
        TwoColumnsLayoutComponent,
    ],
})
export class LayoutModule {
    static forRoot(): ModuleWithProviders<LayoutModule> {
        return {
            ngModule: LayoutModule,
            providers: [
                ...NbThemeModule.forRoot({
                    name: environment.lightTheme,
                }).providers,
            ],
        };
    }
}
