import { Component } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NbMenuItem } from '@nebular/theme';
import { Observable } from 'rxjs';

@Component({
    selector: 'sb-layout-one-column',
    styleUrls: ['./one-column.layout.scss'],
    template: `
        <nb-layout
            *ngrxLet="items$ as items"
            [restoreScrollTop]="false"
            [class.hasMenu]="items.length != 0"
            center
        >
            <nb-layout-header fixed>
                <sb-layout-header></sb-layout-header>
            </nb-layout-header>

            <nb-sidebar
                *ngIf="items.length != 0"
                responsive
                [collapsedBreakpoints]="['xs', 'is', 'sm', 'md']"
                [compactedBreakpoints]="['xs', 'is', 'sm', 'md']"
                class="menu-sidebar"
            >
                <sb-layout-sidebar></sb-layout-sidebar>
            </nb-sidebar>

            <nb-layout-column>
                <ng-content select="router-outlet"></ng-content>
            </nb-layout-column>

            <nb-layout-footer fixed>
                <sb-layout-footer></sb-layout-footer>
            </nb-layout-footer>
        </nb-layout>
    `,
})
export class OneColumnLayoutComponent {
    items$: Observable<NbMenuItem[] | null> = this.store.select(
        (s) => s.layout.sideNav.items
    );

    constructor(private store: Store<SubatoState>) {}
}
