import { NbMenuItem } from '@nebular/theme';
import { Course } from 'app/courses/models/course';
import { environment } from 'environments/environment';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';

export class CourseMenuBuilder {
    public static getMenu(
        currentUrl: string,
        course: Course | null,
        principal: SubatoPrincipal | null
    ): NbMenuItem[] {
        let courseBaseUrl = `/course/${course.id}`;
        let relativeUrl = currentUrl.replace(courseBaseUrl, '');

        let menu: NbMenuItem[] = [
            {
                title: course.term
                    ? `${course.shortName} (${course.term.name})`
                    : course.shortName,
                group: true,
            },
            {
                title: 'Übersicht',
                icon: 'activity-outline',
                link: courseBaseUrl,
                selected: relativeUrl == '',
            },
            {
                title: 'Mein Fortschritt',
                icon: 'bar-chart-outline',
                link: `${courseBaseUrl}/myprogress`,
                hidden: !(
                    principal != null && principal.isParticipant(course.id)
                ),
                selected: relativeUrl.startsWith('/myprogress'),
            },
            {
                title: 'Übungen',
                icon: 'file-text-outline',
                link: `${courseBaseUrl}/exercises`,
                selected:
                    relativeUrl.startsWith('/exercises') ||
                    relativeUrl.startsWith('/submissions') ||
                    relativeUrl.startsWith('/assessment') ||
                    relativeUrl.startsWith('/solutions'),
            },
            {
                title: 'Dateien',
                icon: 'file-outline',
                link: `${courseBaseUrl}/files`,
                selected: relativeUrl.startsWith('/files'),
            },
            {
                title: 'Veranstaltungen',
                icon: 'cube-outline',
                link: `${courseBaseUrl}/unit`,
                selected: relativeUrl.startsWith('/unit'),
            },
            {
                title: 'Teilnehmer',
                icon: 'people-outline',
                link: `${courseBaseUrl}/participants`,
                hidden: !(
                    principal != null && principal.hasAnyGroup(course.id)
                ),
                selected: relativeUrl.startsWith('/participants'),
            },
            {
                title: 'Berechtigungen',
                icon: 'unlock-outline',
                link: `${courseBaseUrl}/permissions`,
                hidden: !(principal != null && principal.isAdmin(course.id)),
                selected: relativeUrl == '/permissions',
            },
            {
                title: 'Einstellungen',
                icon: 'settings-2-outline',
                link: `${courseBaseUrl}/settings`,
                hidden: !(principal != null && principal.isAdmin(course.id)),
                selected: relativeUrl == '/settings',
            },
            {
                icon: 'external-link-outline',
                title: 'Cardelli',
                url: environment.cardelliUrl,
                target: '_blank',
            },
        ];

        return menu;
    }
}
