import { NbMenuItem } from '@nebular/theme';
import { Course } from 'app/courses/models/course';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';

export class MainMenuBuilder {
    public static getMenu(
        currentUrl: string,
        course: Course | null,
        principal: SubatoPrincipal | null
    ): NbMenuItem[] {
        return [
            this.coursesItem(currentUrl, course),
            this.tasksItem(currentUrl, course, principal),
            this.adminItem(currentUrl, principal),
        ];
    }

    private static coursesItem(
        currentUrl: string,
        course: Course | null
    ): NbMenuItem {
        return {
            title: 'Kurse',
            icon: 'compass-outline',
            link: '/course',
            selected: currentUrl.startsWith('/course') || course != null,
        };
    }

    private static tasksItem(
        currentUrl: string,
        course: Course | null,
        principal: SubatoPrincipal | null
    ): NbMenuItem {
        return {
            title: 'Aufgaben',
            icon: 'grid-outline',
            link: `/task_pools`,
            hidden: !(
                principal != null &&
                (principal.hasStaffRole() || principal.hasSuperAdminRole())
            ),
            selected:
                currentUrl.startsWith('/task_pools') ||
                currentUrl.startsWith('/tasks') ||
                (course == null && currentUrl.startsWith('/solutions')),
            data: { topNav: true },
        };
    }

    private static adminItem(
        currentUrl: string,
        principal: SubatoPrincipal | null
    ): NbMenuItem {
        let children = [
            {
                title: 'Semester',
                icon: 'calendar-outline',
                link: `/term`,
                selected: currentUrl.startsWith('/term'),
            },
            {
                title: 'Sprachen',
                icon: 'code-outline',
                link: `/language`,
                selected: currentUrl.startsWith('/language'),
            },
            {
                title: 'Fehlvorstellungen',
                icon: 'alert-triangle-outline',
                link: `/misconception`,
                hidden: !(principal != null && principal.hasSuperAdminRole()),
                selected: currentUrl.startsWith('/misconception'),
            },
            {
                title: 'Kompetenzen',
                icon: 'cube-outline',
                link: `/competence`,
                hidden: !(principal != null && principal.hasSuperAdminRole()),
                selected: currentUrl.startsWith('/competence'),
            },
        ];

        return {
            title: 'Verwaltung',
            icon: 'settings-2-outline',
            hidden: !(
                principal != null &&
                (principal.hasStaffRole() || principal.hasSuperAdminRole())
            ),
            selected: children.some((c) => c.selected),
            children: children,
        };
    }
}
