import { LayoutActions, LayoutActionsUnion } from './layout.actions';
import { NbMenuItem } from '@nebular/theme';
import { Action } from '@ngrx/store';

export interface Menu {
    mainMenu: NbMenuItem[];
    courseMenu: NbMenuItem[];
    compact: boolean;
}

export interface SideNav {
    items: NbMenuItem[];
    showTermSelection: boolean;
}

export interface LayoutState {
    darkMode: boolean;
    menu: Menu;
    sideNav: SideNav;
}

export const initialState: LayoutState = {
    darkMode: false,
    menu: { mainMenu: [], courseMenu: [], compact: false },
    sideNav: { items: [], showTermSelection: false },
};

export function layoutReducer(
    state = initialState,
    action: Action
): LayoutState {
    const a = action as LayoutActionsUnion;

    switch (a.type) {
        case LayoutActions.ThemeChange: {
            return {
                ...state,
                darkMode: a.payload.darkMode,
            };
        }

        case LayoutActions.ChangeMainMenu: {
            return {
                ...state,
                menu: {
                    ...state.menu,
                    mainMenu: a.payload.mainMenu,
                },
            };
        }

        case LayoutActions.ChangeCourseMenu: {
            return {
                ...state,
                menu: {
                    ...state.menu,
                    courseMenu: a.payload.courseMenu,
                },
            };
        }

        case LayoutActions.CompactMenu: {
            return {
                ...state,
                menu: {
                    ...state.menu,
                    compact: a.payload.compact,
                },
            };
        }

        case LayoutActions.UpdateSideNav: {
            return {
                ...state,
                sideNav: {
                    showTermSelection: state.menu.compact,
                    items: state.menu.compact
                        ? state.menu.mainMenu.concat(state.menu.courseMenu)
                        : state.menu.courseMenu,
                },
            };
        }

        default: {
            return state;
        }
    }
}
