import { Component, Input, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { filter, takeUntil } from 'rxjs/operators';
import { EvalResult, EvalStatus } from '../../../../services/eval.service';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { EvalStore } from 'app/misconceptions/pages/misconception/components/eval/eval.store';

@Component({
    selector: 'sb-misconception-eval',
    templateUrl: './eval.component.html',
    styleUrls: ['./eval.component.css'],
    providers: [EvalStore],
})
export class EvalComponent extends BaseComponent implements OnInit {
    @Input() code: string = '';
    @Input() highlight: string | null = null;

    selectedTabIndex: number | null = null;

    darkMode$: Observable<boolean> = this.store.select(
        (s) => s.layout.darkMode
    );

    result$ = this.evalStore.result$;
    loading$ = this.evalStore.loading$;
    error$ = this.evalStore.error$;

    constructor(
        private store: Store<SubatoState>,
        private evalStore: EvalStore
    ) {
        super();
    }

    ngOnInit(): void {
        this.result$
            .pipe(
                takeUntil(this.destroy$),
                filter((result) => result != null)
            )
            .subscribe((result) => {
                if (result.status == EvalStatus.cerror) {
                    this.selectedTabIndex = 2;
                } else {
                    if (!result.out && result.result) {
                        this.selectedTabIndex = 1;
                    } else {
                        this.selectedTabIndex = 0;
                    }
                }
            });
    }

    accent(result: EvalResult) {
        if (!result) {
            return 'basic';
        }

        return this.isSuccess(result) ? 'primary' : 'danger';
    }

    execute(code: string) {
        this.evalStore.execute(code);
    }

    hasCompilerErrors(result: EvalResult) {
        return result && result.status == EvalStatus.cerror;
    }

    isError(result: EvalResult) {
        return result && result.status != EvalStatus.ok;
    }

    isSuccess(result: EvalResult) {
        return result && result.status == EvalStatus.ok;
    }

    formatErrors(result: EvalResult) {
        var r = 'Fehler bei der Kompilierung:';
        result.errors.forEach((e) => {
            r += '\n' + e.message;
            if (e.snippet && (!e.startPos || !e.endPos)) {
                r += ': ' + e.snippet;
            }
        });

        return r;
    }
}
