import {
    AfterViewInit,
    Component,
    EventEmitter,
    Input,
    OnInit,
    Output,
    ViewChild,
} from '@angular/core';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { BaseComponent } from 'app/shared/misc/base.component';
import { EvalResult } from '../../../../services/eval.service';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Observable } from 'rxjs';

export function getContrastYIQ(hexcolor: string): string {
    hexcolor = hexcolor.replace('#', '');
    var r = parseInt(hexcolor.substr(0, 2), 16);
    var g = parseInt(hexcolor.substr(2, 2), 16);
    var b = parseInt(hexcolor.substr(4, 2), 16);
    var yiq = (r * 299 + g * 587 + b * 114) / 1000;
    return yiq >= 128 ? 'black' : 'white';
}

@Component({
    selector: 'sb-misconception-snippet-editor',
    templateUrl: './snippet-editor.component.html',
    styleUrls: ['./snippet-editor.component.css'],
})
export class SnippetEditorComponent
    extends BaseComponent
    implements OnInit, AfterViewInit
{
    @ViewChild('editor', { static: true })
    codemirrorComponent!: CodemirrorComponent;

    @Input() code: string = '';
    @Input() highlight: string | null = null;
    _result: EvalResult | null = null;
    @Output() executionRequest: EventEmitter<string> =
        new EventEmitter<string>();

    darkMode$: Observable<boolean> = this.store.select(
        (s) => s.layout.darkMode
    );

    constructor(private store: Store<SubatoState>) {
        super();
    }

    @Input() set result(val: EvalResult | null) {
        this._result = val;
        if (this.editor) {
            this.editor.getAllMarks().forEach((m: any) => m.clear());
            if (val) {
                this.addMarkers(val);
            }
        }
    }

    get result() {
        return this._result;
    }

    ngAfterViewInit() {
        setTimeout(() => {
            if (this.highlight) {
                var startIdx = this.code.indexOf(this.highlight);
                var endIdx = startIdx + this.highlight.length;
                var s = this.editor.getDoc().posFromIndex(startIdx);
                var e = this.editor.getDoc().posFromIndex(endIdx);
                this.editor.markText(s, e, {
                    className: 'ast-highlighting',
                });
            }
        }, 100);
    }

    ngOnInit(): void {}

    private get editor(): any {
        return this.codemirrorComponent.codeMirror;
    }

    addMarkers(result: EvalResult) {
        for (let e of result.errors) {
            if (!e.endPos || !e.startPos) continue;
            var codeLines = this.code.split('\n');
            var snippetLines = e.snippet.split('\n');

            var startLineIdx = codeLines.findIndex((c) =>
                snippetLines.find((c2) => c == c2)
            );
            //var endLineIdx = startLineIdx + snippetLines.length - 1;

            var n = 0;
            var startLine = -1;
            var startPos = -1;
            for (var i = 0; i < snippetLines.length; i++) {
                if (
                    e.startPos! >= n &&
                    e.startPos! <= snippetLines[i].length + n
                ) {
                    startLine = i;
                    startPos = e.startPos! - n;
                    break;
                }
                n += snippetLines[i].length + 1;
            }

            n = 0;
            var endLine = -1;
            var endPos = -1;
            for (var i = 0; i < snippetLines.length; i++) {
                if (e.endPos! >= n && e.endPos! <= snippetLines[i].length + n) {
                    endLine = i;
                    endPos = e.endPos! - n;
                    break;
                }
                n += snippetLines[i].length + 1;
            }

            console.log(startLine, endLine);
            console.log(
                startLineIdx + startLine,
                startLineIdx + endLine,
                startPos,
                endPos
            );

            if (startLine == endLine && startPos == endPos) {
                startPos = 0;
                endPos = snippetLines[startLine].length;
            }

            var marker = this.editor.markText(
                {
                    line: startLine,
                    ch: startPos,
                },
                {
                    line: endLine,
                    ch: endPos,
                },
                {
                    css: `background-color: ${'#f9ccc9'}; color: ${getContrastYIQ(
                        '#f9ccc9'
                    )}`,
                }
            );
        }
    }

    execute() {
        this.executionRequest.emit(this.code);
    }
}
