import { DOCUMENT } from '@angular/common';
import {
    ApplicationRef,
    ComponentFactoryResolver,
    Directive,
    ElementRef,
    Inject,
    Injector,
    Input,
    OnChanges,
    SimpleChanges,
} from '@angular/core';
import { HtmlHelper } from 'app/shared/misc/helper/html.helper';
import { EvalComponent } from '../components/eval/eval.component';

@Directive({
    // tslint:disable-next-line: directive-selector
    selector: 'div[transformInteractiveEditors]',
    exportAs: 'transformInteractiveEditors', // <-- Make not of this here
})
export class TransformInteractiveEditorsDirective implements OnChanges {
    @Input() html: string = '';

    constructor(
        @Inject(DOCUMENT) private document: Document,
        private injector: Injector,
        private applicationRef: ApplicationRef,
        private componentFactoryResolver: ComponentFactoryResolver,
        private element: ElementRef<HTMLElement>
    ) {}

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.html) {
            this.transform();
        }
    }

    transform(): void {
        this.element.nativeElement
            .querySelectorAll('pre.interactive-editor')
            .forEach((a) => {
                const parent = a.parentElement;
                if (parent) {
                    const container = this.document.createElement('div');
                    const component = this.componentFactoryResolver
                        .resolveComponentFactory(EvalComponent)
                        .create(this.injector, [], container);
                    this.applicationRef.attachView(component.hostView);
                    component.instance.code = HtmlHelper.replaceHtmlEntities(
                        a.children[0].innerHTML || ''
                    );
                    parent.replaceChild(container, a);
                }
            });
    }
}
