import { DOCUMENT } from '@angular/common';
import {
    AfterViewInit,
    ApplicationRef,
    ComponentFactoryResolver,
    Directive,
    ElementRef,
    Inject,
    Injector,
} from '@angular/core';
import { TooltipLinkComponent } from '../components/tooltip-link/tooltip-link.component';

@Directive({
    // tslint:disable-next-line: directive-selector
    selector: 'div[transformTooltips]',
    exportAs: 'transformTooltips', // <-- Make not of this here
})
export class TransformTooltipsDirective implements AfterViewInit {
    constructor(
        @Inject(DOCUMENT) private document: Document,
        private injector: Injector,
        private applicationRef: ApplicationRef,
        private componentFactoryResolver: ComponentFactoryResolver,
        private element: ElementRef<HTMLElement>
    ) {}

    ngAfterViewInit(): void {
        this.element.nativeElement
            .querySelectorAll('a.tooltip')
            .forEach((a) => {
                const parent = a.parentElement;
                if (parent) {
                    const container = this.document.createElement('span');
                    const component = this.componentFactoryResolver
                        .resolveComponentFactory(TooltipLinkComponent)
                        .create(this.injector, [], container);
                    this.applicationRef.attachView(component.hostView);
                    component.instance.tooltip = a.getAttribute('title') || '';
                    component.instance.text = a.textContent || '';

                    parent.replaceChild(container, a);
                }
            });
    }
}
