import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { map, takeUntil } from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { MisconceptionPageStore } from 'app/misconceptions/pages/misconception/misconception.page.store';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

@Component({
    templateUrl: './misconception.page.component.html',
    styleUrls: ['./misconception.page.component.scss'],
    providers: [MisconceptionPageStore],
})
export class MisconceptionPageComponent
    extends BaseComponent
    implements OnInit
{
    misconceptionState$ = this.misconceptionDetailStore.misconceptionState$;
    liked$ = this.misconceptionDetailStore.liked$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Fehlvorstellung konnten wir leider nicht finden.',
        },
    };

    constructor(
        private misconceptionDetailStore: MisconceptionPageStore,
        private router: Router,
        private route: ActivatedRoute
    ) {
        super();
    }

    onAttach() {
        super.onAttach();
        this.misconceptionDetailStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.misconceptionDetailStore.onDetach();
    }

    onLikedChange(val: boolean) {
        this.misconceptionDetailStore.patchState({ liked: val });
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) => params.get('misconception_id'))
            )
            .subscribe((misconceptionId) =>
                this.misconceptionDetailStore.loadMisconception(misconceptionId)
            );
    }
}
