import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadMisconceptions,
    MisconceptionsState,
} from 'app/misconceptions/shared/operators/misconception.operators';

interface MisconceptionsPageState {
    misconceptionsState: MisconceptionsState;
}

export const initialState: MisconceptionsPageState = {
    misconceptionsState: { misconceptions: [], error: null, loading: true },
};

@Injectable()
export class MisconceptionsPageStore extends BaseComponentStore<MisconceptionsPageState> {
    principal$ = this.store.select((s) => s.auth.principal);

    public readonly misconceptionsState$ = this.select(
        (state) => state.misconceptionsState
    );
    public readonly misconceptions$ = this.select(
        (state) => state.misconceptionsState.misconceptions
    );

    constructor(private store: Store<SubatoState>) {
        super(initialState);
    }

    readonly patchMisconceptions = this.updaterFor('misconceptionsState');

    readonly loadMisconceptions = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(loadMisconceptions((s) => this.patchMisconceptions(s)))
    );
}
