import { Observable } from 'rxjs';
import { DistinctHelper } from 'app/shared/misc/helper/distinct.helper';
import { map } from 'rxjs/operators';
import { Injectable } from '@angular/core';
import { CompetenceService } from 'app/misconceptions/services/competence.service';
import { CompetenceGroup } from 'app/misconceptions/models/competence-group';
import { Competence } from 'app/misconceptions/models/competence';

@Injectable({
    providedIn: 'root',
})
export class CompetenceGroupService {
    constructor(private competenceService: CompetenceService) {}

    public getAll(): /*filter?: CourseFilter,*/
    Observable<CompetenceGroup[]> {
        return this.competenceService
            .getAll()
            .pipe(
                map((competences) => this.createCompetenceGroups(competences))
            );
    }

    private createCompetenceGroups(competences: Competence[]) {
        let concepts = competences.map((course) => course.concepts).flat();
        let distinctConcepts = DistinctHelper.distinct(
            concepts,
            (program) => program.id
        );
        let competenceGroups: CompetenceGroup[] = distinctConcepts.map(
            (concept) => {
                return {
                    concept: concept,
                    competences: competences.filter((course) =>
                        course.concepts.some((m) => m.id == concept.id)
                    ),
                    /*.sort(
                        (a, b) =>
                            a.moduleForProgram(concept).scheduledTerm -
                            b.moduleForProgram(concept).scheduledTerm
                    )*/
                };
            }
        );
        //.sort((a, b) => a.program.name.localeCompare(b.program.name));

        var competencesWithoutConcept = competences.filter(
            (c) => c.concepts.length == 0
        );
        if (competencesWithoutConcept.length != 0) {
            competenceGroups.splice(0, 0, {
                concept: null,
                competences: competencesWithoutConcept,
                //description: 'Kurse ohne Bezug zu einem Studiengang',
            });
        }

        return competenceGroups;
    }
}
