import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { EMPTY } from 'rxjs';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EditTermPageStore } from 'app/modules/pages/term-admin/edit-term/edit-term.page.store';
import { CreateTermDto } from 'app/modules/dto/create-term.dto';

@Component({
    templateUrl: './edit-term.page.component.html',
    styleUrls: ['./edit-term.page.component.scss'],
    providers: [EditTermPageStore],
})
export class EditTermPageComponent extends BaseComponent implements OnInit {
    termState$ = this.editTermStore.termState$;
    saving$ = this.editTermStore.saving$;
    validationError$ = this.editTermStore.validationError$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Dieses Semester konnten wir leider nicht finden.',
        },
    };

    constructor(
        private editTermStore: EditTermPageStore,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) => ParamHelper.parseIdOrThrow(params, 'term_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.editTermStore.patchTerm({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((termId) => this.editTermStore.loadTerm(termId));
    }

    saveTerm(dto: CreateTermDto) {
        this.editTermStore.updateTerm(dto);
    }
}
