import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { FormComponent } from 'app/shared/validation/form.component';
import { Iso8601Helper } from 'app/shared/misc/helper/iso8601.helper';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { CreateTermDto } from 'app/modules/dto/create-term.dto';
import { Term } from 'app/modules/models/term';
import { TermService } from 'app/modules/services/term.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ValidationError } from 'app/shared/error/error';

@Component({
    selector: 'sb-term-form',
    templateUrl: './term-form.component.html',
    styleUrls: ['./term-form.component.scss'],
})
export class TermFormComponent
    extends FormComponent
    implements OnInit, OnChanges
{
    @Input() term: Term;
    @Input() validationError: ValidationError;
    @Input() saving: boolean = false;

    @Output() onSave = new EventEmitter<CreateTermDto>();

    termForm: UntypedFormGroup;

    constructor(
        private termService: TermService,
        private router: Router,
        private route: ActivatedRoute,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    createForm(term?: Term) {
        this.termForm = this.fb.group({
            name: [term?.name ?? '', Validators.required],
            lecturesStartAt: [term?.lecturesStartAt ?? ''],
            lecturesEndAt: [term?.lecturesEndAt ?? ''],
        });
    }

    ngOnInit(): void {
        this.createForm(this.term);
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('term')) {
            this.createForm(this.term);
        }

        if (changes.hasOwnProperty('validationError')) {
            this.errors = this.getValidationErrors(this.validationError);
        }
    }

    save() {
        this.saving = true;
        let dto: CreateTermDto = {
            name: this.termForm.get('name').value,
            lecturesStartAt: Iso8601Helper.toString(
                this.termForm.get('lecturesStartAt').value,
                true
            ),
            lecturesEndAt: Iso8601Helper.toString(
                this.termForm.get('lecturesEndAt').value,
                true
            ),
        };

        this.onSave.emit(dto);
    }
}
