import { Component, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { map, takeUntil } from 'rxjs/operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

@Component({
    templateUrl: './terms.page.component.html',
    styleUrls: ['./terms.page.component.scss'],
})
export class TermsPageComponent extends BaseComponent implements OnInit {
    isSuperAdmin$ = this.store.select(
        (s) => s.auth.principal != null && s.auth.principal.hasSuperAdminRole()
    );

    actions$ = this.isSuperAdmin$.pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map((isSuperAdmin) => this.buildActions(isSuperAdmin))
    );

    constructor(private store: Store<SubatoState>) {
        super();
    }

    ngOnInit(): void {}

    buildActions(isSuperAdmin: boolean): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (isSuperAdmin) {
            actions.push({
                name: 'Neues Semester',
                icon: 'plus-outline',
                link: {
                    commands: ['new'],
                },
                status: 'primary',
            });
        }

        return actions;
    }
}
