import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { ProgramModule } from '../models/program-module';

@Injectable({
    providedIn: 'root',
})
export class ProgramModuleService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getAllForTerm(
        termId?: number,
        query?: string
    ): Observable<ProgramModule[]> {
        let params = new HttpParams();
        if (termId != null) {
            params = params.set('termId', termId);
        }

        if (query != null) {
            params = params.set('q', query);
        }

        return this.http
            .get<ProgramModule[]>(`${environment.backendUrl}/module`, {
                params: params,
            })
            .pipe(
                map((programModules) =>
                    programModules.map((ci) => new ProgramModule(ci))
                )
            );
    }
}
