import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { Term } from '../models/term';
import { map } from 'rxjs/operators';
import { BaseService } from 'app/shared/misc/base.service';
import { CreateTermDto } from '../dto/create-term.dto';

@Injectable({
    providedIn: 'root',
})
export class TermService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<Term> {
        return this.http
            .get<Term>(`${environment.backendUrl}/term/${id}`)
            .pipe(map((term) => new Term(term)));
    }

    getAll(): Observable<Term[]> {
        return this.http
            .get<Term[]>(`${environment.backendUrl}/term`)
            .pipe(map((terms) => terms.map((term) => new Term(term))));
    }

    create(dto: CreateTermDto): Observable<Term> {
        return this.http.post<Term>(`${environment.backendUrl}/term`, dto).pipe(
            map((pool) => {
                return new Term(pool);
            })
        );
    }

    update(termId: number, dto: CreateTermDto): Observable<Term> {
        return this.http
            .put<Term>(`${environment.backendUrl}/term/${termId}`, dto)
            .pipe(
                map((term) => {
                    return new Term(term);
                })
            );
    }

    finishTerm(termId: number): Observable<void> {
        return this.http.post<void>(
            `${environment.backendUrl}/term/${termId}/finish`,
            {}
        );
    }
}
