import { Component, Input, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { filter, map, take } from 'rxjs/operators';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { TermSelect } from 'app/modules/store/actions';

@Component({
    selector: 'sb-term-select',
    templateUrl: './term-select.component.html',
    styleUrls: ['./term-select.component.scss'],
})
export class TermSelectComponent extends BaseComponent implements OnInit {
    selectedTerm$ = this.store
        .select((s) => s.modules.currentTerm)
        .pipe(filter((currentTerm) => currentTerm != null));

    terms$ = this.store.select((s) => s.modules.terms);

    @Input()
    style: string | null;

    @Input()
    class: string | null;

    constructor(private store: Store<SubatoState>, private router: Router) {
        super();
    }

    ngOnInit(): void {}

    setSelectedTerm(termId: number) {
        this.terms$
            .pipe(
                take(1),
                map((terms) => terms.find((t) => t.id === termId))
            )
            .subscribe((term) =>
                this.store.dispatch(new TermSelect({ term: term }))
            );
    }
}
