import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';
import {
    concatMap,
    filter,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { TermService } from '../services/term.service';
import { ModulesActions, TermChange, TermsChange, TermSelect } from './actions';
import { Router } from '@angular/router';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';

@Injectable()
export class ModulesEffects {
    loadTerms$ = createEffect(() => {
        return this.actions$.pipe(
            ofType(ModulesActions.LoadTerms),
            switchMap(() => this.termService.getAll()),
            concatMap((terms) => {
                let term = null;

                if (terms.length != 0) {
                    let termId =
                        localStorage.getItem('termId') != null
                            ? parseInt(localStorage.getItem('termId'))
                            : null;

                    if (termId != null) {
                        term = terms.find((t) => t.id == termId);
                    }

                    term = term ?? terms[0];
                }

                return [
                    new TermsChange({ terms: terms }),
                    new TermChange({ term: term }),
                ];
            })
        );
    });

    persistTerm$ = createEffect(
        () => {
            return this.actions$.pipe(
                ofType(ModulesActions.TermSelect),
                tap((a: TermSelect) => {
                    localStorage.setItem(
                        'termId',
                        a.payload.term?.id?.toString()
                    );
                })
            );
        },
        { dispatch: false }
    );

    resetView = createEffect(
        () => {
            return this.actions$.pipe(
                ofType(ModulesActions.TermSelect),
                withLatestFrom(this.store),
                filter(
                    ([a, state]: [TermSelect, SubatoState]) =>
                        state.location.currentUrl != '/course'
                ),
                tap(([a, state]) => {
                    if (state.auth.principal) {
                        this.router.navigate(['/', 'dashboard']);
                    } else {
                        this.router.navigate(['/', 'course']);
                    }
                })
            );
        },
        { dispatch: false }
    );

    constructor(
        private actions$: Actions,
        private termService: TermService,
        private store: Store<SubatoState>,
        private router: Router
    ) {}
}
