import { Component, OnInit } from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { ProfileStore } from 'app/profile/profile-store';
import { UpdateProfileDto } from 'app/shared/user/update-profile.dto';
import { filter } from 'rxjs/operators';
import { FormComponent } from 'app/shared/validation/form.component';
import { environment } from 'environments/environment';
import { DocsService } from 'app/shared/misc/docs.service';

@Component({
    selector: 'sb-profile-form',
    templateUrl: './profile-form.component.html',
    styleUrls: ['./profile-form.component.scss'],
})
export class ProfileFormComponent extends FormComponent implements OnInit {
    profileForm: UntypedFormGroup;
    saving$ = this.profileStore.saving$;
    docsUrls = this.docsService.urls;

    constructor(
        private fb: UntypedFormBuilder,
        private profileStore: ProfileStore,
        private docsService: DocsService
    ) {
        super();
    }

    ngOnInit(): void {
        this.profileStore.principal$
            .pipe(filter((principal) => principal != null))
            .subscribe(
                (principal) => (this.profileForm = this.createForm(principal))
            );
    }

    createForm(principal?: SubatoPrincipal) {
        return this.fb.group({
            consentedToCollection: [principal?.consentedToCollection ?? false],
        });
    }

    save() {
        let dto: UpdateProfileDto = {
            consentedToCollection: this.profileForm.get('consentedToCollection')
                .value,
        };
        this.profileStore.updateProfile(dto);
    }
}
