import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
    ViewChild,
} from '@angular/core';
import { Router } from '@angular/router';
import { NbAutocompleteDirective } from '@nebular/theme';
import { BaseComponent } from 'app/shared/misc/base.component';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SearchStore } from 'app/search/shared/components/search-form/search.store';
import { Item } from 'app/search/models/item';

@Component({
    selector: 'sb-search-form',
    templateUrl: './search-form.component.html',
    styleUrls: ['./search-form.component.scss'],
    providers: [SearchStore],
})
export class SearchFormComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() courseId: number;
    @ViewChild(NbAutocompleteDirective)
    autocompleteDirective: NbAutocompleteDirective<any>;

    principal$ = this.searchStore.principal$;
    itemGroups$ = this.searchStore.itemGroups$;
    searching$ = this.searchStore.searching$;
    error$ = this.searchStore.error$;
    query$ = this.searchStore.query$;

    constructor(
        private globalErrorHandler: GlobalErrorHandler,
        private searchStore: SearchStore,
        private router: Router
    ) {
        super();
    }

    ngOnInit(): void {}

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('courseId')) {
            this.searchStore.patchState({ courseId: this.courseId });
        }
    }

    onInputChange(query: string | Item) {
        if (typeof query == 'string') {
            this.searchStore.patchState({ query: query });
        }
    }

    displayFunction(option: string | Item) {
        return typeof option == 'string' ? option : '';
    }

    onEnter(e) {
        /*
        if(!this.selectedOption) {
          this.router.navigate(['/', 'search'], {
            queryParams: {
              q: this.query
            }
          });
          this.autocompleteDirective.hide();
        }
        */
    }

    onSelectionChange(selectedValue: string | Item) {
        if (selectedValue == null || typeof selectedValue == 'string') return;
        this.router.navigate([selectedValue.url()], {
            queryParams: selectedValue.queryParams(),
        });
        this.autocompleteDirective.hide();
    }
}
