import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';
import { ActivationEnd, NavigationEnd, Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { filter, map, withLatestFrom } from 'rxjs/operators';
import {
    BackNavigationActions,
    GoBack,
    Navigated,
    NavigationFinished,
    NavigationStarted,
} from './actions';
import { Location, LocationStrategy } from '@angular/common';

@Injectable()
export class BackNavigationEffects {
    goBack$ = createEffect(
        () => {
            return this.actions$.pipe(
                ofType(BackNavigationActions.GoBack),
                withLatestFrom(this.store),
                map(([a, state]: [GoBack, SubatoState]) => {
                    if (state.backNav.history.length > 0) {
                        this.location.back();
                    } else {
                        this.router.navigate(a.payload.defaultRoute ?? ['/']);
                    }
                })
            );
        },
        { dispatch: false }
    );

    constructor(
        private actions$: Actions,
        private router: Router,
        private location: Location,
        private locationStrategy: LocationStrategy,
        private store: Store<SubatoState>
    ) {
        this.router.events
            .pipe(filter((e) => e instanceof ActivationEnd))
            .subscribe((e) => {
                store.dispatch(new NavigationFinished());
            });

        this.router.events.subscribe((event) => {
            if (event instanceof NavigationEnd) {
                store.dispatch(new Navigated({ url: event.urlAfterRedirects }));
            }
        });

        locationStrategy.onPopState(() => {
            store.dispatch(new NavigationStarted());
        });
    }
}
