import { BackNavigationActions, BackNavigationActionsUnion } from './actions';
import { Action } from '@ngrx/store';

export interface BackNavState {
    justNavigatedBack: boolean;
    history: string[];
}

export const initialState: BackNavState = {
    justNavigatedBack: false,
    history: [],
};

export function backNavigationReducer(
    state = initialState,
    action: Action
): BackNavState {
    const a = action as BackNavigationActionsUnion;

    switch (a.type) {
        case BackNavigationActions.NavigationStarted: {
            return {
                ...state,
                justNavigatedBack: true,
            };
        }

        case BackNavigationActions.NavigationFinished: {
            return {
                ...state,
                justNavigatedBack: false,
            };
        }

        case BackNavigationActions.Navigated: {
            return {
                ...state,
                history: state.history.concat([a.payload.url]),
            };
        }

        case BackNavigationActions.GoBack: {
            return {
                ...state,
                history: state.history.slice(0, state.history.length - 1),
            };
        }

        default: {
            return state;
        }
    }
}
