import { Component, Input, OnInit } from '@angular/core';
import { Route, Router } from '@angular/router';
import { Breadcrumb } from './breadcrumb';

@Component({
    selector: 'sb-breadcrumbs',
    templateUrl: './breadcrumbs.component.html',
    styleUrls: ['./breadcrumbs.component.scss'],
})
export class BreadcrumbsComponent implements OnInit {
    @Input() breadcrumbs: Breadcrumb[] = [];

    constructor(private router: Router) {}

    ngOnInit(): void {}

    instantiatePath(path: string, breadcrumb: Breadcrumb) {
        let p = path;
        let params = breadcrumb.params ?? {};

        for (let key of Object.keys(params)) {
            p = p.replace(':' + key, params[key]);
        }

        return p;
    }

    getRoute(breadcrumb: Breadcrumb) {
        let selectedRoute = this.getRoutes().filter(
            (x) => x.data?.breadcrumb?.alias == breadcrumb.id
        )[0];
        return selectedRoute?.path
            ? `/${this.instantiatePath(selectedRoute.path, breadcrumb)}`
            : null;
    }

    getLabel(breadcrumb: Breadcrumb) {
        let selectedRoute = this.getRoutes().filter(
            (x) => x.data?.breadcrumb?.alias == breadcrumb.id
        )[0];
        let breadcrumbData = selectedRoute?.data?.breadcrumb;

        if (breadcrumbData) {
            if (typeof breadcrumbData.label === 'string') {
                return breadcrumbData.label;
            }

            if (typeof breadcrumbData.label === 'function' && breadcrumb.ctx) {
                return breadcrumbData.label(breadcrumb.ctx);
            }
        }

        return breadcrumb.id;
    }

    getRoutes(): Route[] {
        let routes = [];
        this.getAllRoutes(routes, '', this.router.config);
        return routes;
    }

    getAllRoutes(routes: Route[], parent: String, config: Route[]) {
        for (let i = 0; i < config.length; i++) {
            const route = config[i];
            routes.push(route);

            if (route.children) {
                const currentPath = route.path
                    ? parent + '/' + route.path
                    : parent;
                this.getAllRoutes(routes, currentPath, route.children);
            }

            // todo: can be removed because we use no lazy loading anymore
            if ('_loadedRoutes' in route) {
                this.getAllRoutes(routes, '', route['_loadedRoutes']);
            }
        }
    }
}
