import { Component, forwardRef, OnInit } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { BaseComponent } from '../../../misc/base.component';
import { environment } from 'environments/environment';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { DocsService } from 'app/shared/misc/docs.service';

@Component({
    selector: 'sb-md-editor',
    templateUrl: './md-editor.component.html',
    styleUrls: ['./md-editor.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => MdEditorComponent),
            multi: true,
        },
    ],
})
export class MdEditorComponent
    extends BaseComponent
    implements OnInit, ControlValueAccessor
{
    docsUrls = this.docsService.urls;
    darkMode$: Observable<boolean> = this.store.select(
        (s) => s.layout.darkMode
    );
    content: string;

    constructor(
        private store: Store<SubatoState>,
        private docsService: DocsService
    ) {
        super();
    }

    ngOnInit(): void {}

    onContentChanged(content: string) {
        this.content = content;
        this.onChange(content);
    }

    // Placeholder functions for the ControlValueAccessor interface
    private onChange: (value: any) => void = () => {};
    private onTouched: () => void = () => {};

    // ControlValueAccessor interface methods
    writeValue(value: any): void {
        // Set the initial value when the form control is initialized
        this.content = value;
    }

    registerOnChange(fn: (value: any) => void): void {
        // Register the callback function to update the form control value
        this.onChange = fn;
    }

    registerOnTouched(fn: () => void): void {
        // Register the callback function for touch events
        this.onTouched = fn;
    }

    setDisabledState(isDisabled: boolean): void {
        // Optional: Implement this method if you want to disable the component
        // based on the form control's disabled state
    }
}
