import { Directive, Renderer2 } from '@angular/core';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { interval, Subject } from 'rxjs';
import { take, throttle } from 'rxjs/operators';
import { CodemirrorHelper } from '../helper/codemirror.helper';

@Directive({
    selector: 'ngx-codemirror[resizable]',
})
export class ResizableCodemirrorDirective {
    private resized: Subject<void> = new Subject<void>();

    constructor(
        private component: CodemirrorComponent,
        private renderer: Renderer2
    ) {
        CodemirrorHelper.waitForEditor(component)
            .pipe(take(1))
            .subscribe(() => {
                var wrapper = component.codeMirror.getWrapperElement();
                wrapper.classList.add('resizable');

                new MutationObserver(() => this.resized.next()).observe(
                    wrapper,
                    {
                        attributes: true,
                        attributeFilter: ['style'],
                    }
                );
            });

        this.resized.pipe(throttle(() => interval(100))).subscribe(() => {
            component.codeMirror.refresh();
        });
    }
}
