import {
    Component,
    EventEmitter,
    forwardRef,
    Input,
    OnInit,
    Output,
} from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

@Component({
    selector: 'sb-dropzone',
    templateUrl: './dropzone.component.html',
    styleUrls: ['./dropzone.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => DropzoneComponent),
            multi: true,
        },
    ],
})
export class DropzoneComponent implements OnInit, ControlValueAccessor {
    @Input() file: File | null;
    @Output() fileChanged = new EventEmitter<File>();

    ngOnInit(): void {}

    fileSelected(event) {
        this.file = event.addedFiles[0];
        this.onChange(this.file);
        this.fileChanged.emit(this.file);
    }

    removeFile() {
        this.file = null;
        this.onChange(this.file);
        this.fileChanged.emit(null);
    }

    // Placeholder functions for the ControlValueAccessor interface
    private onChange: (value: any) => void = () => {};
    private onTouched: () => void = () => {};

    // ControlValueAccessor interface methods
    writeValue(value: any): void {
        // Set the initial value when the form control is initialized
        if (value instanceof File) {
            this.file = value;
        } else {
            this.file = null;
        }
    }

    registerOnChange(fn: (value: any) => void): void {
        // Register the callback function to update the form control value
        this.onChange = fn;
    }

    registerOnTouched(fn: () => void): void {
        // Register the callback function for touch events
        this.onTouched = fn;
    }

    setDisabledState(isDisabled: boolean): void {
        // Optional: Implement this method if you want to disable the component
        // based on the form control's disabled state
    }
}
