import { Component, Input, OnInit } from '@angular/core';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { ErrorContainerConfig } from '../error-container-config';
import { ErrorConfig } from '../error-config';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import {
    isAccessDeniedError,
    isNotFoundError,
    SubatoError,
} from 'app/shared/error/error';

@Component({
    selector: 'sb-error-container',
    templateUrl: './error-container.component.html',
    styleUrls: ['./error-container.component.scss'],
})
export class ErrorContainerComponent implements OnInit {
    @Input() error: SubatoError | null;
    @Input() config: ErrorContainerConfig | null;

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    constructor(private store: Store<SubatoState>) {}

    ngOnInit(): void {}

    getConfig(): ErrorConfig {
        if (isNotFoundError(this.error)) {
            return this.notFoundConfig();
        }

        if (isAccessDeniedError(this.error)) {
            return this.accessDeniedConfig();
        }

        return this.internalConfig();
    }

    notFoundConfig(): ErrorConfig {
        let defaultConfig: ErrorConfig = {
            title: 'Hier ist nichts.',
            message: 'Diesen Eintrag konnten wir leider nicht finden.',
            ...(this.config?.['notFound'] ?? {}),
        };

        return defaultConfig;
    }

    internalConfig(): ErrorConfig {
        let defaultConfig: ErrorConfig = {
            title: 'Hoppla.',
            message:
                'Irgendetwas hat nicht geklappt, bitte versuche es erneut und wende dich an uns, wenn das Problem weiter besteht.',
            ...(this.config?.['internal'] ?? {}),
        };

        return defaultConfig;
    }

    accessDeniedConfig(): ErrorConfig {
        let defaultConfig: ErrorConfig = {
            title: 'You Shall Not Pass.',
            message:
                'Darauf darfst du nicht zugreifen. Wende dich an uns, wenn du denkst, dass das ein Fehler ist.',
            ...(this.config?.['accessDenied'] ?? {}),
        };

        return defaultConfig;
    }
}
