enum ErrorType {
    LOCAL,
    SERVER,
    NOT_FOUND,
    VALIDATION,
    DANGEROUS_ACTION,
    ACCESS_DENIED,
}

export interface SubatoError {
    type: ErrorType;

    getMessage(): string;
}

export class LocalError implements SubatoError {
    type = ErrorType.LOCAL;

    constructor(public cause: any) {}

    public getMessage(): string {
        return `Unbekannter Fehler`;
    }
}

export class NotFoundError implements SubatoError {
    type = ErrorType.NOT_FOUND;

    constructor(public reason: string) {}

    public getMessage(): string {
        return this.reason;
    }
}

export class ServerError implements SubatoError {
    type = ErrorType.SERVER;

    constructor(public reason: string) {}

    public getMessage(): string {
        return this.reason;
    }
}

export class ValidationError implements SubatoError {
    type = ErrorType.VALIDATION;

    constructor(public reason: string, public failures: any[]) {}

    public getMessage(): string {
        return this.reason;
    }
}

export class DangerousActionError implements SubatoError {
    type = ErrorType.DANGEROUS_ACTION;

    constructor(
        public reason: string,
        public description: any,
        public suggestion: string
    ) {}

    public getMessage(): string {
        return this.reason;
    }
}

export class AccessDeniedError implements SubatoError {
    type = ErrorType.ACCESS_DENIED;

    constructor(public reason: string) {}

    public getMessage(): string {
        return this.reason;
    }
}

export function isLocalError(error: SubatoError): error is LocalError {
    return error.type == ErrorType.LOCAL;
}

export function isValidationError(
    error: SubatoError
): error is ValidationError {
    return error.type == ErrorType.VALIDATION;
}

export function isNotFoundError(error: SubatoError): error is NotFoundError {
    return error.type == ErrorType.NOT_FOUND;
}

export function isServerError(error: SubatoError): error is ServerError {
    return error.type == ErrorType.SERVER;
}

export function isDangerousAction(
    error: SubatoError
): error is DangerousActionError {
    return error.type == ErrorType.DANGEROUS_ACTION;
}

export function isAccessDeniedError(
    error: SubatoError
): error is AccessDeniedError {
    return error.type == ErrorType.ACCESS_DENIED;
}
