import { Injectable } from '@angular/core';

import { HttpErrorResponse } from '@angular/common/http';
import { throwError } from 'rxjs';
import { ErrorResponse } from 'app/shared/error/backend-response';
import { LocalError, SubatoError } from 'app/shared/error/error';
import { createSubatoError } from 'app/shared/error/subato-error-factory';

@Injectable({
    providedIn: 'root',
})
export class GlobalErrorHandler {
    handleError(error: any) {
        if (error instanceof HttpErrorResponse) {
            if (error.status == 0) {
                // no connection to backend
                return throwError(new LocalError(error));
            }

            let errorResponse = error.error as ErrorResponse;
            let subatoError = createSubatoError(errorResponse);
            console.error(
                '[Subato API]',
                errorResponse.statusCode,
                errorResponse.reasonPhrase,
                errorResponse.data
            );

            return throwError(subatoError);
        } else {
            if (error.type != undefined) {
                // error is already a SubatoError
                return throwError(error);
            }

            console.error(error?.stack ?? error);
            return throwError(new LocalError(error));
        }
    }

    format(prefix: string, err: SubatoError): string {
        let message = err.getMessage();
        return `${prefix ?? 'Fehler'}: ${message}`;
    }
}
