import { ErrorCode, ErrorResponse } from 'app/shared/error/backend-response';
import {
    AccessDeniedError,
    DangerousActionError,
    NotFoundError,
    ServerError,
    SubatoError,
    ValidationError,
} from 'app/shared/error/error';

export function createSubatoError(errorResponse: ErrorResponse): SubatoError {
    if (errorResponse.statusCode == 404) {
        return new NotFoundError(errorResponse.reasonPhrase);
    }

    if (errorResponse.errorCode == ErrorCode.VALIDATION_FAILURE) {
        return new ValidationError(
            errorResponse.reasonPhrase,
            errorResponse.data?.failures ?? []
        );
    }

    if (errorResponse.errorCode == ErrorCode.DANGEROUS_ACTION) {
        return new DangerousActionError(
            errorResponse.reasonPhrase,
            errorResponse.data.description,
            errorResponse.data.suggestion
        );
    }

    if (errorResponse.statusCode == 403 || errorResponse.statusCode == 401) {
        return new AccessDeniedError(errorResponse.reasonPhrase);
    }

    return new ServerError(errorResponse.reasonPhrase);
}
