import { Injectable } from '@angular/core';

import { Actions } from '@ngrx/effects';
import { NavigationCancel, NavigationStart, Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { filter, map } from 'rxjs/operators';
import { Navigated, UndoNavigation } from './actions';

@Injectable()
export class LocationEffects {
    constructor(
        private actions$: Actions,
        private router: Router,
        private store: Store<SubatoState>
    ) {
        this.router.events
            .pipe(
                filter((event) => event instanceof NavigationStart),
                map((event) => event as NavigationStart)
            )
            .subscribe((event) => {
                store.dispatch(new Navigated({ currentUrl: event.url }));
            });

        this.router.events
            .pipe(
                filter((event) => event instanceof NavigationCancel),
                map((event) => event as NavigationCancel)
            )
            .subscribe((_) => {
                store.dispatch(new UndoNavigation());
            });
    }
}
