import { LocationActions, LocationActionsUnion } from './actions';
import { Action } from '@ngrx/store';

export interface LocationState {
    prevUrl: string | null;
    currentUrl: string | null;
    history: string[];
}

export const initialState: LocationState = {
    prevUrl: null,
    currentUrl: null,
    history: [],
};

export function locationReducer(
    state = initialState,
    action: Action
): LocationState {
    const a = action as LocationActionsUnion;

    switch (a.type) {
        case LocationActions.Navigated: {
            return {
                ...state,
                prevUrl: state.currentUrl,
                currentUrl: a.payload.currentUrl,
                history: state.history.concat([state.currentUrl]),
            };
        }

        case LocationActions.UndoNavigation: {
            return {
                ...state,
                currentUrl: state.prevUrl,
                history: state.history.slice(0, state.history.length - 2),
            };
        }

        default: {
            return state;
        }
    }
}
