import {
    Directive,
    HostBinding,
    HostListener,
    Input,
    OnChanges,
    OnDestroy,
    SimpleChanges,
} from '@angular/core';
import {
    ActivatedRoute,
    NavigationEnd,
    Params,
    Router,
    UrlTree,
} from '@angular/router';
import { Subject, Subscription } from 'rxjs';
import { LocationStrategy } from '@angular/common';

// copy pasterino from:
// https://github.com/angular/angular/blob/5205db5564f21c214a96000e018a08f3de7d44d8/packages/router/src/directives/router_link.ts#L201-L207
// reason: pending feature request in https://github.com/angular/angular/issues/34468

export interface CustomRouterLink {
    commands: any[];
    queryParams?: Params;
}

@Directive({
    selector: '[customRouterLink]',
})
export class CustomRouterLinkDirective implements OnChanges, OnDestroy {
    @Input() customRouterLink: CustomRouterLink | null;
    private subscription: Subscription;

    // the url displayed on the anchor element.
    // TODO(issue/24571): remove '!'.
    @HostBinding() href!: string;

    constructor(
        private router: Router,
        private route: ActivatedRoute,
        private locationStrategy: LocationStrategy
    ) {
        this.subscription = router.events.subscribe((s: any) => {
            if (s instanceof NavigationEnd) {
                this.updateTargetUrlAndHref();
            }
        });
    }

    /** @internal */
    onChanges = new Subject<CustomRouterLinkDirective>();

    /** @nodoc */
    ngOnChanges(changes: SimpleChanges): any {
        this.updateTargetUrlAndHref();
        this.onChanges.next(this);
    }

    /** @nodoc */
    ngOnDestroy(): any {
        this.subscription.unsubscribe();
    }

    private updateTargetUrlAndHref(): void {
        if (this.customRouterLink != null) {
            this.href = this.locationStrategy.prepareExternalUrl(
                this.router.serializeUrl(this.urlTree)
            );
        }
    }

    /** @nodoc */
    @HostListener('click', ['$event'])
    onClick(event: MouseEvent): boolean {
        event.preventDefault();
        const extras = {
            //skipLocationChange: attrBoolValue(this.skipLocationChange),
            //replaceUrl: attrBoolValue(this.replaceUrl),
            //state: this.state,
        };
        let urlTree = this.urlTree;
        if (urlTree != null) {
            this.router.navigateByUrl(urlTree, extras);
        }
        return true;
    }

    get urlTree(): UrlTree {
        return this.customRouterLink != null
            ? this.router.createUrlTree(this.customRouterLink.commands, {
                  relativeTo: this.route,
                  queryParams: this.customRouterLink.queryParams ?? [],
                  //fragment: this.fragment,
                  //queryParamsHandling: this.queryParamsHandling,
                  //preserveFragment: attrBoolValue(this.preserveFragment),
              })
            : null;
    }
}

function attrBoolValue(s: any): boolean {
    return s === '' || !!s;
}
