import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';

export interface ManualUrls {
    base: string;
    dataCollection: string;
    intellij: string;
    tasks: string;
    permission: string;
    hds: string;
    course: string;
    aor: string;
    assessment: string;
}

export interface StefUrls {
    base: string;
    syntaxHighlight: string;
}

export interface DocsUrls {
    base: string;
    support: string;
    impress: string;
    stef: StefUrls;
    manual: ManualUrls;
}

@Injectable({
    providedIn: 'root',
})
@Injectable()
export class DocsService {
    urls: DocsUrls;

    constructor() {
        this.urls = {
            base: this.getDocsUrl(''),
            support: this.getDocsUrl('contact'),
            impress: this.getDocsUrl('impress'),
            stef: {
                base: this.getDocsUrl('stef'),
                syntaxHighlight: this.getDocsUrl(
                    'stef/text#syntax-highlighting'
                ),
            },
            manual: {
                base: this.getManualUrl(''),
                dataCollection: this.getManualUrl('plugins/research'),
                intellij: this.getManualUrl('plugins/intellij'),
                tasks: this.getManualUrl('tasks'),
                permission: this.getManualUrl('permissions'),
                hds: this.getManualUrl('login'),
                course: this.getManualUrl('course'),
                aor: this.getManualUrl('aor'),
                assessment: this.getManualUrl('assessment'),
            },
        };
    }

    private getDocsUrl(path: string) {
        return `${environment.docsUrl}/${path}`;
    }

    private getManualUrl(path: string) {
        return this.getDocsUrl(`docs/${path}`);
    }
}
