import { Observable, OperatorFunction } from 'rxjs';
import { debounceTime, distinctUntilChanged, startWith } from 'rxjs/operators';

// if we apply debounceTime, then the observable emits after the given debounce time.
// in some situations, we need to directly get the initial value and only debounce after subsequent updates
// the following operator function can be used to achieve this
export function debounceWithInitialValue<T>(
    debounceTimeMs: number,
    initialValue: T
): OperatorFunction<T, T> {
    return (source: Observable<T>): Observable<T> => {
        return source.pipe(
            debounceTime(debounceTimeMs),
            startWith(initialValue),
            distinctUntilChanged()
        );
    };
}
