import {
    Component,
    ContentChild,
    Directive,
    Input,
    OnInit,
    TemplateRef,
} from '@angular/core';
import { CustomRouterLink } from 'app/shared/misc/directives/custom-link/custom-router-link.directive';

@Directive({
    selector: '[titleTemplate]',
})
export class TitleTemplateDirective {
    constructor(public templateRef: TemplateRef<any>) {}
}

@Directive({
    selector: '[subtitleTemplate]',
})
export class SubtitleTemplateDirective {
    constructor(public templateRef: TemplateRef<any>) {}
}

@Directive({
    selector: '[detailsTemplate]',
})
export class DetailsTemplateDirective {
    constructor(public templateRef: TemplateRef<any>) {}
}

export interface ExternalLink {
    href: string;
    download?: boolean;
}

export interface PageHeaderAction {
    icon: string;
    name: string;
    status?: string;
    disabled?: boolean;
    link?: CustomRouterLink;
    externalLink?: ExternalLink;
    click?: any;
    loading?: boolean;
    pos?: 'left' | 'right';
}

@Component({
    selector: 'sb-page-header',
    templateUrl: './page-header.component.html',
    styleUrls: ['./page-header.component.scss'],
})
export class PageHeaderComponent implements OnInit {
    @ContentChild(TitleTemplateDirective)
    titleTemplate?: TitleTemplateDirective;

    @ContentChild(SubtitleTemplateDirective)
    subtitleTemplate: TitleTemplateDirective;

    @ContentChild(DetailsTemplateDirective)
    detailsTemplate: DetailsTemplateDirective;

    @Input() title?: string;
    @Input() subtitle?: string;

    private _actions: PageHeaderAction[] = [];
    leftActions: PageHeaderAction[] = [];
    rightActions: PageHeaderAction[] = [];

    @Input() set actions(actions: PageHeaderAction[]) {
        this._actions = actions;
        this.leftActions = actions.filter(
            (a) => a.pos == 'left' || a.pos == null
        );
        this.rightActions = actions.filter((a) => a.pos == 'right');
    }

    get actions() {
        return this._actions;
    }

    constructor() {}

    ngOnInit(): void {}
}
