import { Pipe, PipeTransform } from '@angular/core';

@Pipe({
    name: 'table_format',
})
export class TableFormatPipe implements PipeTransform {
    transform(inHtml: string): any {
        return this.transformHtml(inHtml);
    }

    private transformHtml(inHtml: string) {
        let parser = new DOMParser();
        let doc = parser.parseFromString(inHtml, 'text/html');
        var tableElements = doc.querySelectorAll('table');

        tableElements.forEach((table) => {
            table.classList.add('table');
            table.querySelectorAll('th[align]').forEach((th) => {
                var alignClass = this.getCssClassForAlign(
                    th.attributes.getNamedItem('align')?.value
                );
                th.classList.add(alignClass);
            });
        });

        return doc.body.innerHTML;
    }

    private getCssClassForAlign(align?: string) {
        if (align == null) {
            return '';
        }

        switch (align) {
            case 'left':
                return 'text-left';
            case 'right':
                return 'text-right';
            case 'center':
                return 'text-center';
        }

        return '';
    }
}
