import { ComponentStore } from '@ngrx/component-store';
import { BehaviorSubject, Observable } from 'rxjs';
import { ComponentWithOptionalLifecycleHooks } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

export class BaseComponentStore<T extends object>
    extends ComponentStore<T>
    implements ComponentWithOptionalLifecycleHooks
{
    private _detached = new BehaviorSubject(false);
    public detached$ = this._detached.asObservable();

    constructor(defaultState: T) {
        super(defaultState);
    }

    updaterFor<K extends keyof T>(key: K) {
        return this.updater((state: T, updatedState: Partial<T[K]>) => ({
            ...state,
            [key]: {
                ...state[key],
                ...updatedState,
            },
        }));
    }

    patchStateFor<K extends keyof T>(
        key: K,
        partialStateOrUpdaterFn:
            | Partial<T[K]>
            | Observable<Partial<T[K]>>
            | ((state: T[K]) => Partial<T[K]>)
    ) {
        if (typeof partialStateOrUpdaterFn === 'function') {
            this.patchState((s) => ({
                ...s,
                [key]: { ...s[key], ...partialStateOrUpdaterFn(s[key]) },
            }));
        } /* else if(partialStateOrUpdaterFn instanceof Observable) {

        } else {
            this.patchState({
                [key]: { ...partialStateOrUpdaterFn }
            });
        }*/
    }

    public onAttach() {
        this._detached.next(false);
    }

    public onDetach() {
        this._detached.next(true);
    }
}
