import {
    Component,
    EventEmitter,
    Input,
    OnInit,
    Output,
    ViewChild,
} from '@angular/core';
import { NameFormat, User } from 'app/shared/user/user';
import { BaseComponent } from 'app/shared/misc/base.component';
import { UserPickerStore } from 'app/shared/user/user-picker/user-picker.store';

@Component({
    selector: 'sb-user-picker',
    templateUrl: './user-picker.component.html',
    styleUrls: ['./user-picker.component.scss'],
    providers: [UserPickerStore],
})
export class UserPickerComponent extends BaseComponent implements OnInit {
    @Output() userSelected = new EventEmitter<User>();
    @Input() readonly: boolean = false;

    @ViewChild('autoInput') inputField;

    selectedUser: User | null = null;

    users$ = this.userPickerStore.users$;
    query$ = this.userPickerStore.query$;
    searching$ = this.userPickerStore.searching$;
    error$ = this.userPickerStore.error$;

    constructor(private userPickerStore: UserPickerStore) {
        super();
    }

    ngOnInit(): void {}

    onInputChange(query: string | User) {
        if (this.selectedUser != null) {
            // if something was selected and the user presses backspace, then the current selection should be resetted
            this.selectedUser = null;
            this.userSelected.emit(null);
            query = '';
        }

        if (typeof query == 'string') {
            this.userPickerStore.patchState({ query: query });
        }
    }

    displayFunction(option: string | User) {
        return typeof option == 'string'
            ? option
            : option.getName(NameFormat.Formal, true);
    }

    onSelectionChange(selectedValue: string | User) {
        if (selectedValue == null || typeof selectedValue == 'string') return;
        this.selectedUser = selectedValue;
        this.userSelected.emit(selectedValue);
    }
}
