import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { Page } from 'app/shared/paging/page';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { User } from './user';
import { UpdateProfileDto } from 'app/shared/user/update-profile.dto';

export interface UserParams {
    q?: string;
    projection?: DataQueryParams;
}

@Injectable({
    providedIn: 'root',
})
export class UserService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<User> {
        return this.http
            .get<User>(`${environment.backendUrl}/user/${id}`)
            .pipe(map((user) => new User(user)));
    }

    findAll(params?: UserParams): Observable<Page<User>> {
        let httpParams = DataQueryParams.toHttpParams(params?.projection);

        if (params?.q != null) {
            httpParams = httpParams.set('q', params.q);
        }

        return this.http
            .get<Page<User>>(`${environment.backendUrl}/user`, {
                params: httpParams,
            })
            .pipe(
                map((page) => new Page<User>(page, (item) => new User(item)))
            );
    }

    updateProfile(userId: number, dto: UpdateProfileDto): Observable<User> {
        return this.http
            .put<User>(`${environment.backendUrl}/user/${userId}/profile`, dto)
            .pipe(map((user) => new User(user)));
    }
}
