import { DegreeProgram } from 'app/modules/models/degree-program';

export enum NameFormat {
    Formal = 'formal',
    Personal = 'personal',
}

export class User {
    id: number;
    loginName: string;
    firstName: string;
    name: string;
    lastName: string;
    mail: string;
    program: DegreeProgram;

    constructor(user: any) {
        this.id = user.id;
        this.loginName = user.loginName;
        this.firstName = user.firstName;
        this.lastName = user.lastName;
        this.name = user.name;
        this.mail = user.mail;
        this.program = user.program ? new DegreeProgram(user.program) : null;
    }

    get hasFullName() {
        return this.firstName && this.lastName;
    }

    getName(style: NameFormat, withLoginName?: boolean) {
        withLoginName = withLoginName != null ? withLoginName : true;

        if (this.hasFullName) {
            let name = this.getFormattedName(style);

            if (this.loginName && withLoginName) {
                return `${name} (${this.loginName})`;
            } else {
                return name;
            }
        } else {
            return this.loginName;
        }
    }

    private getFormattedName(format: NameFormat) {
        switch (format) {
            case NameFormat.Formal:
                return `${this.lastName}, ${this.firstName}`;
            case NameFormat.Personal:
            default:
                return `${this.firstName} ${this.lastName}`;
        }
    }
}
