import { Pipe, PipeTransform } from '@angular/core';

function isNil(value: any): value is null | undefined {
    return value === null || typeof value === 'undefined';
}

function isObject(value: any): boolean {
    return value !== null && typeof value === 'object';
}

function isArray(value: any): boolean {
    return Array.isArray(value);
}

function getProperty(value: { [key: string]: any }, key: string): any {
    if (isNil(value) || !isObject(value)) {
        return undefined;
    }

    const keys: string[] = key.split('.');
    let result: any = value[keys.shift()!];

    for (const key of keys) {
        if (isNil(result) || !isObject(result)) {
            return undefined;
        }

        result = result[key];
    }

    return result;
}

@Pipe({
    name: 'prop',
})
export class PropPipe implements PipeTransform {
    transform(input: any, key: string): any {
        if (!isObject(input) || !key) {
            return input;
        }

        return getProperty(input, key);
    }
}
