import { BaseComponent } from '../misc/base.component';
import { ValidationError } from 'app/shared/error/error';

export interface ValidationEntry {
    property: string;
    message: string;
}

export class FormComponent extends BaseComponent {
    errors: ValidationEntry[] = [];

    constructor() {
        super();
    }

    isInvalid(form, controlName) {
        return (
            ((form.get(controlName).touched || form.get(controlName).dirty) &&
                form.get(controlName).invalid) ||
            this.errors.find((e) => e.property == controlName)
        );
    }

    getStatusForControl(form, controlName) {
        return this.isInvalid(form, controlName) ? 'danger' : 'basic';
    }

    public hasRequiredError(form, controlName) {
        return (
            this.isInvalid(form, controlName) &&
            form.get(controlName).errors?.['required']
        );
    }

    public hasFormError(form, err) {
        return form.errors?.[err];
    }

    public getValidationErrors(error: ValidationError): ValidationEntry[] {
        var validationErrors: ValidationEntry[] = [];
        let failures = error?.failures ?? [];

        for (const failure of failures) {
            validationErrors.push({
                property: failure.fieldName,
                message: failure.errorMessage,
            });
        }
        return validationErrors;
    }
}
