import { TaskInstance } from 'app/exercises/models/task-instance';
import * as moment from 'moment/moment';
import { Iso8601Helper } from 'app/shared/misc/helper/iso8601.helper';

export class TaskInstanceState {
    processed: number;
    total: number;
    errors: number;
    successful: number;
    taskInstance: TaskInstance;

    constructor(dto: any) {
        this.processed = dto.processed;
        this.total = dto.total;
        this.errors = dto.errors;
        this.successful = dto.successful;
        this.taskInstance = dto.taskInstance
            ? new TaskInstance(dto.taskInstance)
            : null;
    }

    get progress() {
        return this.total != 0 ? (this.processed / this.total) * 100 : 100;
    }
}

export enum LogType {
    Info,
    Error,
    Warning,
}

export class BulkEvalLog {
    message: string;
    type: LogType;
    at: moment.Moment;

    constructor(dto: any) {
        this.message = dto.message;
        this.type = dto.type;
        this.at = Iso8601Helper.fromString(dto.at);
    }

    get info() {
        return this.type == LogType.Info;
    }

    get warning() {
        return this.type == LogType.Warning;
    }

    get error() {
        return this.type == LogType.Error;
    }
}

export class BulkEvalState {
    exerciseId: number;
    possiblyOutdated: boolean;
    detached: boolean;
    processed: number;
    total: number;
    errors: number;
    successful: number;
    running: boolean;
    taskInstanceStates: TaskInstanceState[];
    logs: BulkEvalLog[];
    initializedAt: moment.Moment;
    lastUpdatedAt: moment.Moment;

    constructor(dto: any) {
        this.exerciseId = dto.exerciseId;
        this.possiblyOutdated = dto.possiblyOutdated;
        this.detached = dto.detached;
        this.processed = dto.processed;
        this.total = dto.total;
        this.errors = dto.errors;
        this.successful = dto.successful;
        this.running = dto.running;
        this.taskInstanceStates =
            dto?.taskInstanceStates.map((s) => new TaskInstanceState(s)) ?? [];
        this.logs = dto?.logs.map((e) => new BulkEvalLog(e)) ?? [];
        this.initializedAt = Iso8601Helper.fromString(dto.initializedAt);
        this.lastUpdatedAt = Iso8601Helper.fromString(dto.lastUpdatedAt);
    }

    get progress() {
        return this.total != 0 ? (this.processed / this.total) * 100 : 100;
    }
}
