export interface Result {
    type: string;
}

export interface SuccessResult extends Result {}

export interface FailureResult extends Result {
    message: string;
    details: string;
}

export interface ErrorResult extends Result {
    message: string;
    details: string;
}

export interface PendingResult extends Result {}

export interface TimeoutResult extends Result {}

export function isSuccessResult(result: Result): result is SuccessResult {
    return result.type == 'success';
}

export function isFailureResult(result: Result): result is FailureResult {
    return result.type == 'failure';
}

export function isErrorResult(result: Result): result is ErrorResult {
    return result.type == 'error';
}

export function isPendingResult(result: Result): result is PendingResult {
    return result.type == 'pending';
}

export function isTimeoutResult(result: Result): result is TimeoutResult {
    return result.type == 'timeout';
}

export interface Testcase {
    id: string;
    name: string;
    className: string;
    result: Result;
    stdout: string;
    stderr: string;
}

export interface Testsuite {
    testcases: Testcase[];
    stdout: string;
    stderr: string;
    frees?: number;
    allocations?: number;
}

export interface TestResult {
    testcases: number;
    failed: number;
    passed: number;
    leaks?: number;
    correctness: number;
    testsuite: Testsuite;
    output: string;
}

export interface CompilationResult {
    compiling: boolean;
    output: string;
}

export interface StyleError {
    filePath: string;
    message: string;
    severity: string;
    line: string;
    column: string;
    type: string;
}

export interface StyleReport {
    errors: StyleError[];
    tool: string;
    version: string;
}

export interface StyleResult {
    errors: number;
    report: StyleReport;
}

export interface EvalResult {
    compilation?: CompilationResult;
    test?: TestResult;
    style?: StyleResult;
}

/*
export class EvalResult2 {
    id: number;
    evaluator: Evaluator;
    compiling: boolean;
    testErrors: number;
    allocations: number;
    frees: number;
    leaks: number;
    testsRun: number;
    testFailures: number;
    testsPassed: number;
    correctness: number;
    runtimeOutput: string;
    compilationOutput: string;
    testCaseResults: TestCaseResult[];
    styleCheckResults: StyleCheckResult[];
    styleCheckErrors: number;
    rating: EvalRating;
    failed: boolean;

    constructor(evalResult: any) {
        this.id = evalResult.id;
        this.evaluator = evalResult.evaluator
            ? new Evaluator(evalResult.evaluator)
            : null;
        this.compiling = evalResult.compiling;
        this.testErrors = evalResult.testErrors;
        this.allocations = evalResult.allocations;
        this.frees = evalResult.frees;
        this.leaks = evalResult.leaks;
        this.testsRun = evalResult.testsRun;
        this.testFailures = evalResult.testFailures;
        this.testsPassed = evalResult.testsPassed;
        this.correctness = evalResult.correctness;
        this.runtimeOutput = evalResult.runtimeOutput;
        this.compilationOutput = evalResult.compilationOutput;
        this.testCaseResults = evalResult.testCaseResults
            ? evalResult.testCaseResults.map((r) => new TestCaseResult(r))
            : [];
        this.styleCheckResults = evalResult.styleCheckResults
            ? evalResult.styleCheckResults.map((r) => new StyleCheckResult(r))
            : [];
        this.styleCheckErrors = evalResult.styleCheckErrors;
        this.rating = evalResult.rating;
        this.failed = evalResult.failed;
    }
}
*/
