import { User } from 'app/shared/user/user';
import { FileTree } from 'app/shared/fs/file-tree';

import * as moment from 'moment';
import { EvalRating } from './eval-rating';
import { SolutionComment } from './solution-comment';
import { Iso8601Helper } from 'app/shared/misc/helper/iso8601.helper';
import { Task } from 'app/tasks/models/task';
import { Exercise } from 'app/exercises/models/exercise';
import { Submission } from 'app/assessment/models/submission';
import { Client } from './client';
import { EvalState } from 'app/solutions/models/eval-state';

export class Solution {
    id: number;
    exercise: Exercise;
    task: Task;
    submittedBy: User | null;
    tree: FileTree;
    submittedAt: moment.Moment;
    evalState: EvalState;
    acceptanceEvalState: EvalState;
    submission: Submission;
    delayed: boolean;
    attemptsExceeded: boolean;
    comments: SolutionComment[];
    commentsCount: number;
    submittedWith: Client;

    constructor(solution: any) {
        this.id = solution.id;
        this.exercise = solution.exercise
            ? new Exercise(solution.exercise)
            : null;
        this.task = solution.task ? new Task(solution.task) : null;
        this.submittedBy = solution.submittedBy
            ? new User(solution.submittedBy)
            : null;
        this.tree = solution.tree ? new FileTree(solution.tree) : null;
        this.submittedAt = Iso8601Helper.fromString(solution.submittedAt);
        this.evalState = solution.evalState ? solution.evalState : null;
        this.acceptanceEvalState = solution.acceptanceEvalState
            ? solution.acceptanceEvalState
            : null;
        this.submission = solution.submission
            ? new Submission(solution.submission)
            : null;
        this.delayed = solution.delayed;
        this.submittedWith = solution.submittedWith;
        this.attemptsExceeded = solution.attemptsExceeded;
        this.commentsCount = solution.commentsCount;
        this.comments = solution.comments
            ? solution.comments.map((c) => new SolutionComment(c))
            : [];
    }

    get hasResult() {
        return this.evalState?.result != null;
    }

    get evalFailed() {
        return this.evalState?.failed;
    }

    get isGood() {
        return (
            this.hasResult &&
            !this.evalFailed &&
            this.evalState.rating == EvalRating.Good
        );
    }

    get isOk() {
        return (
            this.hasResult &&
            !this.evalFailed &&
            this.evalState.rating == EvalRating.Ok
        );
    }

    get isBad() {
        return (
            this.hasResult &&
            !this.evalFailed &&
            this.evalState.rating == EvalRating.Bad
        );
    }
}
