import { Component, Input, OnChanges, SimpleChanges } from '@angular/core';
import { Solution } from '../../../../models/solution';
import { Breadcrumb } from 'app/shared/breadcrumbs/breadcrumb';

@Component({
    selector: 'sb-solution-breadcrumbs',
    templateUrl: './breadcrumbs.component.html',
    styleUrls: ['./breadcrumbs.component.scss'],
})
export class BreadcrumbsComponent implements OnChanges {
    @Input() solution: Solution;
    breadcrumbs: Breadcrumb[] = [];

    constructor() {}

    ngOnChanges(changes: SimpleChanges) {
        let shouldRefresh = changes.hasOwnProperty('solution');

        if (shouldRefresh) {
            this.breadcrumbs = this.buildBreadcrumbs(this.solution);
        }
    }

    private buildBreadcrumbs(solution: Solution): Breadcrumb[] {
        if (solution.exercise) {
            return this.breadcrumbsForExercise(solution);
        } else {
            return this.breadcrumbsForTask(solution);
        }
    }

    private breadcrumbsForTask(solution: Solution) {
        return [
            {
                id: 'pools',
            },
            {
                id: 'pool',
                params: {
                    pool_id: solution.task.pool.id,
                },
                ctx: solution.task.pool,
            },
            {
                id: 'task',
                params: {
                    task_id: solution.task.id,
                },
                ctx: solution.task,
            },
            {
                id: 'solutions',
                params: {
                    solution_id: solution.id,
                },
                ctx: solution,
            },
        ];
    }

    private breadcrumbsForExercise(solution: Solution) {
        return [
            {
                id: 'exercises',
                params: {
                    course_id: solution.exercise.course.id,
                },
            },
            {
                id: 'exercise',
                params: {
                    exercise_id: solution.exercise.id,
                },
                ctx: solution.exercise,
            },
            {
                id: 'submitSolution',
                params: {
                    exercise_id: solution.exercise.id,
                    task_id: solution.task.id,
                },
                ctx: { exercise: solution.exercise, task: solution.task },
            },
            {
                id: 'solutions',
                params: {
                    solution_id: solution.id,
                },
                ctx: solution,
            },
        ];
    }
}
