import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SolutionPageStore } from 'app/solutions/pages/solution/solution.page.store';
import { EMPTY } from 'rxjs';

@Component({
    templateUrl: './solution.page.component.html',
    styleUrls: ['./solution.page.component.scss'],
    providers: [SolutionPageStore],
})
export class SolutionPageComponent extends BaseComponent implements OnInit {
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Lösung konnten wir leider nicht finden.',
        },
    };

    principal$ = this.solutionDetailStore.principal$;
    solutionState$ = this.solutionDetailStore.solutionState$;
    nextTaskInstance$ = this.solutionDetailStore.nextTaskInstance$;
    justSubmitted$ = this.solutionDetailStore.justSubmitted$;
    hasAccessToAcceptance$ = this.solutionDetailStore.hasAccessToAcceptance$;

    constructor(
        private store: Store<SubatoState>,
        private solutionDetailStore: SolutionPageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private router: Router
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'solution_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.solutionDetailStore.patchSolution({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((solutionId) =>
                this.solutionDetailStore.loadSolution(solutionId)
            );

        this.route.paramMap.pipe(takeUntil(this.destroy$)).subscribe(() =>
            this.solutionDetailStore.patchState({
                justSubmitted:
                    window.history.state && window.history.state.justSubmitted,
            })
        );
    }
}
