import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { EMPTY, Observable } from 'rxjs';
import { BulkEvalState } from 'app/solutions/models/bulk-eval-state';
import { catchError, map } from 'rxjs/operators';
import { Message } from '@stomp/stompjs';
import { RunExerciseAcceptanceDto } from 'app/solutions/dto/run-exercise-acceptance.dto';
import { StompService } from 'app/shared/misc/stomp.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

@Injectable({
    providedIn: 'root',
})
export class AcceptanceService extends BaseService {
    constructor(
        private http: HttpClient,
        private stomp: StompService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super();
    }

    getState(exerciseId: number): Observable<BulkEvalState> {
        return this.http
            .get<BulkEvalState>(
                `${environment.backendUrl}/exercise/${exerciseId}/acceptance`
            )
            .pipe(map((state) => new BulkEvalState(state)));
    }

    run(
        exerciseId: number,
        params: RunExerciseAcceptanceDto
    ): Observable<BulkEvalState> {
        return this.http
            .post<BulkEvalState>(
                `${environment.backendUrl}/exercise/${exerciseId}/acceptance`,
                params
            )
            .pipe(map((state) => new BulkEvalState(state)));
    }

    cancel(exerciseId: number): Observable<void> {
        return this.http.post<void>(
            `${environment.backendUrl}/exercise/${exerciseId}/acceptance/cancel`,
            {}
        );
    }

    reset(exerciseId: number): Observable<void> {
        return this.http.post<void>(
            `${environment.backendUrl}/exercise/${exerciseId}/acceptance/reset`,
            {}
        );
    }

    onStateChange(exerciseId: number): Observable<BulkEvalState> {
        return this.stomp
            .watch(`/topic/exercise/${exerciseId}/acceptance`)
            .pipe(
                map((message: Message) => {
                    let parsed = JSON.parse(message.body);
                    return new BulkEvalState(parsed);
                }),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    return EMPTY;
                })
            );
    }
}
