import { Component, Input, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import {
    isErrorResult,
    isFailureResult,
    isPendingResult,
    isSuccessResult,
    isTimeoutResult,
    Testcase,
} from 'app/solutions/models/eval-result';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Observable } from 'rxjs';
import { EvalState } from 'app/solutions/models/eval-state';

@Component({
    selector: 'sb-solution-eval-result',
    templateUrl: './solution-eval-result.component.html',
    styleUrls: ['./solution-eval-result.component.scss'],
})
export class SolutionEvalResultComponent
    extends BaseComponent
    implements OnInit
{
    @Input() evalState: EvalState;
    showMoreMap: { [id: number]: boolean } = {};
    showOutputMap: { [id: number]: boolean } = {};
    darkMode$: Observable<boolean> = this.store.select(
        (s) => s.layout.darkMode
    );

    constructor(private store: Store<SubatoState>) {
        super();
    }

    toggleMore(testCaseResult: Testcase) {
        this.showMoreMap[testCaseResult.id] = !(
            this.showMoreMap[testCaseResult.id] ?? false
        );
    }

    showMore(testCaseResult: Testcase) {
        return this.showMoreMap[testCaseResult.id] ?? false;
    }

    showOutput(testCaseResult: Testcase) {
        return this.showOutputMap[testCaseResult.id] ?? false;
    }

    toggleOutput(testCaseResult: Testcase) {
        this.showOutputMap[testCaseResult.id] = !(
            this.showOutputMap[testCaseResult.id] ?? false
        );
    }

    getIcon(testcase: Testcase) {
        if (isSuccessResult(testcase.result)) {
            return 'checkmark-circle-2-outline';
        }

        if (isPendingResult(testcase.result)) {
            return 'more-horizontal-outline';
        }

        return 'close-circle-outline';
    }

    getIconClass(testcase: Testcase) {
        if (isSuccessResult(testcase.result)) {
            return 'text-success';
        }

        if (isPendingResult(testcase.result)) {
            return '';
        }

        return 'text-danger';
    }

    ngOnInit(): void {}

    protected readonly eval = eval;
    protected readonly isFailureResult = isFailureResult;
    protected readonly isErrorResult = isErrorResult;
    protected readonly isTimeoutResult = isTimeoutResult;
    protected readonly isPendingResult = isPendingResult;
}
