import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { BaseComponent } from 'app/shared/misc/base.component';
import { CreateSolutionCommentDto } from 'app/solutions/dto/create-solution-comment.dto';
import { SolutionComment } from 'app/solutions/models/solution-comment';
import { Solution } from 'app/solutions/models/solution';
import { SolutionCommentsStore } from 'app/solutions/shared/components/solution-comments/solution-comments.store';

@Component({
    selector: 'sb-solution-comments',
    templateUrl: './solution-comments.component.html',
    styleUrls: ['./solution-comments.component.scss'],
    providers: [SolutionCommentsStore],
})
export class SolutionCommentsComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() solution: Solution;
    commentForm: UntypedFormGroup;

    comments$ = this.solutionCommentsStore.comments$;
    saving$ = this.solutionCommentsStore.saving$;
    deletingComment$ = this.solutionCommentsStore.deletingComment$;
    principal$ = this.solutionCommentsStore.principal$;

    constructor(
        private solutionCommentsStore: SolutionCommentsStore,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    ngOnInit(): void {
        this.createCommentForm();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('solution')) {
            this.solutionCommentsStore.loadComments(this.solution);
        }
    }

    createCommentForm() {
        this.commentForm = this.fb.group({
            text: ['', [Validators.required]],
        });
    }

    createComment() {
        let dto: CreateSolutionCommentDto = {
            text: this.commentForm.get('text').value,
            solutionId: this.solution.id,
        };

        this.solutionCommentsStore.createComment({
            dto: dto,
            onSuccess: (comment) => this.createCommentForm(),
        });
    }

    deleteComment(comment: SolutionComment) {
        this.solutionCommentsStore.deleteComment(comment);
    }
}
