import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import { Solution } from 'app/solutions/models/solution';
import { takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/misc/base.component';
import { SolutionHistoryStore } from 'app/solutions/shared/components/solution-history/solution-history.store';

@Component({
    selector: 'sb-solution-history',
    templateUrl: './solution-history.component.html',
    styleUrls: ['./solution-history.component.scss'],
    providers: [SolutionHistoryStore],
})
export class SolutionHistoryComponent
    extends BaseComponent
    implements OnChanges, OnInit
{
    @Input() exerciseId: number;
    @Input() taskId: number;
    @Input() userId: number;
    @Input() submissionId: number;
    @Input() solution: Solution | null = null;
    @Input() style: string = '';
    @Input() showExternalLink: boolean = false;

    @Output() solutionSelected: EventEmitter<Solution> =
        new EventEmitter<Solution>();
    @Output() solutionsLoaded: EventEmitter<Solution[]> = new EventEmitter<
        Solution[]
    >();

    constructor(public solutionHistoryStore: SolutionHistoryStore) {
        super();
    }

    ngOnInit() {
        this.solutionHistoryStore.paging.items$
            .pipe(takeUntil(this.destroy$))
            .subscribe((solutions) => {
                this.solutionsLoaded.emit(solutions);
            });
    }

    ngOnChanges(changes: SimpleChanges) {
        if (changes.hasOwnProperty('taskId')) {
            this.solutionHistoryStore.updateFilter({ taskId: this.taskId });
        }

        if (changes.hasOwnProperty('exerciseId')) {
            this.solutionHistoryStore.updateFilter({
                exerciseId: this.exerciseId,
            });
        }

        if (changes.hasOwnProperty('submissionId')) {
            this.solutionHistoryStore.updateFilter({
                submissionId: this.submissionId,
            });
        }

        if (changes.hasOwnProperty('userId')) {
            this.solutionHistoryStore.updateFilter({ userId: this.userId });
        }
    }

    selectSolution(solution: Solution) {
        this.solutionSelected.emit(solution);
    }
}
