import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { FileTree } from 'app/shared/fs/file-tree';
import { Subject } from 'rxjs';
import { debounceTime, takeUntil } from 'rxjs/operators';

interface TreeNode<T> {
    data: T;
    children?: TreeNode<T>[];
    expanded?: boolean;
}

@Component({
    selector: 'sb-solution-file-browser',
    templateUrl: './file-browser.component.html',
    styleUrls: ['./file-browser.component.scss'],
})
export class FileBrowserComponent extends BaseComponent implements OnInit {
    @Output() treeSelectedEvent = new EventEmitter<FileTree>();
    _tree: FileTree;
    filteredTree: FileTree;

    data: TreeNode<FileTree>[] = [];

    customColumn = 'filename';
    defaultColumns = [];
    allColumns = [this.customColumn, ...this.defaultColumns];

    selectedTree: FileTree | null = null;

    query: string;
    private queryChanged: Subject<string> = new Subject<string>();

    constructor() {
        super();
    }

    @Input() set tree(val: FileTree) {
        this._tree = val;
        this.setFilteredTree(val);
        this.query = null;
    }

    get tree(): FileTree {
        return this._tree;
    }

    private treeNodeFor(
        tree: FileTree,
        expanded: boolean = false
    ): TreeNode<FileTree> {
        return {
            data: tree,
            children: tree.children.map((child) =>
                this.treeNodeFor(child, expanded)
            ),
            expanded: expanded,
        };
    }

    setFilteredTree(val: FileTree, expand: boolean = false) {
        this.filteredTree = val;
        this.data =
            this.filteredTree?.children?.map((tree) =>
                this.treeNodeFor(tree, expand)
            ) ?? [];
    }

    ngOnInit(): void {
        this.queryChanged
            .pipe(takeUntil(this.destroy$), debounceTime(300))
            .subscribe((query) => {
                this.setFilteredTree(this.tree.filterByName(query), true);
            });
    }

    onTreeClicked(tree: FileTree) {
        if (!tree.directory) {
            this.selectedTree = tree;

            this.treeSelectedEvent.emit(tree);
        }
    }

    filterTree(query) {
        this.query = query;
        this.queryChanged.next(query);
    }
}
