import {
    Directive,
    ElementRef,
    HostListener,
    Inject,
    Renderer2,
} from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { DOCUMENT, ViewportScroller } from '@angular/common';

// https://sreyaj.dev/fullscreen-toggle-angular-using-directives
// + enhanced by .noscroll + esc key + restore scroll pos

@Directive({
    selector: '[maximize]',
    exportAs: 'maximize', // <-- Make not of this here
})
export class MaximizeDirective {
    private isMaximizedSubject = new BehaviorSubject(false);
    isMaximized$ = this.isMaximizedSubject.pipe();
    scrollPosition: [number, number];

    constructor(
        private el: ElementRef,
        private renderer: Renderer2,
        @Inject(DOCUMENT) private document: Document,
        private scroller: ViewportScroller
    ) {}

    toggle() {
        this.isMaximizedSubject?.getValue() ? this.minimize() : this.maximize();
    }

    maximize() {
        if (this.el) {
            this.scrollPosition = this.scroller.getScrollPosition();
            this.isMaximizedSubject.next(true);
            this.renderer.addClass(this.el.nativeElement, 'fullscreen');
            this.renderer.addClass(this.document.body, 'noscroll');
        }
    }

    minimize() {
        if (this.el) {
            this.isMaximizedSubject.next(false);
            this.renderer.removeClass(this.el.nativeElement, 'fullscreen');
            this.renderer.removeClass(this.document.body, 'noscroll');
            this.scroller.scrollToPosition(this.scrollPosition);
        }
    }

    @HostListener('document:keyup.escape', ['$event']) onEscKeyUpPressed(
        event: KeyboardEvent
    ) {
        let isMaximized = this.isMaximizedSubject?.getValue();
        if (isMaximized) {
            this.minimize();
        }
    }

    @HostListener('window:popstate', ['$event'])
    onWindowPopstateChanged(event: Event) {
        let isMaximized = this.isMaximizedSubject?.getValue();
        if (isMaximized) {
            this.minimize();
        }
    }
}
