import {
    AfterViewInit,
    ChangeDetectorRef,
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
    ViewChild,
} from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { FileTree } from 'app/shared/fs/file-tree';
import { Solution } from 'app/solutions/models/solution';
import { SolutionService } from 'app/solutions/services/solution.service';
import { ViewerHostDirective } from 'app/solutions/shared/solution-ide/viewer/viewer-host.directive';
import { Viewer } from 'app/solutions/shared/solution-ide/viewer/viewer';
import { SolutionIdeStore } from 'app/solutions/shared/solution-ide/solution-ide.store';

@Component({
    selector: 'sb-solution-ide',
    templateUrl: './solution-ide.component.html',
    styleUrls: ['./solution-ide.component.scss'],
    providers: [SolutionIdeStore],
})
export class SolutionIdeComponent
    extends BaseComponent
    implements OnInit, OnChanges, AfterViewInit
{
    @Input() withBorder: boolean = false;
    @Input() solution: Solution;

    @ViewChild(ViewerHostDirective)
    viewerHost: ViewerHostDirective;

    Viewer = Viewer;

    selectedTree$ = this.solutionIdeStore.selectedTree$;
    compactMode$ = this.solutionIdeStore.compactMode$;
    fileBrowserEnabled$ = this.solutionIdeStore.fileBrowserEnabled$;

    browserWidth$ = this.solutionIdeStore.browserWidth$;

    viewer$ = this.solutionIdeStore.viewer$;
    viewerLoading$ = this.solutionIdeStore.viewerLoading$;
    viewerError$ = this.solutionIdeStore.viewerError$;

    constructor(
        private solutionIdeStore: SolutionIdeStore,
        private solutionService: SolutionService,
        private cd: ChangeDetectorRef
    ) {
        super();
    }

    ngAfterViewInit() {
        this.solutionIdeStore.patchState({
            viewContainerRef: this.viewerHost.viewContainerRef,
        });
        this.cd.detectChanges();
    }

    ngOnInit(): void {}

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('solution')) {
            this.solutionIdeStore.patchState({ solution: this.solution });
        }
    }

    onTreeSelected(tree: FileTree) {
        if (tree.children.length != 0) {
            return;
        }

        this.solutionIdeStore.patchState({ selectedTree: tree });
    }

    toggleMenu() {
        this.solutionIdeStore.patchState((s) => ({
            compactMode: !s.compactMode,
        }));
    }

    get downloadSolutionUrl() {
        return this.solutionService.getDownloadSolutionUrl(this.solution.id);
    }
}
