import { Component, OnInit } from '@angular/core';
import { Solution } from 'app/solutions/models/solution';
import { SolutionService } from 'app/solutions/services/solution.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { FileTree } from 'app/shared/fs/file-tree';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { catchError, takeUntil } from 'rxjs/operators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SubatoError } from 'app/shared/error/error';

@Component({
    templateUrl: './default-viewer.component.html',
    styleUrls: ['./default-viewer.component.scss'],
})
export class DefaultViewerComponent extends BaseComponent implements OnInit {
    tree: FileTree;
    solution: Solution;

    constructor(
        private solutionService: SolutionService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super();
    }

    ngOnInit(): void {}

    download() {
        this.solutionService
            .downloadFile(this.solution.id, this.tree)
            .pipe(
                takeUntil(this.destroy$),
                catchError((err) => this.globalErrorHandler.handleError(err))
            )
            .subscribe(
                () => {},
                (error: SubatoError) => {
                    this.notificationService.showError(
                        this.globalErrorHandler.format(
                            'Fehler beim Herunterladen der Lösung',
                            error
                        )
                    );
                }
            );
    }
}
