import { Injectable, ViewContainerRef } from '@angular/core';
import { Solution } from 'app/solutions/models/solution';
import { SolutionService } from 'app/solutions/services/solution.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { FileTree } from 'app/shared/fs/file-tree';
import { Observable } from 'rxjs';
import { PdfViewerComponent } from 'app/solutions/shared/solution-ide/viewer/pdf-viewer/pdf-viewer.component';
import { SignService } from 'app/shared/signing/sign.service';
import { map, takeUntil } from 'rxjs/operators';

@Injectable({
    providedIn: 'root',
})
export class PdfViewerFactory extends BaseComponent {
    constructor(
        private solutionService: SolutionService,
        private signService: SignService
    ) {
        super();
    }

    createViewer(
        solution: Solution,
        tree: FileTree,
        containerRef: ViewContainerRef
    ): Observable<void> {
        let url = this.solutionService.getFileUrl(solution.id, tree.uuid);

        return this.signService.sign({ url: url }).pipe(
            takeUntil(this.destroy$),
            map((response) => {
                const componentRef =
                    containerRef.createComponent<PdfViewerComponent>(
                        PdfViewerComponent
                    );
                componentRef.instance.url = response.signedUrl;
            })
        );
    }
}
