import { Injectable, ViewContainerRef } from '@angular/core';
import { Solution } from 'app/solutions/models/solution';
import { SolutionService } from 'app/solutions/services/solution.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { FileTree } from 'app/shared/fs/file-tree';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Observable } from 'rxjs';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { SourceCodeViewerComponent } from 'app/solutions/shared/solution-ide/viewer/source-code-viewer/source-code-viewer.component';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SubatoError } from 'app/shared/error/error';

@Injectable({
    providedIn: 'root',
})
export class SourceCodeViewerFactory extends BaseComponent {
    constructor(
        private solutionService: SolutionService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super();
    }

    createViewer(
        solution: Solution,
        tree: FileTree,
        containerRef: ViewContainerRef
    ): Observable<void> {
        return this.solutionService.getFile(solution.id, tree).pipe(
            takeUntil(this.destroy$),
            map((content) => {
                let file = {
                    tree: tree,
                    content: content,
                };

                const componentRef =
                    containerRef.createComponent<SourceCodeViewerComponent>(
                        SourceCodeViewerComponent
                    );
                componentRef.instance.file = file;
            }),
            catchError((err) => this.globalErrorHandler.handleError(err)),
            catchError((error: SubatoError) => {
                this.notificationService.showError(
                    this.globalErrorHandler.format(
                        'Fehler beim Herunterladen der Lösung',
                        error
                    )
                );
                throw error;
            })
        );
    }
}
