import { Component, forwardRef, Input, OnInit } from '@angular/core';
import { SubmissionMode } from 'app/tasks/models/submission-mode';
import { Task } from 'app/tasks/models/task';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

@Component({
    selector: 'sb-submit-solution-form-submission-mode-select',
    templateUrl: './submission-mode-select.component.html',
    styleUrls: ['./submission-mode-select.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => SubmissionModeSelectComponent),
            multi: true,
        },
    ],
})
export class SubmissionModeSelectComponent
    implements OnInit, ControlValueAccessor
{
    @Input() task: Task;
    @Input() editorDisabled: boolean;
    @Input() controlName: string;
    protected readonly SubmissionMode = SubmissionMode;
    mode: SubmissionMode;

    constructor() {}

    ngOnInit(): void {}

    get submissionModes(): SubmissionMode[] {
        let submissionModes = [];
        if (this.task.supportsFileSubmissionMode) {
            submissionModes.push(SubmissionMode.FILE);
        }

        if (this.task.supportsTextSubmissionMode && !this.editorDisabled) {
            submissionModes.push(SubmissionMode.TEXT);
        }

        return submissionModes;
    }

    // Placeholder functions for the ControlValueAccessor interface
    private onChange: (value: any) => void = () => {};
    private onTouched: () => void = () => {};

    // Method to handle the file change event
    valueChanged(mode: SubmissionMode): void {
        this.mode = mode;
        this.onChange(mode); // Update the form control value
    }

    // ControlValueAccessor interface methods
    writeValue(value: any): void {
        this.mode = value;
    }

    registerOnChange(fn: (value: any) => void): void {
        // Register the callback function to update the form control value
        this.onChange = fn;
    }

    registerOnTouched(fn: () => void): void {
        // Register the callback function for touch events
        this.onTouched = fn;
    }

    setDisabledState(isDisabled: boolean): void {
        // Optional: Implement this method if you want to disable the component
        // based on the form control's disabled state
    }
}
