import { Component, forwardRef, Input, OnInit } from '@angular/core';
import { Task } from 'app/tasks/models/task';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

@Component({
    selector: 'sb-submit-solution-form-upload',
    templateUrl: './upload.component.html',
    styleUrls: ['./upload.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => UploadComponent),
            multi: true,
        },
    ],
})
export class UploadComponent implements OnInit, ControlValueAccessor {
    @Input() task: Task;

    ngOnInit(): void {}

    // Variable to store the selected file
    private file: File | null = null;

    // Placeholder functions for the ControlValueAccessor interface
    private onChange: (value: any) => void = () => {};
    private onTouched: () => void = () => {};

    // Method to handle the file change event
    handleFile(file: File): void {
        this.file = file;
        this.onChange(file); // Update the form control value
    }

    // ControlValueAccessor interface methods
    writeValue(value: any): void {
        // Set the initial value when the form control is initialized
        if (value instanceof File) {
            this.file = value;
        } else {
            this.file = null;
        }
    }

    registerOnChange(fn: (value: any) => void): void {
        // Register the callback function to update the form control value
        this.onChange = fn;
    }

    registerOnTouched(fn: () => void): void {
        // Register the callback function for touch events
        this.onTouched = fn;
    }

    setDisabledState(isDisabled: boolean): void {
        // Optional: Implement this method if you want to disable the component
        // based on the form control's disabled state
    }
}
