import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';
import { AuthActions } from '../auth/store/auth.actions';
import { filter, map, tap, withLatestFrom } from 'rxjs/operators';
import { combineLatest } from 'rxjs';
import { ConfigActions } from '../shared/config/store/config.actions';
import { AppActions, Init } from './app.actions';
import { ModulesActions } from '../modules/store/actions';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { SubatoState } from './app.reducers';

@Injectable()
export class AppEffects {
    waitForInit$ = createEffect(() =>
        combineLatest([
            this.actions$.pipe(ofType(AuthActions.PrincipalChange)),
            this.actions$.pipe(ofType(ConfigActions.LoadedConfig)),
            this.actions$.pipe(ofType(ModulesActions.TermsChange)),
        ]).pipe(
            map(() => {
                return new Init();
            })
        )
    );

    redirectToDashboard$ = createEffect(
        () => {
            return this.actions$.pipe(
                ofType(AppActions.Init),
                withLatestFrom(this.store),
                filter(
                    ([a, state]: [Init, SubatoState]) =>
                        state.auth.principal != null &&
                        state.location.currentUrl == '/'
                ),
                tap(() => this.router.navigate(['/dashboard']))
            );
        },
        { dispatch: false }
    );

    constructor(
        private actions$: Actions,
        private store: Store<SubatoState>,
        private router: Router
    ) {}
}
