import {
    initialState as initialLayoutState,
    layoutReducer,
    LayoutState,
} from '../layout/store/layout.reducers';
import { AppActions } from './app.actions';
import {
    initialState as initialModulesState,
    modulesReducer,
    ModulesState,
} from '../modules/store/reducers';
import {
    configReducer,
    ConfigState,
    initialState as initialConfigState,
} from '../shared/config/store/config.reducers';
import {
    coursesReducer,
    CoursesState,
    initialState as initialCoursesState,
} from '../courses/store/courses.reducers';
import {
    authReducer,
    AuthState,
    initialState as initialAuthState,
} from '../auth/store/auth.reducers';
import {
    initialState as initialLocationState,
    locationReducer,
    LocationState,
} from '../shared/location/store/reducers';
import {
    backNavigationReducer,
    BackNavState,
    initialState as initialBackNavState,
} from '../shared/back-navigation/store/reducers';
import { Action, ActionReducer } from '@ngrx/store';

export interface AppState {
    initialized: boolean;
}

export interface SubatoState {
    app: AppState;
    backNav: BackNavState;
    location: LocationState;
    layout: LayoutState;
    modules: ModulesState;
    config: ConfigState;
    courses: CoursesState;
    auth: AuthState;
}

export const initialAppState: AppState = {
    initialized: false,
};

export const initialState: SubatoState = {
    app: initialAppState,
    courses: initialCoursesState,
    auth: initialAuthState,
    layout: initialLayoutState,
    location: initialLocationState,
    backNav: initialBackNavState,
    modules: initialModulesState,
    config: initialConfigState,
};

export function appReducer(state = initialAppState, action: Action): AppState {
    switch (action.type) {
        case AppActions.Init: {
            return {
                ...state,
                initialized: true,
            };
        }

        default: {
            return state;
        }
    }
}

export function logger(reducer: ActionReducer<SubatoState>): ActionReducer<SubatoState> {
    return (state: SubatoState, action: any): any => {
        const result = reducer(state, action);
        console.groupCollapsed(action.type);
        console.log('prev state', state);
        console.log('action', action);
        console.log('next state', result);
        console.groupEnd();

        return result;
    };
}

export const rootReducer = {
    app: appReducer,
    layout: layoutReducer,
    modules: modulesReducer,
    location: locationReducer,
    config: configReducer,
    backNav: backNavigationReducer,
    auth: authReducer,
    courses: coursesReducer,
};
