import { Language } from './language';
import { SubmissionMode } from './submission-mode';
import { TaskPool } from './task-pool';
import { EvaluationStack } from 'app/tasks/models/evaluation-stack';
import { Competence } from 'app/misconceptions/models/competence';

export class Task {
    id: number;
    softId: string;
    name: string;
    bundle?: string;
    description: string;
    attempts: number | null;
    defaultSubmissionMode: SubmissionMode;
    supportsFileSubmissionMode: boolean;
    supportsTextSubmissionMode: boolean;
    supportedByEclipsePlugin: boolean;
    language: Language;
    pool: TaskPool;
    acceptanceTestAvailable: boolean;
    evaluationStack: EvaluationStack;
    competences: Competence[];
    repositoryUrl: string;
    archived: boolean;
    allowedExtensions: string[];

    constructor(task: any) {
        this.id = task.id;
        this.softId = task.softId;
        this.name = task.name;
        this.description = task.description;
        this.bundle = task.bundle;
        this.attempts = task.attempts;
        this.defaultSubmissionMode = task.defaultSubmissionMode;
        this.supportsFileSubmissionMode = task.supportsFileSubmissionMode;
        this.supportsTextSubmissionMode = task.supportsTextSubmissionMode;
        this.evaluationStack = task.evaluationStack
            ? new EvaluationStack(task.evaluationStack)
            : null;
        this.pool = task.pool ? new TaskPool(task.pool) : null;
        this.language = task.language ? new Language(task.language) : null;
        this.acceptanceTestAvailable = task.acceptanceTestAvailable;
        this.supportedByEclipsePlugin = task.supportedByEclipsePlugin;
        this.repositoryUrl = task.repositoryUrl;
        this.allowedExtensions = task.allowedExtensions;
        this.archived = task.archived;
        this.competences = task.competences
            ? task.competences.map((c) => new Competence(c))
            : [];
    }
}
