import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { FormComponent } from 'app/shared/validation/form.component';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { CreateLanguageDto } from 'app/tasks/dto/create-language.dto';
import { Language } from 'app/tasks/models/language';
import { LanguageService } from 'app/tasks/services/language.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ValidationError } from 'app/shared/error/error';

@Component({
    selector: 'sb-language-form',
    templateUrl: './language-form.component.html',
    styleUrls: ['./language-form.component.scss'],
})
export class LanguageFormComponent
    extends FormComponent
    implements OnInit, OnChanges
{
    @Input() language: Language | null;
    @Input() validationError: ValidationError;
    @Input() saving: boolean = false;
    @Input() deleting: boolean = false;
    @Input() showDelete: boolean = false;

    @Output() onSave = new EventEmitter<CreateLanguageDto>();
    @Output() onDelete = new EventEmitter<void>();

    languageForm: UntypedFormGroup;

    constructor(
        private languageService: LanguageService,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    createForm(language?: Language) {
        this.languageForm = this.fb.group({
            id: [language?.id ?? '', Validators.required],
            name: [language?.name ?? '', Validators.required],
        });
    }

    ngOnInit(): void {
        this.createForm(this.language);
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('language')) {
            this.createForm(this.language);
        }

        if (changes.hasOwnProperty('validationError')) {
            this.errors = this.getValidationErrors(this.validationError);
        }
    }

    save() {
        let dto: CreateLanguageDto = {
            id: this.languageForm.get('id').value,
            name: this.languageForm.get('name').value,
        };

        this.onSave.emit(dto);
    }

    delete() {
        this.onDelete.emit();
    }
}
