import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Language } from 'app/tasks/models/language';
import { LanguageService } from 'app/tasks/services/language.service';
import {
    LanguagesState,
    loadLanguages,
} from 'app/tasks/shared/operators/language.operators';

interface LanguageListState {
    languagesState: LanguagesState;
    deletingLanguage: Language;
}

export const initialState: LanguageListState = {
    languagesState: { languages: [], error: null, loading: true },
    deletingLanguage: null,
};

@Injectable()
export class LanguageListStore extends BaseComponentStore<LanguageListState> {
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    public readonly languagesState$ = this.select(
        (state) => state.languagesState
    );
    public readonly languages$ = this.select(
        (state) => state.languagesState.languages
    );
    public readonly deletingLanguage$ = this.select(
        (state) => state.deletingLanguage
    );

    isSuperAdmin$ = this.select(this.principal$, (principal) =>
        principal.hasSuperAdminRole()
    );

    constructor(
        private store: Store<SubatoState>,
        private languageService: LanguageService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly patchLanguages = this.updaterFor('languagesState');

    readonly loadLanguages = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(loadLanguages((s) => this.patchLanguages(s)))
    );
}
