import { Injectable } from '@angular/core';
import { filter, map, withLatestFrom } from 'rxjs/operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Observable } from 'rxjs';
import { TaskPool } from 'app/tasks/models/task-pool';
import {
    loadLogs,
    TaskImportLogsState,
} from 'app/tasks/shared/operators/task-import-log.operators';

interface ImportLogViewerState {
    logsState: TaskImportLogsState;
    pool: TaskPool;
}

export const initialState: ImportLogViewerState = {
    logsState: { logs: [], error: null, loading: true },
    pool: null,
};

@Injectable()
export class ImportLogViewerStore extends BaseComponentStore<ImportLogViewerState> {
    public readonly pool$ = this.select((state) => state.pool);
    public readonly logsState = this.select((state) => state.logsState);

    constructor() {
        super(initialState);
    }

    readonly patchLogs = this.updaterFor('logsState');

    readonly setPool = this.updater((state, pool: TaskPool) => ({
        ...state,
        pool: pool,
    }));

    readonly refresh = this.effect<void>((params$: Observable<void>) =>
        params$.pipe(
            withLatestFrom(this.pool$),
            filter((pool) => pool != null),
            map(([_, pool]) => pool.id),
            loadLogs((s) => this.patchLogs(s))
        )
    );

    readonly loadLogs$ = this.effect(() =>
        this.pool$.pipe(
            filter((pool) => pool != null),
            map((pool) => pool.id),
            loadLogs((s) => this.patchLogs(s))
        )
    );
}
