import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { Task } from 'app/tasks/models/task';
import { FormComponent } from 'app/shared/validation/form.component';
import { TaskPool } from 'app/tasks/models/task-pool';
import { environment } from 'environments/environment';
import { TaskUploadStore } from 'app/tasks/pages/task-pool/components/upload/task-upload-store.service';
import { DocsService } from 'app/shared/misc/docs.service';

@Component({
    selector: 'sb-task-pool-task-upload',
    templateUrl: './task-upload.component.html',
    styleUrls: ['./task-upload.component.scss'],
    providers: [TaskUploadStore],
})
export class TaskUploadComponent
    extends FormComponent
    implements OnInit, OnChanges
{
    @Input() pool: TaskPool;
    @Output() taskUploaded = new EventEmitter<Task>();

    importing$ = this.taskUploadStore.importing$;
    fileForm: UntypedFormGroup;
    docsUrls = this.docsService.urls;

    constructor(
        private taskUploadStore: TaskUploadStore,
        private router: Router,
        private route: ActivatedRoute,
        private fb: UntypedFormBuilder,
        private docsService: DocsService
    ) {
        super();
    }

    ngOnInit(): void {
        this.createForm();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('pool')) {
            this.taskUploadStore.setPool(this.pool);
        }
    }

    createForm() {
        this.fileForm = this.fb.group({
            file: ['', Validators.required],
        });
    }

    onTaskImported(task: Task) {
        this.fileForm.reset();
        this.taskUploaded.emit(task);
    }

    save() {
        this.taskUploadStore.importTask({
            file: this.fileForm.get('file').value,
            onSuccess: (task) => this.onTaskImported(task),
        });
    }
}
