import { Injectable } from '@angular/core';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Observable } from 'rxjs';
import { TaskPool } from 'app/tasks/models/task-pool';
import { SubatoError } from 'app/shared/error/error';
import { loadPools } from 'app/tasks/shared/operators/task-pool.operators';

interface TaskPoolsState {
    pools: TaskPool[];
    error: SubatoError;
    loading: boolean;
}

interface TaskPoolListState {
    poolsState: TaskPoolsState;
}

export const initialState: TaskPoolListState = {
    poolsState: { pools: [], error: null, loading: true },
};

@Injectable()
export class TaskPoolListStore extends BaseComponentStore<TaskPoolListState> {
    public readonly poolsState = this.select((state) => state.poolsState);

    constructor() {
        super(initialState);
    }

    readonly patchPools = this.updaterFor('poolsState');

    readonly loadPools = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(loadPools((s) => this.patchPools(s)))
    );
}
