import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { BaseService } from 'app/shared/misc/base.service';
import { Language } from '../models/language';
import { CreateLanguageDto } from '../dto/create-language.dto';

@Injectable({
    providedIn: 'root',
})
export class LanguageService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: string): Observable<Language> {
        return this.http
            .get<Language>(`${environment.backendUrl}/language/${id}`)
            .pipe(map((lang) => new Language(lang)));
    }

    getAll(): Observable<Language[]> {
        return this.http
            .get<Language[]>(`${environment.backendUrl}/language`)
            .pipe(
                map((languages) => languages.map((lang) => new Language(lang)))
            );
    }

    create(dto: CreateLanguageDto): Observable<Language> {
        return this.http
            .post<Language>(`${environment.backendUrl}/language`, dto)
            .pipe(
                map((lang) => {
                    return new Language(lang);
                })
            );
    }

    update(langId: string, dto: CreateLanguageDto): Observable<Language> {
        return this.http
            .put<Language>(`${environment.backendUrl}/language/${langId}`, dto)
            .pipe(
                map((lang) => {
                    return new Language(lang);
                })
            );
    }

    delete(langId: string): Observable<void> {
        return this.http.delete<void>(
            `${environment.backendUrl}/language/${langId}`
        );
    }
}
