import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Exercise } from 'app/exercises/models/exercise';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { TaskService } from 'app/tasks/services/task.service';
import { Task } from 'app/tasks/models/task';
import { AuthService } from 'app/auth/auth.service';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { FileRef } from 'app/tasks/models/file-ref';
import { FileType } from 'app/tasks/models/file-type';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { TaskFileListStore } from 'app/tasks/shared/components/task-file-list/task-file-list.store';

@Component({
    selector: 'sb-task-file-list',
    templateUrl: './task-file-list.component.html',
    styleUrls: ['./task-file-list.component.scss'],
    providers: [TaskFileListStore],
})
export class TaskFileListComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() task: Task;
    @Input() exercise: Exercise;
    @Input() taskInstance: TaskInstance;

    filesState$ = this.taskFileListStore.filesState;
    FileType = FileType;

    constructor(
        private notificationService: NotificationService,
        private globalErrorHandler: GlobalErrorHandler,
        private taskFileListStore: TaskFileListStore,
        private authService: AuthService,
        private taskService: TaskService,
        private exerciseService: ExerciseService
    ) {
        super();
    }

    ngOnInit(): void {}

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('exercise')) {
            this.taskFileListStore.setExercise(this.exercise);
        }

        if (changes.hasOwnProperty('task')) {
            this.taskFileListStore.setTask(this.task);
        }
    }

    get solutionPublic() {
        return this.taskInstance?.solutionPublic ?? false;
    }

    isFileVisible(file: FileRef): boolean {
        return (
            file.public ||
            (this.solutionPublic && file.type == FileType.Solution)
        );
    }

    get downloadBundleUrl() {
        return this.taskInstance != null
            ? this.exerciseService.getBundleUrl(this.exercise.id, this.task.id)
            : this.taskService.getBundleUrl(this.task.id);
    }

    downloadFileUrl(ref: FileRef) {
        return this.taskInstance != null
            ? this.exerciseService.getFileUrl(
                  this.exercise.id,
                  this.task.id,
                  ref
              )
            : this.taskService.getFileUrl(this.task.id, ref);
    }
}
