import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { combineLatest } from 'rxjs';
import { Exercise } from 'app/exercises/models/exercise';
import { Task } from 'app/tasks/models/task';
import {
    loadFiles,
    TaskFilesState,
} from 'app/tasks/shared/operators/task-file.operators';

interface TaskFileListState {
    filesState: TaskFilesState;
    exercise: Exercise;
    task: Task;
}

export const initialState: TaskFileListState = {
    filesState: { files: [], error: null, loading: true },
    exercise: null,
    task: null,
};

@Injectable()
export class TaskFileListStore extends BaseComponentStore<TaskFileListState> {
    public readonly exercise$ = this.select((state) => state.exercise);
    public readonly task$ = this.select((state) => state.task);
    public readonly files = this.select((state) => state.filesState.files);
    public readonly filesState = this.select((state) => state.filesState);

    constructor() {
        super(initialState);
    }

    readonly patchFiles = this.updaterFor('filesState');

    readonly setExercise = this.updater((state, exercise: Exercise) => ({
        ...state,
        exercise: exercise,
    }));

    readonly setTask = this.updater((state, task: Task) => ({
        ...state,
        task: task,
    }));

    readonly loadFiles$ = this.effect(() =>
        combineLatest([this.exercise$, this.task$]).pipe(
            filter(([exercise, task]) => task != null),
            map(([exercise, task]) => ({ exercise: exercise, task: task })),
            loadFiles((s) => this.patchFiles(s))
        )
    );
}
