import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { Task } from 'app/tasks/models/task';
import { TaskService } from 'app/tasks/services/task.service';

export interface TaskState {
    task: Task | null;
    error: SubatoError;
    loading: boolean;
}

export function loadTask(updaterFn: (state: Partial<TaskState>) => void) {
    let op = inject(TaskOperators);
    return (taskId$: Observable<number>) => op.loadTask(taskId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskOperators {
    constructor(
        private taskService: TaskService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadTask(
        taskId$: Observable<number>,
        updaterFn: (state: Partial<TaskState>) => void
    ) {
        return taskId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((taskId) =>
                this.taskService.getById(taskId).pipe(
                    tap((task) =>
                        updaterFn({
                            task: task,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            task: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
