import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { TaskPoolPageComponent } from 'app/tasks/pages/task-pool/task-pool.page.component';
import { NewLanguagePageComponent } from 'app/tasks/pages/language-admin/new-language/new-language.page.component';
import { EditTaskPoolPageComponent } from 'app/tasks/pages/task-pool-admin/edit-task-pool/edit-task-pool.page.component';
import { NewTaskPoolPageComponent } from 'app/tasks/pages/task-pool-admin/new-task-pool/new-task-pool.page.component';
import { TaskPool } from './models/task-pool';
import { Task } from './models/task';
import { TaskPoolsPageComponent } from 'app/tasks/pages/task-pools/task-pools.page.component';
import { TaskPageComponent } from 'app/tasks/pages/task/task.page.component';
import { LanguagesPageComponent } from 'app/tasks/pages/languages/languages.page.component';
import { EditLanguagePageComponent } from 'app/tasks/pages/language-admin/edit-language/edit-language.page.component';

export const routes: Routes = [
    {
        path: 'task_pools',
        component: TaskPoolsPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'pools',
                label: 'Aufgaben-Pools',
            },
        },
    },
    {
        path: 'task_pools/new',
        component: NewTaskPoolPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'newPool',
                label: 'Erstellen',
            },
        },
    },
    {
        path: 'task_pools/:pool_id/edit',
        component: EditTaskPoolPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'editPool',
                label: 'Bearbeiten',
            },
        },
    },
    {
        path: 'task_pools/:pool_id',
        component: TaskPoolPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'pool',
                label: (pool: TaskPool) => `${pool.name}`,
            },
            reuseRoute: true,
        },
    },
    {
        path: 'tasks/:task_id',
        component: TaskPageComponent,
        data: {
            authRequired: true,
            breadcrumb: {
                alias: 'task',
                label: (task: Task) => `${task.name}`,
            },
        },
    },
    {
        path: 'language',
        component: LanguagesPageComponent,
        data: {
            authRequired: true,
        },
    },
    {
        path: 'language/new',
        component: NewLanguagePageComponent,
        data: {
            authRequired: true,
        },
    },
    {
        path: 'language/:language_id/edit',
        component: EditLanguagePageComponent,
        data: {
            authRequired: true,
        },
    },
];

@NgModule({
    exports: [RouterModule],
})
export class TasksRoutingModule {}
