package de.hsrm.sls.subato.aor.core;

import de.hsrm.sls.subato.aor.core.usecase.ImportCoreDataUseCase;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import java.io.File;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

// gradle bootRun --args='import:core /home/fischer/Desktop/stammdaten_v3.json'
@Component
public class CoreDataImportRunner implements CommandLineRunner {

    @Autowired
    ImportCoreDataUseCase importCoreDataUseCase;

    Logger logger = LoggerFactory.getLogger(CoreDataImportRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("import:core")) {
            return;
        }

        var filePath = args[1];
        var f = new File(filePath);
        logger.info("Importing core data: %s".formatted(filePath));

        String content = "";
        try {
            content = FileHelper.readString(f);
        } catch (Exception e) {
            logger.error("File %s not readable".formatted(filePath));
            System.exit(0);
        }

        importCoreDataUseCase.importFromJson(content);
        logger.info("Finished importing core data");

        System.exit(0);
    }
}
