package de.hsrm.sls.subato.aor.core.usecase;

import static de.hsrm.sls.subato.aor.core.usecase.ModuleFrequencyConverter.convertFrequency;

import de.hsrm.sls.subato.aor.core.JsonCoreDataReader;
import de.hsrm.sls.subato.aor.core.dto.CoreData;
import de.hsrm.sls.subato.aor.core.dto.ModuleNode;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class ImportCoreDataUseCase {

    @Autowired
    DegreeProgramRepository programRepository;

    @Autowired
    ImportProgramsUseCase importProgramsUseCase;

    @Autowired
    ImportModulesUseCase importModulesUseCase;

    @Autowired
    JsonCoreDataReader jsonCoreDataReader;

    public void importFromJson(String json) {
        var coreData = jsonCoreDataReader.read(json);
        importProgramsUseCase.importPrograms(coreData.getCoursePrograms());

        var modulesToImport = getModulesForImport(coreData);
        importModulesUseCase.importModules(modulesToImport);
    }

    private List<ModuleNode> getModulesForImport(CoreData coreData) {
        var existingPrograms = programRepository.findAll();
        var filteredModuleNodes = coreData.getModuleNodes().stream()
            .filter(
                n -> existingPrograms.stream()
                    .anyMatch(p -> n.getProgramId().equals(p.getSupplementId())))
            //.filter(n -> n.getListId() == null)
            .filter(n -> convertFrequency(n.getFrequency()) != null)
            .filter(n -> n.getModuleParts().stream().anyMatch(
                mp -> mp.getCourseType() != null && (mp.getCourseType().getMatchcode().equals("P")
                    || mp.getCourseType().getMatchcode().equals("Ü"))))
            .collect(Collectors.toList());
        return filteredModuleNodes;
    }
}
