package de.hsrm.sls.subato.aor.core.usecase;

import de.hsrm.sls.subato.aor.core.dto.CourseProgram;
import de.hsrm.sls.subato.modules.program.DegreeProgram;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
class ImportProgramsUseCase {

    @Autowired
    DegreeProgramRepository programRepository;

    public void importPrograms(List<CourseProgram> coursePrograms) {
        var filteredCoursePrograms = coursePrograms.stream()
            .filter(cp -> cp.getName().toLowerCase().contains("informatik"))
            .filter(cp -> cp.getPoYear() != null) // SBI
            .collect(Collectors.toList());
        var existingPrograms = programRepository.findAll();

        // Set programs inactive which are not in import
        var deactivatePrograms = existingPrograms.stream()
            .filter(p -> p.isActive() && filteredCoursePrograms.stream()
                .allMatch(cp -> !cp.getId().equals(p.getSupplementId())))
            .collect(Collectors.toList());

        for (var deactivateProgram : deactivatePrograms) {
            deactivateProgram.setActive(false);
            programRepository.save(deactivateProgram);
        }

        // Add new programs
        for (var courseProgram : filteredCoursePrograms) {
            var degreeProgram = existingPrograms.stream()
                .filter(p -> p.getSupplementId().equals(courseProgram.getId()))
                .findFirst();

            DegreeProgram program;
            if (degreeProgram.isPresent()) {
                program = degreeProgram.get();
            } else {
                program = new DegreeProgram();
                program.setSupplementId(courseProgram.getId());
            }

            program.setName(courseProgram.getName());
            program.setShortName(courseProgram.getMatchcode());
            program.setPoYear(courseProgram.getPoYear());
            program.setPosCode(courseProgram.getPosCode());
            program.setActive(true);

            programRepository.save(program);
        }
    }
}
