package de.hsrm.sls.subato.aor.term;

import de.hsrm.sls.subato.aor.term.usecase.ImportTermDataUseCase;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import java.io.File;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

// gradle bootRun --args='import:term /home/fischer/Desktop/term-data'
@Component
public class TermDataImportRunner implements CommandLineRunner {

    @Autowired
    ImportTermDataUseCase importTermDataUseCase;

    @Autowired
    TermRepository termRepository;

    Logger logger = LoggerFactory.getLogger(TermDataImportRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("import:term")) {
            return;
        }

        var filePath = args[1];
        var coreDataDir = new File(filePath);
        if (!coreDataDir.exists()) {
            logger.error("Directory %s does not exist".formatted(filePath));
            System.exit(0);
        }

        logger.info("Importing term data from files in directory %s...".formatted(filePath));

        for (var file : coreDataDir.listFiles()) {
            String json;
            try {
                json = FileHelper.readString(file);
            } catch (Exception e) {
                logger.error("File %s not readable".formatted(file.toPath()), e);
                continue;
            }

            importTermDataUseCase.importFromJson(json);
            logger.info("Finished importing term data %s".formatted(file.toPath().toString()));
        }

        logger.info("Finished importing term data");
        System.exit(0);
    }
}
