package de.hsrm.sls.subato.aor.term.usecase;

import de.hsrm.sls.subato.aor.term.dto.Lecturer;
import de.hsrm.sls.subato.aor.term.dto.ModulePartInstance;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.util.ArrayList;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
class ImportLecturersUseCase {

    @Autowired
    UserRepository userRepository;

    @Autowired
    CourseUnitRepository courseUnitRepository;

    public List<User> importLecturers(ModulePartInstance modulePartInstance) {
        var users = new ArrayList<User>();
        for (var lecturer : modulePartInstance.getLecturers()) {
            var user = getOrCreateLecturer(lecturer);
            users.add(user);
        }

        return users;
    }

    private User getOrCreateLecturer(Lecturer lecturer) {
        User user = null;
        if (lecturer.getUser().getHdsName() != null) {
            var hdsLoginName = lecturer.getUser().getHdsName();
            user = userRepository.findByLoginNameIgnoreCase(hdsLoginName).orElse(null);
        }

        if (user == null) {
            user = userRepository.findByMailIgnoreCase(lecturer.getEmail()).orElse(null);
        }

        if (user == null) {
            user = new User();
            user.setLoginName(lecturer.getUser().getHdsName());
            user.setFirstName(lecturer.getFirstName());
            user.setLastName(lecturer.getLastName());
            user.setMail(lecturer.getEmail());
            return userRepository.save(user);
        }

        return user;
    }
}
