package de.hsrm.sls.subato.aor.term.usecase;

import de.hsrm.sls.subato.aor.term.dto.ModulePartInstance;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
class ImportParticipantsUseCase {

    private static Logger logger = LoggerFactory.getLogger(ImportParticipantsUseCase.class);

    @Autowired
    UserRepository userRepository;

    @Autowired
    CourseUnitRepository courseUnitRepository;

    public List<User> importParticipants(List<ModulePartInstance> modulePartInstances) {
        var participants = new ArrayList<User>();
        for (var modulePartInstance : modulePartInstances) {
            for (var participant : modulePartInstance.getStudents()) {
                var user = getOrCreateUser(participant);
                if (user != null) {
                    participants.add(user);
                } else {
                    logger.error(
                        "participant for mpi %s is null?".formatted(modulePartInstance.getId()));
                }
            }
        }
        return participants;
    }

    private User getOrCreateUser(String hdsLoginName) {
        if (hdsLoginName == null) {
            logger.error("hdsLoginName is null");
            return null;
        }

        var user = userRepository.findByLoginNameIgnoreCase(hdsLoginName).orElse(null);
        if (user == null) {
            user = new User();
            user.setLoginName(hdsLoginName);
            user = userRepository.save(user);
        }

        return user;
    }
}
