package de.hsrm.sls.subato.aor.term.usecase;

import de.hsrm.sls.subato.aor.term.JsonTermDataReader;
import de.hsrm.sls.subato.aor.term.dto.TermData;
import de.hsrm.sls.subato.aor.term.model.TermDataImport;
import de.hsrm.sls.subato.aor.term.repository.TermDataImportRepository;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.modules.term.Term;
import java.time.LocalDateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class ImportTermDataUseCase {

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    JsonTermDataReader jsonTermDataReader;

    @Autowired
    ImportTermUseCase importTermUseCase;

    @Autowired
    ImportUnitsUseCase importUnitsUseCase;

    @Autowired
    private TermDataImportRepository termDataImportRepository;

    @Transactional
    public void importFromJson(String json) {
        var termData = jsonTermDataReader.read(json);
        if (termDataImportRepository.importExistsForTermData(termData)) {
            return;
        }

        var term = importTermUseCase.importTerm(termData.getTerm());
        storeImport(term, termData);

        var courses = courseRepository.findByTermId(term.getId())
            .stream().filter(c -> c.getModules().size() != 0 && !c.isFinished())
            .toList();
        for (var course : courses) {
            importUnitsUseCase.importUnits(course, termData);
        }
    }

    private TermDataImport storeImport(Term term, TermData termData) {
        var termDataImport = new TermDataImport(term, termData, LocalDateTime.now());
        return termDataImportRepository.save(termDataImport);
    }
}
