package de.hsrm.sls.subato.assessment.submission.model;

import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.user.User;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Entity
@Table(name = "submission")
public class Submission {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @OneToMany(mappedBy = "submission")
    private List<Solution> solution = new ArrayList<>();

    @ManyToOne
    private User submittedBy;

    @ManyToOne
    private Exercise exercise;

    @ManyToOne
    private User checkedBy;

    private Boolean passed;

    @OneToMany(mappedBy = "submission", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<SubmissionPointRating> pointRatings = new ArrayList<>();

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public List<Solution> getSolution() {
        return solution;
    }

    public void setSolution(List<Solution> solution) {
        this.solution = solution;
    }

    public User getSubmittedBy() {
        return submittedBy;
    }

    public void setSubmittedBy(User submittedBy) {
        this.submittedBy = submittedBy;
    }

    public Exercise getExercise() {
        return exercise;
    }

    public void setExercise(Exercise exercise) {
        this.exercise = exercise;
    }

    public User getCheckedBy() {
        return checkedBy;
    }

    public void setCheckedBy(User checkedBy) {
        this.checkedBy = checkedBy;
    }

    public Boolean getPassed() {
        return passed;
    }

    public void setPassed(Boolean passed) {
        this.passed = passed;
    }

    public List<SubmissionPointRating> getPointRatings() {
        return pointRatings;
    }

    public void setPointRatings(List<SubmissionPointRating> pointRatings) {
        this.pointRatings = pointRatings;
    }

    public int getTotalPoints() {
        return pointRatings.stream().map(r -> r.getPoints()).reduce(0, (total, b) -> total + b);
    }

    public long getPointRatingCount() {
        return pointRatings.stream().count();
    }

    public List<TaskInstanceAttempts> getAttempts() {
        var group = solution.stream().collect(Collectors.groupingBy(
            p -> exercise.getTaskInstances().stream()
                .filter(ti -> ti.getTask().getId() == p.getTask().getId()).findFirst().get()));
        return group.entrySet().stream()
            .map(e -> new TaskInstanceAttempts(e.getKey(), e.getValue().size())).toList();
    }
}
