package de.hsrm.sls.subato.assessment.submission.repository;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import org.springframework.data.jpa.domain.Specification;

public class SubmissionFilter {

    private final Long submittedById;
    private final Long exerciseId;
    private final Long unitId;
    private final Long courseId;

    private SubmissionFilter(Long submittedById, Long exerciseId, Long unitId, Long courseId) {
        this.submittedById = submittedById;
        this.exerciseId = exerciseId;
        this.unitId = unitId;
        this.courseId = courseId;
    }

    public Long getSubmittedById() {
        return submittedById;
    }

    public Long getExerciseId() {
        return exerciseId;
    }

    public Long getUnitId() {
        return unitId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public Specification<Submission> toSpecification() {
        Specification<Submission> spec = Specification.where(null);
        if (submittedById != null) {
            spec = spec.and(SubmissionSpecifications.submittedByEquals(submittedById));
        }

        if (exerciseId != null) {
            spec = spec.and(SubmissionSpecifications.exerciseEquals(exerciseId));
        }

        if (unitId != null) {
            spec = spec.and(SubmissionSpecifications.unitEquals(unitId));
        }

        if (courseId != null) {
            spec = spec.and(SubmissionSpecifications.courseEquals(courseId));
        }

        return spec;
    }

    public static class Builder {

        private Long submittedById;
        private Long exerciseId;
        private Long unitId;
        private Long courseId;

        public Builder submittedById(Long submittedById) {
            this.submittedById = submittedById;
            return this;
        }

        public Builder exerciseId(Long exerciseId) {
            this.exerciseId = exerciseId;
            return this;
        }

        public Builder unitId(Long unitId) {
            this.unitId = unitId;
            return this;
        }

        public Builder courseId(Long courseId) {
            this.courseId = courseId;
            return this;
        }

        public SubmissionFilter build() {
            return new SubmissionFilter(submittedById, exerciseId, unitId, courseId);
        }
    }
}
