package de.hsrm.sls.subato.assessment.submission.usecase;

import de.hsrm.sls.subato.assessment.submission.dto.UpdatePointsDto;
import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.assessment.submission.model.SubmissionPointRating;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionRepository;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.exercises.exercise.model.AssessmentMode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UpdatePointsUseCase {

    @Autowired
    private SubmissionRepository submissionRepository;

    public Submission updatePoints(Submission submission, UpdatePointsDto dto) {
        if (submission.getExercise().getCourse().isFinished()) {
            throw new CourseFinishedException();
        }

        if (submission.getExercise().getAssessmentMode() != AssessmentMode.Points) {
            throw new ExerciseNotUsingPointsException(submission.getExercise().getId());
        }

        var taskInstance = submission.getExercise().getTaskInstances().stream()
            .filter(ti -> ti.getTask().getId() == dto.getTaskId())
            .findFirst()
            .orElseThrow(() -> new TaskInstanceNotFoundException(submission.getExercise().getId(),
                dto.getTaskId()));

        var submissionPointRating = submission.getPointRatings().stream()
            .filter(r -> r.getTaskInstance().getId() == taskInstance.getId())
            .findFirst()
            .orElse(null);

        if (submissionPointRating == null) {
            submissionPointRating = new SubmissionPointRating();
            submissionPointRating.setSubmission(submission);
            submissionPointRating.setTaskInstance(taskInstance);
            submission.getPointRatings().add(submissionPointRating);
        }

        submissionPointRating.setPoints(dto.getPoints());
        return submissionRepository.save(submission);
    }
}
