package de.hsrm.sls.subato.attendances.attendance.usecase;

import de.hsrm.sls.subato.attendances.attendance.AttendanceRepository;
import de.hsrm.sls.subato.attendances.attendance.model.ParticipantAttendance;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.shared.helper.StreamHelper;
import de.hsrm.sls.subato.user.User;
import java.util.List;
import java.util.stream.Stream;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class GetAttendanceUseCase {

    private record SelectedCourseUnit(CourseUnit unit, boolean active) {

    }

    @Autowired
    private AttendanceRepository attendanceRepository;

    public List<ParticipantAttendance> forCourseAndUser(Course course, User user) {
        var selectedUnits = selectUnits(course, user);
        return selectedUnits.stream()
            .map(selectedUnit -> forUnitAndUser(selectedUnit.unit, user, selectedUnit.active))
            .toList();
    }

    private List<SelectedCourseUnit> selectUnits(Course course, User user) {
        var courseUnits = user.getParticipatingUnits().stream()
            .filter(unit -> unit.getCourse().getId() == course.getId())
            .filter(unit -> unit.getType() == CourseUnitType.Lab)
            .map(unit -> new SelectedCourseUnit(unit, true));

        // additionally, get all the units, where the user was a participant.
        // because we do not store the participation history, we look for attendance entries in other units.
        var unitsWithAttendance = attendanceRepository.findByStudentAndCourse(user.getId(),
                course.getId()).stream().map(a -> a.getEvent().getUnit())
            .map(unit -> new SelectedCourseUnit(unit, false));

        var uniqueUnits = Stream.concat(courseUnits, unitsWithAttendance)
            .filter(StreamHelper.distinctByKey(unitInput -> unitInput.unit.getId()));

        return uniqueUnits.toList();
    }

    public ParticipantAttendance forUnitAndUser(CourseUnit courseUnit, User user, boolean active) {
        var attendances = attendanceRepository.findByStudentAndCourseUnit(user.getId(),
            courseUnit.getId());

        var unitAttendance = new ParticipantAttendance(user, courseUnit, active);
        for (var event : courseUnit.getEvents()) {
            var attendance = attendances.stream().filter(
                a -> a.getStudent().getId() == user.getId() && a.getEvent().getId() == event.getId()
            ).findFirst().orElse(null);

            if (attendance != null) {
                unitAttendance.getEventAttendances().add(attendance);
            }
        }

        return unitAttendance;
    }
}
