package de.hsrm.sls.subato.auth.masking;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import de.hsrm.sls.subato.auth.principal.PrincipalService;
import java.io.IOException;

/**
 * see {@link MaskerSerializerModifier}
 */
class MaskerSerializer<T> extends JsonSerializer<T> {

    private final PrincipalService principalService;

    private final Masker<T> masker;

    private final JsonSerializer<T> defaultSerializer;

    public MaskerSerializer(Masker<T> masker, JsonSerializer<T> defaultSerializer,
        PrincipalService principalService) {
        this.masker = masker;
        this.defaultSerializer = defaultSerializer;
        this.principalService = principalService;
    }

    public void serialize(T obj, JsonGenerator jsonGenerator,
        SerializerProvider serializerProvider) throws IOException {
        var maskedObj = masker.mask(obj, principalService.getPrincipal());
        defaultSerializer.serialize(maskedObj, jsonGenerator, serializerProvider);
    }
}
