package de.hsrm.sls.subato.auth.oauth;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.converter.Converter;
import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Component;

@Component
public class JwtAuthConverter implements Converter<Jwt, AbstractAuthenticationToken> {

    @Autowired
    private KeycloakLoginHandler loginHandler;

    @Override
    public synchronized AbstractAuthenticationToken convert(Jwt jwt) {
        var userInfo = buildUserInfo(jwt);
        SubatoPrincipal principal = loginHandler.handleToken(userInfo);
        return new UsernamePasswordAuthenticationToken(principal, jwt, principal.getAuthorities());
        //return new JwtAuthenticationToken(jwt, authorities, getPrincipalClaimName(jwt));
    }

    private KeycloakUserInfo buildUserInfo(Jwt auth) {
        var username = (String) auth.getClaim(ClaimConstants.USERNAME);
        var mail = (String) auth.getClaim(ClaimConstants.EMAIL);
        var alias = (String) auth.getClaim(ClaimConstants.ALIAS);
        var firstName = (String) auth.getClaim(ClaimConstants.FIRST_NAME);
        var lastName = (String) auth.getClaim(ClaimConstants.LAST_NAME);
        var studentInfo = (String) auth.getClaim(ClaimConstants.STUDENT_INFO);

        Map<String, Object> resourceAccess = auth.getClaim(ClaimConstants.REALM_ACCESS);
        List<String> plainRoles = (List<String>) resourceAccess.get(ClaimConstants.ROLES);

        return new KeycloakUserInfo.Builder().setUsername(username).setFirstName(firstName)
            .setLastName(lastName).setMail(mail).setAlias(alias).setStudentInfo(studentInfo)
            .setRoles(plainRoles).build();
    }
}