package de.hsrm.sls.subato.auth.oauth;

import java.util.List;

/**
 * Represents the claims from keycloak, that can be extracted from the token.
 */
public class KeycloakUserInfo {

    private final String alias;
    private final String username;
    private final String firstName;
    private final String lastName;
    private final String mail;
    private final String studentInfo;
    private final List<String> roles;

    private KeycloakUserInfo(String alias, String username, String firstName, String lastName,
        String mail, String studentInfo, List<String> roles) {
        this.alias = alias;
        this.username = username;
        this.firstName = firstName;
        this.lastName = lastName;
        this.mail = mail;
        this.studentInfo = studentInfo;
        this.roles = roles;
    }

    public String getAlias() {
        return alias;
    }

    public String getUsername() {
        return username;
    }

    public String getFirstName() {
        return firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public String getMail() {
        return mail;
    }

    public String getStudentInfo() {
        return studentInfo;
    }

    public List<String> getRoles() {
        return roles;
    }

    public static class Builder {

        private String alias;
        private String username;
        private String firstName;
        private String lastName;
        private String mail;
        private String studentInfo;
        private List<String> roles;

        public Builder setAlias(String alias) {
            this.alias = alias;
            return this;
        }

        public Builder setUsername(String username) {
            this.username = username;
            return this;
        }

        public Builder setFirstName(String firstName) {
            this.firstName = firstName;
            return this;
        }

        public Builder setLastName(String lastName) {
            this.lastName = lastName;
            return this;
        }

        public Builder setMail(String mail) {
            this.mail = mail;
            return this;
        }

        public Builder setStudentInfo(String studentInfo) {
            this.studentInfo = studentInfo;
            return this;
        }

        public Builder setRoles(List<String> roles) {
            this.roles = roles;
            return this;
        }

        public KeycloakUserInfo build() {
            return new KeycloakUserInfo(alias, username, firstName, lastName, mail, studentInfo,
                roles);
        }
    }
}
