package de.hsrm.sls.subato.auth.oauth;

import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.converter.Converter;
import org.springframework.security.oauth2.client.authentication.OAuth2AuthenticationToken;
import org.springframework.security.oauth2.client.authentication.OAuth2LoginAuthenticationToken;
import org.springframework.stereotype.Component;

/**
 * Spring does not support enhancing the {@link OAuth2AuthenticationToken} directly with a custom
 * principal. So once the login has been completed, we (create) and look up the user in the database
 * using the information in the token.
 */
@Component
public class OAuth2ClientAuthResultConverter implements
    Converter<OAuth2LoginAuthenticationToken, OAuth2AuthenticationToken> {

    @Autowired
    private KeycloakLoginHandler loginHandler;

    @Override
    public synchronized OAuth2AuthenticationToken convert(OAuth2LoginAuthenticationToken token) {
        var userInfo = buildUserInfo(token);
        var principal = loginHandler.handleToken(userInfo);

        return new OAuth2AuthenticationToken(principal,
            principal.getAuthorities(),
            token.getClientRegistration().getRegistrationId());
    }

    private KeycloakUserInfo buildUserInfo(OAuth2LoginAuthenticationToken token) {
        var username = (String) token.getPrincipal().getAttribute(ClaimConstants.USERNAME);
        var alias = (String) token.getPrincipal().getAttribute(ClaimConstants.ALIAS);
        var mail = (String) token.getPrincipal().getAttribute(ClaimConstants.EMAIL);
        var firstName = (String) token.getPrincipal().getAttribute(ClaimConstants.FIRST_NAME);
        var lastName = (String) token.getPrincipal().getAttribute(ClaimConstants.LAST_NAME);
        var studentInfo = (String) token.getPrincipal().getAttribute(ClaimConstants.STUDENT_INFO);

        Map<String, Object> resourceAccess = token.getPrincipal()
            .getAttribute(ClaimConstants.REALM_ACCESS);
        List<String> plainRoles = (List<String>) resourceAccess.get(ClaimConstants.ROLES);

        return new KeycloakUserInfo.Builder().setUsername(username).setFirstName(firstName)
            .setLastName(lastName).setMail(mail).setAlias(alias).setStudentInfo(studentInfo)
            .setRoles(plainRoles).build();
    }
}
