package de.hsrm.sls.subato.auth.policy;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import java.util.Map;

public class PolicyContext {

    private final SubatoPrincipal subject;
    private final Object obj;
    private final Object permission;
    private final Map<String, Object> environment;

    private PolicyContext(SubatoPrincipal subject, Object obj, Object permission,
        Map<String, Object> environment) {
        this.subject = subject;
        this.obj = obj;
        this.permission = permission;
        this.environment = environment;
    }

    public boolean authenticated() {
        return subject != null;
    }

    public boolean unauthenticated() {
        return !authenticated();
    }

    public boolean matches(String permission) {
        return this.permission != null && this.permission.equals(permission);
    }

    public SubatoPrincipal getSubject() {
        return subject;
    }

    public Object getObj() {
        return obj;
    }

    public Object getPermission() {
        return permission;
    }

    public Map<String, Object> getEnvironment() {
        return environment;
    }

    public static class Builder {

        private SubatoPrincipal subject;
        private Object obj;
        private Object permission;
        private Map<String, Object> environment;

        public SubatoPrincipal getSubject() {
            return subject;
        }

        public Builder setSubject(SubatoPrincipal subject) {
            this.subject = subject;
            return this;
        }

        public Object getObj() {
            return obj;
        }

        public Builder setObj(Object obj) {
            this.obj = obj;
            return this;
        }

        public Object getPermission() {
            return permission;
        }

        public Builder setPermission(Object permission) {
            this.permission = permission;
            return this;
        }

        public Map<String, Object> getEnvironment() {
            return environment;
        }

        public Builder setEnvironment(Map<String, Object> environment) {
            this.environment = environment;
            return this;
        }

        public PolicyContext build() {
            return new PolicyContext(subject, obj, permission, environment);
        }
    }
}
