package de.hsrm.sls.subato.auth.policy;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Component;

@Component
public class PolicyEvaluator {

    private static Logger logger = LoggerFactory.getLogger(PolicyEvaluator.class);

    @Autowired
    private List<PolicyRule> rules;

    public boolean check(PolicyContext context) {

        for (PolicyRule rule : rules) {
            try {
                if (rule.check(context)) {
                    return true;
                }
            } catch (Exception ex) {
                logger.info("An error occurred while evaluating PolicyRule.", ex);
            }
        }

        return false;
    }

    /**
     * For usage in controller body
     * <p>
     * policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(id, Permission.MI_UPDATE));
     *
     * @param context
     */
    public void checkAndRaise(PolicyContext context) {
        var access = check(context);
        if (!access) {
            throw new AccessDeniedException("");
        }
    }

    public Specification checkAsSpec(PolicyContext context) {

        var totalSpec = Specification.where(null);
        for (PolicyRule rule : rules) {
            try {
                var spec = rule.checkAsSpec(context);
                totalSpec = totalSpec.or(spec);
            } catch (Exception ex) {
                logger.info("An error occurred while evaluating PolicyRule.", ex);
            }
        }

        return totalSpec;
    }
}
