package de.hsrm.sls.subato.auth.policy.attendance;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.user.User;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view attendances within a course
 */
@Component
public class AttendanceViewPolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(AttendancePermission.ATTENDANCE_VIEW) || context.unauthenticated()) {
            return false;
        }

        var req = (Pair<Course, User>) context.getObj();
        var course = req.getLeft();
        var user = req.getRight();

        return user.getId() == context.getSubject().getId() || context.getSubject()
            .hasAnyGroup(course.getId());
    }
}
