package de.hsrm.sls.subato.auth.policy.courses;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to join a course unit within a certain course
 */
@Component
public class CourseUnitJoinPolicy implements PolicyRule {

    @Autowired
    @Lazy
    private PolicyEvaluator policy;

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(CoursesAction.COURSE_UNIT_JOIN) || context.unauthenticated()) {
            return false;
        }

        var unit = (CourseUnit) context.getObj();

        var ctx = new PolicyContext.Builder()
            .setSubject(context.getSubject())
            .setObj(unit.getCourse())
            .setPermission(CoursesAction.COURSE_VIEW).build();

        var canViewCourse = policy.check(ctx);
        return context.getSubject().hasAnyGroup(unit.getCourse().getId())
            || (canViewCourse && unit.getCourse().isAllowJoiningUnits());
    }
}
