package de.hsrm.sls.subato.auth.policy.exercises;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.assessment.submission.model.Submission_;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.policy.SubatoPrincipalSpecifications;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.model.Course_;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise_;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view a submission
 */
@Component
public class SubmissionViewPolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(ExercisesAction.SUBMISSION_VIEW) || context.unauthenticated()) {
            return false;
        }

        var submission = (Submission) context.getObj();
        return submission.getSubmittedBy().getId() == context.getSubject().getId()
            || context.getSubject().hasAnyGroup(submission.getExercise().getCourse().getId());
    }

    @Override
    public Specification checkAsSpec(PolicyContext context) {
        if (!context.matches(ExercisesAction.SUBMISSION_VIEW)) {
            return Specification.where(null);
        }

        return accessesOwnSubmission(context.getSubject()).or(hasAnyGroup(context.getSubject()));
    }

    public static Specification<Submission> accessesOwnSubmission(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(ex.get(Submission_.submittedBy).get(User_.id),
            principal != null ? principal.getId() : -1);
    }

    public static Specification<Submission> hasAnyGroup(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var exerciseCourseJoin = ex
                .join(Submission_.exercise)
                .join(Exercise_.course);

            var courseStaffInBuilder = SubatoPrincipalSpecifications.courseStaffIn(principal, cb,
                exerciseCourseJoin.get(Course_.id));
            return courseStaffInBuilder;
        };
    }
}
