package de.hsrm.sls.subato.auth.policy.exercises;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view a file for a certain task instance
 */
@Component
public class TaskFileViewPolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(ExercisesAction.TASK_FILE_VIEW)) {
            return false;
        }

        var pair = (Pair<FileRef, TaskInstance>) context.getObj();
        var file = pair.getLeft();
        var taskInstance = pair.getRight();

        return visibleFor(file, context.getSubject(), taskInstance);
    }

    public boolean visibleFor(FileRef file, SubatoPrincipal principal, TaskInstance ti) {
        var isStaff =
            principal != null && principal.hasAnyGroup(ti.getExercise().getCourse().getId());
        return file.isPublic() || (ti.isSolutionPublic() && file.getType() == FileType.SOLUTION)
            || isStaff;
    }
}
