package de.hsrm.sls.subato.auth.policy.modules;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.modules.term.Term;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view terms
 */
@Component
public class TermViewPolicy implements PolicyRule {

    @Autowired
    @Lazy
    private PolicyEvaluator policy;

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(ModulesAction.TERM_VIEW)) {
            return false;
        }

        var term = (Term) context.getObj();
        return (context.authenticated() && context.getSubject().hasRole(Role.Staff))
            || canViewAtLeastOneCourseIn(term, context);
    }

    private boolean canViewAtLeastOneCourseIn(Term term, PolicyContext context) {
        return term.getCourses().stream().anyMatch(
            c -> {
                var ctx = new PolicyContext.Builder()
                    .setSubject(context.getSubject())
                    .setObj(c)
                    .setPermission(CoursesAction.COURSE_VIEW).build();
                return policy.check(ctx);
            });
    }
}