package de.hsrm.sls.subato.auth.policy.solutions;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to run acceptance tests for a solution within a certain course
 */
@Component
public class SolutionRunAcceptanceTestsPolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(SolutionsAction.SOLUTION_RUN_ACCEPTANCE_TESTS)
            || context.unauthenticated()) {
            return false;
        }

        var solution = (Solution) context.getObj();
        return solution.getExercise() != null
            ? context.getSubject()
            .hasGroup(solution.getExercise().getCourse().getId(), StaffGroup.LECTURER)
            : context.getSubject().hasRole(Role.Staff);
    }
}
