package de.hsrm.sls.subato.auth.policy.solutions;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.course.model.SubmitPolicy;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.tasks.task.Task;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to submit a solution to an exercise within a certain course
 */
@Component
public class SolutionSubmitExercisePolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(SolutionsAction.SOLUTION_SUBMIT_EXERCISE)) {
            return false;
        }

        var taskInstance = (Pair<Exercise, Task>) context.getObj();
        var course = taskInstance.getKey().getCourse();
        return course.getSubmitPolicy() == SubmitPolicy.EVERYONE ||
            (context.authenticated() && context.getSubject().isParticipant(course.getId()));
    }
}
