package de.hsrm.sls.subato.auth.policy.users;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;


/**
 * The current user is allowed to update its own user profile
 */
@Component
public class UserProfileUpdatePolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(UsersAction.USER_PROFILE_UPDATE)) {
            return false;
        }

        var user = (User) context.getObj();
        return context.authenticated() && (user.getId() == context.getSubject().getId());
    }

    @Override
    public Specification checkAsSpec(PolicyContext context) {
        if (!context.matches(UsersAction.USER_PROFILE_UPDATE)) {
            return Specification.where(null);
        }

        return accessesOwnUser(context.getSubject());
    }

    public static Specification<User> accessesOwnUser(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(ex.get(User_.id),
            principal != null ? principal.getId() : -1);
    }
}
