package de.hsrm.sls.subato.auth.policy.users;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;


/**
 * The current user is allowed to view a certain user
 */
@Component
public class UserViewPolicy implements PolicyRule {

    @Override
    public boolean check(PolicyContext context) {
        if (!context.matches(UsersAction.USER_VIEW)) {
            return false;
        }

        var user = (User) context.getObj();
        return context.authenticated() && (user.getId() == context.getSubject().getId()
            || context.getSubject().hasRole(Role.Staff) || context.getSubject().hasAnyGroup());
    }

    @Override
    public Specification checkAsSpec(PolicyContext context) {
        if (!context.matches(UsersAction.USER_VIEW)) {
            return Specification.where(null);
        }

        return accessesOwnUser(context.getSubject()).or(isStaff(context.getSubject()))
            .or(isCourseStaff(context.getSubject()));
    }

    public static Specification<User> accessesOwnUser(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(ex.get(User_.id),
            principal != null ? principal.getId() : -1);
    }

    public static Specification<User> isStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(cb.literal(true),
            principal != null && principal.hasRole(Role.Staff));
    }

    public static Specification<User> isCourseStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(cb.literal(true),
            principal != null && principal.hasAnyGroup());
    }
}
