package de.hsrm.sls.subato.auth.stream;

import de.hsrm.sls.subato.auth.stream.token.StreamTokenConfig;
import java.util.regex.Pattern;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

/**
 * attempts to find all signed urls in the given string and sanitizes them by stripping the token.
 */
@Component
public class SignedUrlSanitizer {

    public String sanitize(String content) {
        if (StringUtils.isBlank(content)) {
            return content;
        }

        // content may be html or markdown, so we try to do that with regex to keep things simple.
        // the regex searches for the query parameter that contains the token. the token is a UUID.
        String regex = "(&|\\?)%s=[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}".formatted(
            StreamTokenConfig.QUERY_PARAM_NAME);

        Pattern pattern = Pattern.compile(regex);
        return pattern.matcher(content).replaceAll("");
    }
}
