package de.hsrm.sls.subato.auth.stream;

import de.hsrm.sls.subato.auth.SecurityConfig;
import de.hsrm.sls.subato.auth.stream.token.InvalidStreamTokenException;
import de.hsrm.sls.subato.auth.stream.token.StreamTokenManager;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletRequestWrapper;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.web.savedrequest.HttpSessionRequestCache;
import org.springframework.security.web.savedrequest.RequestCache;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

/**
 * Intercepts requests before they reach the controller method to "activate" the stream token, if
 * it's present.
 * <p>
 * This filter has to be configured AFTER the JWT has been processed because stream tokens can be
 * ignored for authenticated users.
 */
@Component
public class StreamFilter extends OncePerRequestFilter {

    private class RequestWithoutQueryString extends HttpServletRequestWrapper {

        public RequestWithoutQueryString(HttpServletRequest request) {
            super(request);
        }

        @Override
        public String getQueryString() {
            return "";
        }
    }

    @Autowired
    private StreamTokenManager tokenManager;

    private final RequestCache requestCache = new HttpSessionRequestCache();

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response,
        FilterChain filterChain) throws ServletException, IOException {

        try {
            tokenManager.activate(request);
        } catch (InvalidStreamTokenException ex) {
            // store current requested url for auto redirect after successful login
            // but remove the invalid token from the query string to prevent infinite loops
            var modifiedRequest = new RequestWithoutQueryString(request);
            requestCache.saveRequest(modifiedRequest, response);

            // redirect to login
            String redirectUrl =
                request.getContextPath() + SecurityConfig.OAUTH_LOGIN_URL;
            response.sendRedirect(redirectUrl);

            // stop the filter chain
            return;
        }

        filterChain.doFilter(request, response);
    }
}