package de.hsrm.sls.subato.auth.stream.token;

import java.time.LocalDateTime;

class StreamTokenContext {

    /**
     * the original (relative) url without the token as query param
     */
    private final String originalUrl;

    /**
     * username of the user that is associated with the token
     */
    private final String username;

    /**
     * represented as UUID
     */
    private final String token;

    /**
     * date/time when the token was generated
     */
    private final LocalDateTime issuedAt;

    /**
     * the signed (relative) url, including the token as query param
     */
    private final String signedUrl;


    public StreamTokenContext(String signedUrl, String username, String originalUrl, String token,
        LocalDateTime issuedAt) {
        this.signedUrl = signedUrl;
        this.username = username;
        this.originalUrl = originalUrl;
        this.issuedAt = issuedAt;
        this.token = token;
    }

    public LocalDateTime getIssuedAt() {
        return issuedAt;
    }

    public String getSignedUrl() {
        return signedUrl;
    }

    public String getUsername() {
        return username;
    }

    public String getToken() {
        return token;
    }

    public String getOriginalUrl() {
        return originalUrl;
    }

    public boolean isExpired(long expiration) {
        var expirationDate = issuedAt.plusMinutes(expiration);
        var now = LocalDateTime.now();
        return now.isAfter(expirationDate);
    }

    public StreamTokenContext refresh() {
        return new StreamTokenContext(
            signedUrl,
            username,
            originalUrl,
            token,
            LocalDateTime.now()
        );
    }

    @Override
    public String toString() {
        return "StreamTokenContext{" +
            "signedUrl='" + signedUrl + '\'' +
            ", username='" + username + '\'' +
            ", originalUrl='" + originalUrl + '\'' +
            ", token='" + token + '\'' +
            ", issuedAt=" + issuedAt +
            '}';
    }
}
