package de.hsrm.sls.subato.auth.stream.token;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.LocalDateTime;
import java.util.UUID;
import org.springframework.stereotype.Service;

/**
 * Generates a new token and appends the token to the url.
 */
@Service
class StreamTokenGenerator {

    public StreamTokenContext generate(String url, String username) {
        var token = UUID.randomUUID().toString();

        try {
            // Parse the URL
            URI uri = new URI(url);

            // Get the existing query string
            String existingQuery = uri.getQuery();

            // Append the new query parameter
            String newQuery;
            if (existingQuery == null) {
                newQuery = StreamTokenConfig.QUERY_PARAM_NAME + "=" + token;
            } else {
                newQuery = existingQuery + "&" + StreamTokenConfig.QUERY_PARAM_NAME + "="
                    + token;
            }

            URI updatedUri = new URI(uri.getScheme(), uri.getAuthority(), uri.getPath(), newQuery,
                uri.getFragment());
            return new StreamTokenContext(
                updatedUri.toString(),
                username,
                url,
                token,
                LocalDateTime.now()
            );
        } catch (URISyntaxException e) {
            throw new RuntimeException("Generating token failed because the url is invalid", e);
        }
    }
}
