package de.hsrm.sls.subato.auth.websocket;

import de.hsrm.sls.subato.auth.oauth.JwtAuthConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.messaging.Message;
import org.springframework.messaging.simp.SimpMessageType;
import org.springframework.security.authorization.AuthorizationManager;
import org.springframework.security.config.annotation.web.socket.EnableWebSocketSecurity;
import org.springframework.security.messaging.access.intercept.MessageMatcherDelegatingAuthorizationManager;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationProvider;

@Configuration
@EnableWebSocketSecurity
class WebSocketSecurityConfig {

    @Autowired
    private ExerciseAcceptanceAuthorizationManager manager;

    @Bean
    AuthorizationManager<Message<?>> authorizationManager(
        MessageMatcherDelegatingAuthorizationManager.Builder messages) {

        messages.simpTypeMatchers(SimpMessageType.CONNECT,
                SimpMessageType.HEARTBEAT,
                SimpMessageType.UNSUBSCRIBE,
                SimpMessageType.DISCONNECT)
            .permitAll().simpDestMatchers("/topic/exercise/{\\d+}/acceptance").access(manager)
            .anyMessage().authenticated();
        return messages.build();
    }


    @Bean
    JwtAuthenticationProvider webSocketAuthenticationProvider(
        JwtDecoder decoder,
        JwtAuthConverter authConverter) {
        var authenticationProvider = new JwtAuthenticationProvider(decoder);
        authenticationProvider.setJwtAuthenticationConverter(authConverter);
        return authenticationProvider;
    }
}