package de.hsrm.sls.subato.courses.course.dto;

import de.hsrm.sls.subato.courses.course.model.CourseVisibility;
import de.hsrm.sls.subato.courses.course.model.SubmitPolicy;
import java.util.ArrayList;
import java.util.List;

public class CreateCourseRequest {

    private final Long id;
    private final String name;
    private final String shortName;
    private final CourseVisibility visibility;
    private final SubmitPolicy submitPolicy;
    private final boolean showEmails;
    private final Long sourceId;
    private final Long termId;
    private final List<Long> moduleIds;
    private final Long courseId;

    private CreateCourseRequest(Long id, String name, String shortName, CourseVisibility visibility,
        SubmitPolicy submitPolicy, boolean showEmails, Long sourceId, Long termId,
        List<Long> moduleIds, Long courseId) {
        this.id = id;
        this.name = name;
        this.shortName = shortName;
        this.visibility = visibility;
        this.submitPolicy = submitPolicy;
        this.showEmails = showEmails;
        this.sourceId = sourceId;
        this.termId = termId;
        this.moduleIds = moduleIds;
        this.courseId = courseId;
    }

    public Long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getShortName() {
        return shortName;
    }

    public CourseVisibility getVisibility() {
        return visibility;
    }

    public SubmitPolicy getSubmitPolicy() {
        return submitPolicy;
    }

    public boolean isShowEmails() {
        return showEmails;
    }

    public Long getSourceId() {
        return sourceId;
    }

    public Long getTermId() {
        return termId;
    }

    public List<Long> getModuleIds() {
        return moduleIds;
    }

    public Long getCourseId() {
        return courseId;
    }

    public static class Builder {

        private Long id;
        private String name;
        private String shortName;
        private CourseVisibility visibility;
        private SubmitPolicy submitPolicy;
        private boolean showEmails;
        private Long sourceId;
        private Long termId;
        private List<Long> moduleIds = new ArrayList<>();
        private Long courseId;

        public Builder setId(Long id) {
            this.id = id;
            return this;
        }

        public Builder setName(String name) {
            this.name = name;
            return this;
        }

        public Builder setShortName(String shortName) {
            this.shortName = shortName;
            return this;
        }

        public Builder setVisibility(CourseVisibility visibility) {
            this.visibility = visibility;
            return this;
        }

        public Builder setSubmitPolicy(SubmitPolicy submitPolicy) {
            this.submitPolicy = submitPolicy;
            return this;
        }

        public Builder setShowEmails(boolean showEmails) {
            this.showEmails = showEmails;
            return this;
        }

        public Builder setSourceId(Long sourceId) {
            this.sourceId = sourceId;
            return this;
        }

        public Builder setTermId(Long termId) {
            this.termId = termId;
            return this;
        }

        public Builder setModuleIds(List<Long> moduleIds) {
            this.moduleIds = moduleIds;
            return this;
        }

        public Builder setCourseId(Long courseId) {
            this.courseId = courseId;
            return this;
        }

        public CreateCourseRequest build() {
            return new CreateCourseRequest(id, name, shortName, visibility, submitPolicy,
                showEmails, sourceId, termId, moduleIds, courseId);
        }
    }
}
