package de.hsrm.sls.subato.courses.course.repository;

import static de.hsrm.sls.subato.courses.course.repository.CourseSpecifications.termEquals;

import de.hsrm.sls.subato.courses.course.model.Course;
import org.springframework.data.jpa.domain.Specification;

public class CourseFilter {

    private final Long termId;
    private final String query;

    private CourseFilter(Long termId, String query) {
        this.termId = termId;
        this.query = query;
    }

    public Long getTermId() {
        return termId;
    }

    public String getQuery() {
        return query;
    }

    public Specification<Course> toSpecification() {
        Specification<Course> spec = Specification.where(null);

        if (termId != null) {
            spec = spec.and(termEquals(termId));
        }

        if (query != null) {
            spec = spec.and(CourseSpecifications.applyQuery(query));
        }

        return spec;
    }

    public static class Builder {

        private Long termId;
        private String query;

        public Builder query(String query) {
            this.query = query;
            return this;
        }

        public Builder termId(Long termId) {
            this.termId = termId;
            return this;
        }

        public CourseFilter build() {
            return new CourseFilter(termId, query);
        }
    }
}
