package de.hsrm.sls.subato.courses.course.usecase;

import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.dto.UpdateCourseDto;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import jakarta.validation.Validation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UpdateCourseUseCase {

    @Autowired
    private CourseRepository courseRepository;

    public Course update(Course course, UpdateCourseDto dto) {
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        course.setDescription(dto.getDescription());
        course.setName(dto.getName());
        course.setShortName(dto.getShortName());
        course.setMotd(dto.getMotd());
        course.setVisibility(dto.getVisibility());
        course.setShowEmails(dto.isShowEmails());
        course.setAllowJoiningUnits(dto.isAllowJoiningUnits());
        course.setDisableEditor(dto.isDisableEditor());
        course.setSubmitPolicy(dto.getSubmitPolicy());

        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(course);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }

        return courseRepository.save(course);
    }
}
