package de.hsrm.sls.subato.courses.event;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.event.dto.CourseUnitEventMapper;
import de.hsrm.sls.subato.courses.event.dto.CourseUnitEventSummaryDto;
import de.hsrm.sls.subato.courses.event.dto.UpdateEventCancellationDto;
import de.hsrm.sls.subato.courses.event.repository.CourseUnitEventRepository;
import de.hsrm.sls.subato.courses.event.usecase.UpdateEventCancellationUseCase;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseUnitEventController {

    @Autowired
    private UpdateEventCancellationUseCase updateEventCancellationUseCase;

    @Autowired
    private CourseUnitEventRepository courseUnitEventRepository;

    @Autowired
    private CourseUnitEventMapper courseUnitEventMapper;

    @Autowired
    private PolicyEvaluator policy;


    @PutMapping("/course_unit_event/{id}/cancellation")
    @SubatoApiResponse(CourseUnitEventNotFoundException.class)
    public CourseUnitEventSummaryDto updateCancellation(@PathVariable Long id,
        @RequestBody UpdateEventCancellationDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var event = courseUnitEventRepository.findById(id)
            .orElseThrow(() -> new CourseUnitEventNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(event.getUnit())
            .setPermission(CoursesAction.COURSE_UNIT_UPDATE).build();
        policy.checkAndRaise(context);

        event = updateEventCancellationUseCase.updateCancellation(event, dto.isCanceled());
        return courseUnitEventMapper.mapSummary(event);
    }
}
