package de.hsrm.sls.subato.courses.file.usecase;

import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.file.CourseFile;
import de.hsrm.sls.subato.courses.file.dto.CreateCourseFileDto;
import de.hsrm.sls.subato.courses.file.repository.CourseFileContentRepository;
import de.hsrm.sls.subato.courses.file.repository.CourseFileRepository;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.mime.MimeDetector;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CreateFileUseCase {

    @Autowired
    private CourseFileRepository courseFileRepository;

    @Autowired
    private CourseFileContentRepository courseFileContentRepository;

    @Autowired
    private MimeDetector mimeDetector;

    public CourseFile create(FileContent content, Course course, CreateCourseFileDto metadata) {
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        var courseFile = new CourseFile();
        courseFile.setDescription(metadata.getDescription());
        courseFile.setCourse(course);
        courseFile.setName(content.getFileName());

        var contentType = mimeDetector.getContentType(content.getFileName());
        var isSourceFile = FileHelper.isSourceFile(contentType);
        courseFile.setSrc(isSourceFile);

        courseFile = courseFileRepository.save(courseFile);
        courseFileContentRepository.store(content, courseFile);

        return courseFile;
    }
}
