package de.hsrm.sls.subato.courses.staff.usecase;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.repository.AddStaffResult;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.staff.CourseStaffMembershipAlreadyExistsException;
import de.hsrm.sls.subato.courses.staff.event.CourseStaffMembershipAddedEvent;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;

@Service
public class AddStaffUseCase {

    @Autowired
    PrincipalService principalService;

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    /**
     * @throws CourseStaffMembershipAlreadyExistsException
     */
    public AddStaffResult addStaff(Course course, User user, StaffGroup group) {
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        if (courseRepository.hasMembership(course, user, group)) {
            throw new CourseStaffMembershipAlreadyExistsException();
        }

        var result = courseRepository.addStaff(course, user, group);
        principalService.evictPrincipal(user);

        applicationEventPublisher.publishEvent(
            new CourseStaffMembershipAddedEvent(this, result.membership()));
        return result;
    }
}
