package de.hsrm.sls.subato.courses.staff.usecase;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.staff.CourseStaffMembershipNotFoundException;
import de.hsrm.sls.subato.courses.staff.event.CourseStaffMembershipRemovedEvent;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;

@Service
public class RemoveStaffUseCase {

    @Autowired
    PrincipalService principalService;

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    /**
     * @throws CourseStaffMembershipNotFoundException
     */
    public void removeStaff(Course course, User user, StaffGroup group) {
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }
        
        var membership = courseRepository.getStaffMembership(course, user, group);

        if (membership == null) {
            throw new CourseStaffMembershipNotFoundException();
        }

        courseRepository.removeStaffMembership(course, membership);
        principalService.evictPrincipal(membership.getUser());

        applicationEventPublisher.publishEvent(
            new CourseStaffMembershipRemovedEvent(this, membership));
    }
}
