package de.hsrm.sls.subato.courses.unit;

import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.user.User;
import java.util.ArrayList;
import java.util.List;

public class CreateCourseUnitRequest {

    private final Course course;
    private final String name;
    private final CourseUnitType type;
    private final Long supplementId;
    private final List<User> participants;
    private final List<User> lecturers;

    private CreateCourseUnitRequest(Course course, String name, CourseUnitType type,
        Long supplementId,
        List<User> participants, List<User> lecturers) {
        this.course = course;
        this.name = name;
        this.type = type;
        this.supplementId = supplementId;
        this.participants = participants;
        this.lecturers = lecturers;
    }

    public Course getCourse() {
        return course;
    }

    public String getName() {
        return name;
    }

    public CourseUnitType getType() {
        return type;
    }

    public Long getSupplementId() {
        return supplementId;
    }

    public List<User> getParticipants() {
        return participants;
    }

    public List<User> getLecturers() {
        return lecturers;
    }

    public static class Builder {

        private Course course;
        private String name;
        private CourseUnitType type;
        private Long supplementId;
        private List<User> participants = new ArrayList<>();
        private List<User> lecturers = new ArrayList<>();

        public Builder setCourse(Course course) {
            this.course = course;
            return this;
        }

        public Builder setName(String name) {
            this.name = name;
            return this;
        }

        public Builder setType(CourseUnitType type) {
            this.type = type;
            return this;
        }

        public Builder setSupplementId(Long supplementId) {
            this.supplementId = supplementId;
            return this;
        }

        public Builder setParticipants(List<User> participants) {
            this.participants = participants;
            return this;
        }

        public Builder setLecturers(List<User> lecturers) {
            this.lecturers = lecturers;
            return this;
        }

        public CreateCourseUnitRequest build() {
            return new CreateCourseUnitRequest(course, name, type, supplementId, participants,
                lecturers);
        }
    }
}
