package de.hsrm.sls.subato.courses.unit.model;

import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEvent;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEventDefinition;
import de.hsrm.sls.subato.user.User;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import java.util.ArrayList;
import java.util.List;

@Entity
@Table(name = "course_unit")
public class CourseUnit {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    private String name;

    private CourseUnitType type;

    @ManyToOne
    private Course course;

    @OneToMany(mappedBy = "unit", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<CourseUnitEventDefinition> eventDefinitions = new ArrayList<>();

    @ManyToMany
    private List<User> lecturers = new ArrayList<>();

    @ManyToMany
    private List<User> participants = new ArrayList<>();

    @OneToMany(mappedBy = "unit", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<CourseUnitEvent> events = new ArrayList<>();

    private Long supplementId;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public CourseUnitType getType() {
        return type;
    }

    public void setType(CourseUnitType type) {
        this.type = type;
    }

    public List<User> getLecturers() {
        return lecturers;
    }

    public void setLecturers(List<User> lecturers) {
        this.lecturers = lecturers;
    }

    public List<User> getParticipants() {
        return participants;
    }

    public void setParticipants(List<User> participants) {
        this.participants = participants;
    }

    public List<CourseUnitEvent> getEvents() {
        return events;
    }

    public void setEvents(List<CourseUnitEvent> events) {
        this.events = events;
    }

    public Course getCourse() {
        return course;
    }

    public void setCourse(Course course) {
        this.course = course;
    }

    public Long getSupplementId() {
        return supplementId;
    }

    public void setSupplementId(Long supplementId) {
        this.supplementId = supplementId;
    }

    public List<CourseUnitEventDefinition> getEventDefinitions() {
        return eventDefinitions;
    }

    public String getFullName() {
        return type == CourseUnitType.Misc
            ? name
            : "%s %s".formatted(type.getLabel(), name);
    }

    public String getShortName() {
        return type == CourseUnitType.Misc
            ? name
            : "%s-%s".formatted(type.getShortLabel(), name);
    }
}
