package de.hsrm.sls.subato.courses.unit.repository;

import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEvent;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserFilter;
import de.hsrm.sls.subato.user.repository.UserQuery;
import de.hsrm.sls.subato.user.repository.UserRepository;
import de.hsrm.sls.subato.user.repository.UserSort;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;

class CourseUnitRepositoryImpl implements CourseUnitRepositoryCustom {

    @Autowired
    UserRepository userRepository;

    @Autowired
    @Lazy
    CourseUnitRepository courseUnitRepository;


    // todo: query with SPEL
    public boolean isParticipant(CourseUnit unit, User user) {
        return unit.getParticipants().stream()
            .filter(u -> u.getId() == user.getId())
            .count() != 0;
    }

    public User getParticipant(CourseUnit unit, long userId) {
        var participant = unit.getParticipants().stream()
            .filter(u -> u.getId() == userId)
            .findFirst().orElse(null);
        return participant;
    }

    public List<User> getParticipants(CourseUnit unit, String query) {
        var filter = new UserFilter.Builder().unitId(unit.getId()).query(query).build();
        var userQuery = new UserQuery.Builder().setFilter(filter).setSort(UserSort.byName())
            .build();

        return userRepository.findAll(userQuery);
    }

    public List<CourseUnitEvent> getEvents(CourseUnit unit) {
        var events = unit.getEvents();
        events.sort(Comparator.comparing(CourseUnitEvent::getTakesPlaceAt));
        return events;
    }

    // todo: refactor
    public List<CourseUnit> getParticipatingUnits(Course course, User user) {
        var units = courseUnitRepository.findParticipatingUnits(course.getId(), user.getId());
        this.applySort(units);
        return units;
    }

    private void applySort(List<CourseUnit> units) {
        units.sort(Comparator.comparing((CourseUnit o) -> o.getType().getOrder())
            .thenComparing(CourseUnit::getName));
    }

    public List<CourseUnit> getLecturingUnits(User user, Course course) {
        var units = course.getUnits().stream()
            .filter(unit -> unit.getLecturers().stream().anyMatch(l -> l.getId() == user.getId()))
            .distinct()
            .collect(Collectors.toList());
        this.applySort(units);
        return units;
    }

    // todo: refactor
    public List<CourseUnit> findByCourse(Course course) {
        var units = courseUnitRepository.findByCourseId(course.getId());
        this.applySort(units);
        return units;
    }
}
