package de.hsrm.sls.subato.courses.unit.usecase;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.unit.exception.UserIsAlreadyParticipantException;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class AddParticipantUseCase {

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private RemoveFromOtherLabsUseCase removeFromOtherLabsUseCase;

    public void add(CourseUnit unit, User user) {
        var course = unit.getCourse();
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        var isParticipant = courseUnitRepository.isParticipant(unit, user);
        if (isParticipant) {
            throw new UserIsAlreadyParticipantException();
        }

        unit.getParticipants().add(user);
        unit = courseUnitRepository.save(unit);

        if (unit.getType() == CourseUnitType.Lab) {
            removeFromOtherLabsUseCase.execute(unit, user);
        }

        principalService.evictPrincipal(user);
    }
}
