package de.hsrm.sls.subato.courses.unit.usecase;

import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.course.repository.CreateUnitResult;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.courses.staff.usecase.AddStaffUseCase;
import de.hsrm.sls.subato.courses.unit.CreateCourseUnitRequest;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.user.User;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CreateUnitUseCase {

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private AddStaffUseCase addStaffUseCase;

    public CreateUnitResult create(CreateCourseUnitRequest request) {
        var course = request.getCourse();
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        var unit = new CourseUnit();
        unit.setSupplementId(request.getSupplementId());
        unit.setCourse(course);
        unit.setName(request.getName());
        unit.setType(request.getType());
        unit.setLecturers(request.getLecturers());
        unit.setParticipants(request.getParticipants());
        unit = courseUnitRepository.save(unit);
        course.getUnits().add(unit);

        switch (unit.getType()) {
            case Lecture ->
                course = addMembershipIfMissing(course, unit.getLecturers(), StaffGroup.ADMIN);
            case Lab ->
                course = addMembershipIfMissing(course, unit.getLecturers(), StaffGroup.LECTURER);
            case Tut ->
                course = addMembershipIfMissing(course, unit.getLecturers(), StaffGroup.TUTOR);
        }

        unit.setCourse(course);
        return new CreateUnitResult(course, unit);
    }

    private Course addMembershipIfMissing(Course course, List<User> users, StaffGroup group) {
        var newCourse = course;
        for (var user : users) {
            if (!courseRepository.hasMembership(course, user, group)) {
                var result = addStaffUseCase.addStaff(course, user, group);
                newCourse = result.course();
            }
        }

        return newCourse;
    }
}
