package de.hsrm.sls.subato.courses.unit.usecase;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.unit.exception.UserIsNotAParticipantException;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class RemoveParticipantUseCase {

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @Autowired
    private PrincipalService principalService;

    public void remove(CourseUnit unit, User user) {
        var course = unit.getCourse();
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        var participant = courseUnitRepository.getParticipant(unit, user.getId());
        if (participant == null) {
            throw new UserIsNotAParticipantException();
        }

        unit.getParticipants().remove(participant);
        unit = courseUnitRepository.save(unit);

        principalService.evictPrincipal(participant);
    }
}
