package de.hsrm.sls.subato.dashboard;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.dashboard.dto.DashboardStateDto;
import de.hsrm.sls.subato.dashboard.dto.DashboardStateMapper;
import de.hsrm.sls.subato.dashboard.usecase.BuildDashboardUseCase;
import de.hsrm.sls.subato.user.repository.UserRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Querschnitt")
public class DashboardController {

    @Autowired
    DashboardStateMapper dashboardStateMapper;

    @Autowired
    BuildDashboardUseCase buildDashboardUseCase;

    @Autowired
    UserRepository userRepository;

    @GetMapping("/dashboard")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Querschnittliche Aggregation von relevanten Daten für Benutzer, wie z.B. deren Kursen, die Anwesenheit darin, Fortschritt, Bewertungsinformationen etc.")
    DashboardStateDto dashboard(@RequestParam Optional<Long> termId,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var dashboard = buildDashboardUseCase.buildFor(termId.orElse(null), principal);

        return dashboardStateMapper.map(dashboard);
    }
}
