package de.hsrm.sls.subato.dashboard.usecase;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.dashboard.model.CourseInfo;
import de.hsrm.sls.subato.dashboard.model.DashboardState;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.util.ArrayList;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class BuildDashboardUseCase {

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private CourseRepository courseRepository;

    public DashboardState buildFor(Long termId, SubatoPrincipal principal) {
        var user = userRepository.findByPrincipal(principal);

        var courseInfos = new ArrayList<CourseInfo>();
        var memberCourses = courseRepository.findCoursesWhereIsMember(user, termId);
        var participatingCourses = courseRepository.findParticipatingCourses(user, termId);

        for (var course : participatingCourses) {
            if (memberCourses.contains(course)) {
                continue;
            }

            var context = new PolicyContext.Builder()
                .setSubject(principal)
                .setObj(course)
                .setPermission(CoursesAction.COURSE_VIEW).build();
            var canViewCourse = policy.check(context);
            if (!canViewCourse) {
                continue;
            }

            var participatingUnits = courseUnitRepository.getParticipatingUnits(course, user);
            participatingUnits = participatingUnits.stream()
                .filter(u -> u.getType() != CourseUnitType.Misc).collect(Collectors.toList());
            var courseInfo = new CourseInfo();
            courseInfo.setParticipant(true);
            courseInfo.setCourse(course);
            courseInfo.setUnits(participatingUnits);

            courseInfos.add(courseInfo);
        }

        for (var course : memberCourses) {
            var lecturingUnits = courseUnitRepository.getLecturingUnits(user, course);

            var courseInfo = new CourseInfo();
            courseInfo.setParticipant(false);
            courseInfo.setCourse(course);
            courseInfo.setUnits(lecturingUnits);

            courseInfos.add(courseInfo);
        }

        return new DashboardState(courseInfos);
    }
}
