package de.hsrm.sls.subato.exercises.exercise.model;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

@Entity
@Table(name = "exercise")
public class Exercise {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    private String name;

    private AssessmentMode assessmentMode;

    @JdbcTypeCode(SqlTypes.LONGVARCHAR)
    private String summary;

    @JdbcTypeCode(SqlTypes.LONGVARCHAR)
    private String description;

    @ManyToOne
    private Course course;

    @OneToMany(mappedBy = "exercise", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<TaskInstance> taskInstances = new ArrayList<>();

    @Column(columnDefinition = "TIMESTAMP")
    private LocalDateTime dueTo;

    @Column(columnDefinition = "TIMESTAMP")
    private LocalDateTime visibleFrom;

    @OneToMany(mappedBy = "exercise", cascade = CascadeType.REMOVE)
    private List<Submission> submissions;

    @Column(unique = true)
    private Long sourceId;

    // LASTSUBMISSION TIMESTAMP

    @OneToMany(mappedBy = "exercise", cascade = CascadeType.REMOVE)
    private List<Solution> solutions;

    private int orderNumber;

    public Exercise() {
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Course getCourse() {
        return course;
    }

    public void setCourse(Course course) {
        this.course = course;
    }

    public List<TaskInstance> getTaskInstances() {
        return taskInstances;
    }

    public void setTaskInstances(List<TaskInstance> taskInstances) {
        this.taskInstances = taskInstances;
    }

    public LocalDateTime getDueTo() {
        return dueTo;
    }

    public void setDueTo(LocalDateTime dueTo) {
        this.dueTo = dueTo;
    }

    public List<Submission> getSubmissions() {
        return submissions;
    }

    public void setSubmissions(List<Submission> submissions) {
        this.submissions = submissions;
    }

    public AssessmentMode getAssessmentMode() {
        return assessmentMode;
    }

    public void setAssessmentMode(AssessmentMode assessmentMode) {
        this.assessmentMode = assessmentMode;
    }

    public LocalDateTime getVisibleFrom() {
        return visibleFrom;
    }

    public void setVisibleFrom(LocalDateTime visibleFrom) {
        this.visibleFrom = visibleFrom;
    }

    public String getSummary() {
        return summary;
    }

    public void setSummary(String summary) {
        this.summary = summary;
    }

    public List<Solution> getSolutions() {
        return solutions;
    }

    public void setSolutions(List<Solution> solutions) {
        this.solutions = solutions;
    }

    public int getOrderNumber() {
        return orderNumber;
    }

    public void setOrderNumber(int orderNumber) {
        this.orderNumber = orderNumber;
    }

    public int getReachablePoints() {
        return taskInstances.stream()
            .filter(ti -> ti.getReachablePoints() != null)
            .map(ti -> ti.getReachablePoints())
            .reduce(0, (total, a) -> total + a);
    }

    public long getTaskInstancesCount() {
        return taskInstances.stream().count();
    }

    public boolean isVisible() {
        if (visibleFrom == null) {
            return true;
        }

        var now = LocalDateTime.now();
        return now.isAfter(visibleFrom) || now.isEqual(visibleFrom);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Exercise exercise = (Exercise) o;
        return id == exercise.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }
}
