package de.hsrm.sls.subato.exercises.exercise.repository;

import static de.hsrm.sls.subato.exercises.exercise.repository.ExerciseSpecifications.applyQuery;
import static de.hsrm.sls.subato.exercises.exercise.repository.ExerciseSpecifications.courseEquals;

import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import org.springframework.data.jpa.domain.Specification;

public class ExerciseFilter {

    private final Long courseId;
    private final String query;

    private ExerciseFilter(Long courseId, String query) {
        this.courseId = courseId;
        this.query = query;
    }

    public Long getCourseId() {
        return courseId;
    }

    public String getQuery() {
        return query;
    }

    public Specification<Exercise> toSpecification() {
        Specification<Exercise> spec = Specification.where(null);
        if (courseId != null) {
            spec = spec.and(courseEquals(courseId));
        }

        if (query != null) {
            spec = spec.and(applyQuery(query));
        }

        return spec;
    }

    public static class Builder {

        private Long courseId;
        private String query;

        public Builder query(String query) {
            this.query = query;
            return this;
        }

        public Builder courseId(Long courseId) {
            this.courseId = courseId;
            return this;
        }

        public ExerciseFilter build() {
            return new ExerciseFilter(courseId, query);
        }
    }
}
