package de.hsrm.sls.subato.exercises.exercise.repository;

import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.jpa.domain.Specification;

class ExerciseRepositoryImpl implements ExerciseRepositoryCustom {

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    @Lazy
    private ExerciseRepository exerciseRepository;

    public List<Exercise> findAll(ExerciseQuery query) {
        Specification<Exercise> spec = createSpec(query);

        var exercises = exerciseRepository.findAll(spec);
        if (query.getSort() != null) {
            exercises.sort(query.getSort());
        }

        return exercises;
    }

    private Specification<Exercise> createSpec(ExerciseQuery query) {
        Specification<Exercise> spec = query.getFilter().toSpecification();

        var policyContext = query.getPolicyContext();
        if (policyContext != null && policyContext.getPermission() != null) {
            var authSpec = policy.checkAsSpec(query.getPolicyContext());
            spec = spec.and(authSpec);
        }

        return spec;
    }
}
