package de.hsrm.sls.subato.exercises.taskinstance.repository;

import static de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSpecifications.applyQuery;
import static de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSpecifications.courseEquals;
import static de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSpecifications.exerciseEquals;
import static de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSpecifications.taskEquals;

import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import org.springframework.data.jpa.domain.Specification;

public class TaskInstanceFilter {

    private final Long courseId;
    private final Long exerciseId;
    private final Long taskId;
    private final String query;

    private TaskInstanceFilter(Long courseId, Long exerciseId, Long taskId, String query) {
        this.courseId = courseId;
        this.exerciseId = exerciseId;
        this.taskId = taskId;
        this.query = query;
    }

    public Specification<TaskInstance> toSpecification() {
        Specification<TaskInstance> spec = Specification.where(null);
        if (courseId != null) {
            spec = spec.and(courseEquals(courseId));
        }

        if (exerciseId != null) {
            spec = spec.and(exerciseEquals(exerciseId));
        }

        if (taskId != null) {
            spec = spec.and(taskEquals(taskId));
        }

        if (query != null) {
            spec = spec.and(applyQuery(query));
        }

        return spec;
    }

    public static class Builder {

        private Long courseId;
        private Long exerciseId;
        private Long taskId;
        private String query;

        public Builder query(String query) {
            this.query = query;
            return this;
        }

        public Builder courseId(Long courseId) {
            this.courseId = courseId;
            return this;
        }

        public Builder exerciseId(Long exerciseId) {
            this.exerciseId = exerciseId;
            return this;
        }

        public Builder taskId(Long taskId) {
            this.taskId = taskId;
            return this;
        }

        public TaskInstanceFilter build() {
            return new TaskInstanceFilter(courseId, exerciseId, taskId, query);
        }
    }
}
