package de.hsrm.sls.subato.exercises.taskinstance.repository;

import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import java.util.List;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.jpa.domain.Specification;

class TaskInstanceRepositoryImpl implements TaskInstanceRepositoryCustom {

    @Autowired
    @Lazy
    private TaskInstanceRepository taskInstanceRepository;

    public Optional<TaskInstance> findByExerciseAndTask(long exerciseId, long taskId) {
        var filter = new TaskInstanceFilter.Builder()
            .exerciseId(exerciseId)
            .taskId(taskId)
            .build();

        var query = new TaskInstanceQuery.Builder()
            .setFilter(filter)
            .setSort(TaskInstanceSort.byOrderNumber())
            .build();

        var taskInstances = findAll(query);
        return taskInstances.stream().findFirst();
    }

    public List<TaskInstance> findAll(TaskInstanceQuery query) {
        Specification<TaskInstance> spec = query.getFilter().toSpecification();
        var taskInstances = taskInstanceRepository.findAll(spec);
        if (query.getSort() != null) {
            taskInstances.sort(query.getSort());
        }
        return taskInstances;
    }
}
