package de.hsrm.sls.subato.modules.progmodule;

import de.hsrm.sls.subato.modules.term.TermNotFoundException;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Modulverwaltung")
public class ProgramModuleController {

    @Autowired
    private ProgramModuleService programModuleService;

    @Autowired
    private ProgramModuleMapper programModuleMapper;

    @GetMapping("/module")
    @SubatoApiResponse(TermNotFoundException.class)
    @ResponseStatusMapping(exception = TermNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    List<ProgramModuleSummaryDto> getAll(@RequestParam Optional<Long> termId,
        @RequestParam("q") Optional<String> query) {
        var modules = programModuleService.getAllForTerm(termId.orElse(null),
            query.orElse(null));
        return modules.stream().map(i -> programModuleMapper.mapSummary(i))
            .collect(Collectors.toList());
    }
}
