package de.hsrm.sls.subato.modules.progmodule;

import de.hsrm.sls.subato.modules.term.Term;
import de.hsrm.sls.subato.modules.term.TermNotFoundException;
import de.hsrm.sls.subato.modules.term.TermRepository;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ProgramModuleService {

    @Autowired
    private ProgramModuleRepository programModuleRepository;

    @Autowired
    private TermRepository termRepository;

    private List<ProgramModule> getModulesForTerm(Term term) {
        return programModuleRepository.findAll().stream()
            .filter(m -> term == null || m.plannableForTerm(term))
            .collect(Collectors.toList());
    }

    /**
     * @throws de.hsrm.sls.subato.modules.term.TermNotFoundException
     */
    List<ProgramModule> getAllForTerm(Long termId, String query) {
        var term = termId != null
            ? termRepository.findById(termId)
            .orElseThrow(() -> new TermNotFoundException(termId))
            : null;
        var modules = getModulesForTerm(term);

        if (query != null) {
            modules = modules.stream().filter(
                    m -> m.getName().toLowerCase().contains(query.toLowerCase()) ||
                        m.getShortName().toLowerCase().contains(query.toLowerCase()))
                .collect(Collectors.toList());
        }

        modules.sort(Comparator.comparing(
            (ProgramModule programModule) -> programModule.getProgram().getName()).thenComparing(
            Comparator.comparing(ProgramModule::getScheduledTerm)
                .thenComparing(ProgramModule::getName)));
        return modules;
    }
}
