package de.hsrm.sls.subato.modules.term;

import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UpdateTermUseCase {

    @Autowired
    private TermRepository termRepository;

    /**
     * @param dto attributes for the new term
     * @return created term
     * @throws ConstraintViolationException
     */
    public Term create(CreateTermDto dto) {
        var term = new Term();
        updateTerm(term, dto);
        validateTerm(term);

        return termRepository.save(term);
    }

    /**
     * @throws ConstraintViolationException
     * @throws TermNotFoundException
     */
    public Term update(long id, CreateTermDto dto) {
        var term = termRepository.findById(id).orElseThrow(() -> new TermNotFoundException(id));
        updateTerm(term, dto);
        validateTerm(term);

        return termRepository.save(term);
    }

    private void updateTerm(Term term, CreateTermDto dto) {
        term.setName(dto.getName());
        term.setLecturesStartAt(dto.getLecturesStartAt());
        term.setLecturesEndAt(dto.getLecturesEndAt());
    }

    /**
     * @throws ConstraintViolationException if invalid
     */
    private void validateTerm(Term term) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(term);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }
    }
}
