package de.hsrm.sls.subato.search;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseFilter;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseQuery;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseSort;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceFilter;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceQuery;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceRepository;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSort;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserFilter;
import de.hsrm.sls.subato.user.repository.UserQuery;
import de.hsrm.sls.subato.user.repository.UserRepository;
import de.hsrm.sls.subato.user.repository.UserSort;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SearchUseCase {

    @Autowired
    ExerciseRepository exerciseRepository;

    @Autowired
    TaskInstanceRepository taskInstanceRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PrincipalService principalService;

    // todo: SearchFilter/SearchQuery?

    public List<TaskInstance> searchTaskInstances(Course course, String query) {
        var filter = new TaskInstanceFilter.Builder().courseId(course.getId()).query(query).build();
        var taskInstanceQuery = new TaskInstanceQuery.Builder()
            .setFilter(filter)
            .setSort(TaskInstanceSort.byOrderNumber()).build();
        return taskInstanceRepository.findAll(taskInstanceQuery);
    }

    public List<Exercise> searchExercises(Course course, String query) {
        var filter = new ExerciseFilter.Builder().courseId(course.getId()).query(query).build();
        var context = new PolicyContext.Builder().setSubject(principalService.getPrincipal())
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        var exerciseQuery = new ExerciseQuery.Builder()
            .setPolicyContext(context)
            .setFilter(filter)
            .setSort(ExerciseSort.byId())
            .build();

        return exerciseRepository.findAll(exerciseQuery);
    }

    public List<User> searchCourseParticipants(String query, Course course) {
        var filter = new UserFilter.Builder().query(query).courseId(course.getId()).build();
        var userQuery = new UserQuery.Builder().setFilter(filter).setSort(UserSort.byName())
            .build();

        return userRepository.findAll(userQuery);
    }
}
