package de.hsrm.sls.subato.seed;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategies.SnakeCaseStrategy;
import de.hsrm.sls.subato.seed.logic.course.CourseSeedingLogic;
import de.hsrm.sls.subato.seed.logic.exercise.ExerciseSeedingLogic;
import de.hsrm.sls.subato.seed.logic.language.LanguageSeedingLogic;
import de.hsrm.sls.subato.seed.logic.progmodule.ProgramModuleSeedingLogic;
import de.hsrm.sls.subato.seed.logic.program.DegreeProgramSeedingLogic;
import de.hsrm.sls.subato.seed.logic.taskpool.TaskPoolSeedingLogic;
import de.hsrm.sls.subato.seed.logic.term.TermSeedingLogic;
import de.hsrm.sls.subato.seed.logic.user.UserSeedingLogic;
import java.nio.file.Path;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
class SeedUseCase {

    private static Logger logger = LoggerFactory.getLogger(SeedUseCase.class);

    @Autowired
    private UserSeedingLogic userSeedingLogic;

    @Autowired
    private TermSeedingLogic termSeedingLogic;

    @Autowired
    private LanguageSeedingLogic languageSeedingLogic;

    @Autowired
    private DegreeProgramSeedingLogic degreeProgramSeedingLogic;

    @Autowired
    private ProgramModuleSeedingLogic programModuleSeedingLogic;

    @Autowired
    private CourseSeedingLogic courseSeedingLogic;

    @Autowired
    private TaskPoolSeedingLogic taskPoolSeedingLogic;

    @Autowired
    private ExerciseSeedingLogic exerciseSeedingLogic;

    @Autowired
    private ObjectMapper objectMapper;

    public void seed(String seedDirPath) {
        var seedDir = Path.of(seedDirPath).toFile();
        if (!seedDir.exists()) {
            logger.info(
                "Seed directory %s does not exist, nothing to seed.".formatted(seedDirPath));
            return;
        }

        logger.info("Seeding from directory %s".formatted(seedDirPath));

        var mapper = objectMapper.copy();
        mapper.setPropertyNamingStrategy(new SnakeCaseStrategy());

        var seedingLogics = List.of(
            termSeedingLogic,
            languageSeedingLogic,
            degreeProgramSeedingLogic,
            programModuleSeedingLogic,
            userSeedingLogic,
            courseSeedingLogic,
            taskPoolSeedingLogic,
            exerciseSeedingLogic
        );

        logger.info("Starting to seed");
        for (var logic : seedingLogics) {
            logic.run(seedDir, mapper);
        }

        logger.info("Seeding finished.");
    }
}
