package de.hsrm.sls.subato.seed.logic.language;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.tasks.lang.Language;
import de.hsrm.sls.subato.tasks.lang.LanguageRepository;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class LanguageSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(LanguageSeedingLogic.class);

    @Autowired
    LanguageRepository languageRepository;

    public void run(File seedDir, ObjectMapper objectMapper) {
        var languagesFile = seedDir.toPath().resolve("languages.json").toFile();
        if (!languagesFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(languagesFile);
            var languageSeeds = objectMapper.readValue(json,
                new TypeReference<List<LanguageSeed>>() {
                });

            logger.debug("Seeding %s languages...".formatted(languageSeeds.size()));

            for (var languageSeed : languageSeeds) {
                if (languageSeed.getId() != null && languageRepository.existsById(
                    languageSeed.getId())) {
                    continue;
                }

                var language = new Language();
                language.setId(languageSeed.getId());
                language.setName(languageSeed.getName());

                languageRepository.save(language);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
