package de.hsrm.sls.subato.seed.logic.progmodule;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.modules.progmodule.ProgramModule;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class ProgramModuleSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(ProgramModuleSeedingLogic.class);

    @Autowired
    ProgramModuleRepository programModuleRepository;

    @Autowired
    DegreeProgramRepository programRepository;

    public void run(File seedDir, ObjectMapper objectMapper) {
        var programModulesFile = seedDir.toPath().resolve("progmodules.json").toFile();
        if (!programModulesFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(programModulesFile);
            var programModuleSeeds = objectMapper.readValue(json,
                new TypeReference<List<ProgramModuleSeed>>() {
                });
            logger.debug("Seeding %s program modules...".formatted(programModuleSeeds.size()));

            for (var programModuleSeed : programModuleSeeds) {
                if (programModuleSeed.getId() != null
                    && programModuleRepository.existsBySupplementId(
                    programModuleSeed.getId())) {
                    continue;
                }

                var programModule = new ProgramModule();
                programModule.setSupplementId(programModuleSeed.getId());
                programModule.setName(programModuleSeed.getName());
                programModule.setShortName(programModuleSeed.getShortName());
                programModule.setFrequency(programModuleSeed.getFrequency());
                programModule.setScheduledTerm(programModuleSeed.getScheduledTerm());

                var program = programRepository.findBySupplementId(programModuleSeed.getProgramId())
                    .orElse(null);
                programModule.setProgram(program);

                programModuleRepository.save(programModule);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
