package de.hsrm.sls.subato.seed.logic.program;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.modules.program.DegreeProgram;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class DegreeProgramSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(DegreeProgramSeedingLogic.class);

    @Autowired
    DegreeProgramRepository programRepository;

    public void run(File seedDir, ObjectMapper objectMapper) {
        var programsFile = seedDir.toPath().resolve("programs.json").toFile();
        if (!programsFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(programsFile);
            var programSeeds = objectMapper.readValue(json,
                new TypeReference<List<DegreeProgramSeed>>() {
                });
            logger.debug("Seeding %s programs...".formatted(programSeeds.size()));

            for (var programSeed : programSeeds) {
                if (programSeed.getId() != null && programRepository.existsBySupplementId(
                    programSeed.getId())) {
                    continue;
                }

                var program = new DegreeProgram();
                program.setSupplementId(programSeed.getId());
                program.setName(programSeed.getName());
                program.setShortName(programSeed.getShortName());
                program.setPosCode(programSeed.getPosCode());
                program.setPoYear(programSeed.getPoYear());
                program.setActive(true);

                programRepository.save(program);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
