package de.hsrm.sls.subato.seed.logic.taskpool;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.pool.repository.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.SyncTaskPoolUseCase;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class TaskPoolSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(TaskPoolSeedingLogic.class);

    @Autowired
    private SyncTaskPoolUseCase syncTaskPoolUseCase;

    @Autowired
    private TaskPoolRepository taskPoolRepository;

    public void run(File seedDir, ObjectMapper objectMapper) {
        var taskPoolsFile = seedDir.toPath().resolve("task_pools.json").toFile();
        if (!taskPoolsFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(taskPoolsFile);
            var poolSeeds = objectMapper.readValue(json, new TypeReference<List<TaskPoolSeed>>() {
            });

            logger.debug("Seeding %s task pools...".formatted(poolSeeds.size()));

            for (var poolSeed : poolSeeds) {
                if (poolSeed.getId() != null && taskPoolRepository.existsBySourceId(
                    poolSeed.getId())) {
                    continue;
                }

                var pool = new TaskPool();
                pool.setName(poolSeed.getName());
                pool.setDescription(poolSeed.getDescription());
                pool.setRepositoryUrl(poolSeed.getRepositoryUrl());
                pool.setSourceId(poolSeed.getId());
                pool = taskPoolRepository.save(pool);

                if (pool.isLinkedToRepository()) {
                    syncTaskPoolUseCase.sync(pool, false);
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
