package de.hsrm.sls.subato.seed.logic.term;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.modules.term.Term;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class TermSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(TermSeedingLogic.class);

    @Autowired
    TermRepository termRepository;

    public void run(File seedDir, ObjectMapper objectMapper) {
        var termsFile = seedDir.toPath().resolve("terms.json").toFile();
        if (!termsFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(termsFile);
            var termSeeds = objectMapper.readValue(json, new TypeReference<List<TermSeed>>() {
            });
            logger.debug("Seeding %s terms...".formatted(termSeeds.size()));

            for (var termSeed : termSeeds) {
                if (termSeed.getId() != null && termRepository.existsBySourceId(termSeed.getId())) {
                    continue;
                }

                var term = new Term();
                term.setSourceId(termSeed.getId());
                term.setName(termSeed.getName());
                term.setLecturesStartAt(termSeed.getLecturesStartAt());
                term.setLecturesEndAt(termSeed.getLecturesEndAt());

                termRepository.save(term);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
