package de.hsrm.sls.subato.seed.logic.user;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class UserSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(UserSeedingLogic.class);

    @Autowired
    UserRepository userRepository;

    @Autowired
    DegreeProgramRepository programRepository;

    public void run(File seedDir, ObjectMapper objectMapper) {
        var termsFile = seedDir.toPath().resolve("users.json").toFile();
        if (!termsFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(termsFile);
            var userSeeds = objectMapper.readValue(json, new TypeReference<List<UserSeed>>() {
            });
            logger.debug("Seeding %s users...".formatted(userSeeds.size()));

            for (var userSeed : userSeeds) {
                if (userRepository.findByLoginNameIgnoreCase(userSeed.getLoginName()).isPresent()) {
                    continue;
                }

                var user = new User();
                user.setLoginName(userSeed.getLoginName());
                user.setFirstName(userSeed.getFirstName());
                user.setLastName(userSeed.getLastName());
                user.setMail(userSeed.getMail());
                user.setActive(true);

                if (userSeed.getProgramId() != null) {
                    var program = programRepository.findBySupplementId(userSeed.getProgramId())
                        .orElse(null);
                    user.setProgram(program);
                }

                userRepository.save(user);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
