package de.hsrm.sls.subato.shared;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.shared.helper.HttpServletRequestHelper;
import de.hsrm.sls.subato.shared.helper.UriHelper;
import jakarta.servlet.http.HttpServletRequest;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.regex.Pattern;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class HtmlUrlRewriter {

    @Autowired
    private SubatoConfig subatoConfig;

    /**
     * Replaces relative urls in the html content
     *
     * @param html            to search for relative urls
     * @param replaceFunction taking the base url (e.g. http://localhost:8081) and the relative url
     *                        (e.g. images/star.png)
     * @param request         for deriving the base url
     * @return the transformed html
     */
    public String replaceRelativeUrlsInHtml(String html,
        BiFunction<String, String, String> replaceFunction,
        HttpServletRequest request) {
        var baseUrls = HttpServletRequestHelper.getBaseUrls(request, subatoConfig);
        var baseUrl = baseUrls.get(0);

        return replaceUrlsInHtml(html,
            url -> UriHelper.isRelative(url) ? replaceFunction.apply(baseUrl, url) : url
        );
    }

    /**
     * replaces urls in the given html (only those, who appear in the src-attribute, e.g. in
     * img-tags)
     *
     * @param html             to search for urls
     * @param replacerFunction function that receives the original url and should return the new
     *                         url
     * @return the transformed html
     */
    public String replaceUrlsInHtml(String html, Function<String, String> replacerFunction) {
        var matcher = Pattern.compile("src=\"(\\S+)\"").matcher(html);
        return matcher.replaceAll(r -> {
            var matchedUrl = r.group(1);
            var replacedUrl = replacerFunction.apply(matchedUrl);
            return "src=\"%s\"".formatted(replacedUrl);
        });
    }
}