package de.hsrm.sls.subato.shared;

import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.filter.Filter;
import ch.qos.logback.core.spi.FilterReply;

/**
 * suppresses invalid log messages from dependencies
 */
public class LogFilter extends Filter<ILoggingEvent> {

    @Override
    public FilterReply decide(ILoggingEvent event) {
        if (isWrongLogMessage(event)) {
            return FilterReply.DENY;
        }

        return FilterReply.NEUTRAL;
    }

    // This line (and following lines) log an error, which is not an actual error:
    // https://github.com/spring-projects/spring-framework/blob/main/spring-websocket/src/main/java/org/springframework/web/socket/messaging/StompSubProtocolHandler.java#L333
    // Explanation: handleError() will be called in the catch-block. This method will call the StompSubProtocolErrorHandler
    // which is responsible for formatting the error, that will be sent to the client.
    private boolean isWrongLogMessage(ILoggingEvent event) {
        return event.getLoggerName()
            .endsWith("org.springframework.web.socket.messaging.StompSubProtocolHandler")
            && event.getMessage()
            .startsWith("Failed to send message to MessageChannel in session");
    }
}





