package de.hsrm.sls.subato.shared;


import java.util.Arrays;
import java.util.stream.StreamSupport;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.env.OriginTrackedMapPropertySource;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.core.env.AbstractEnvironment;
import org.springframework.core.env.EnumerablePropertySource;
import org.springframework.core.env.Environment;
import org.springframework.core.env.MutablePropertySources;
import org.springframework.stereotype.Component;

@Component
public class StartupPropertyLogger {

    private static final Logger LOGGER = LoggerFactory.getLogger(StartupPropertyLogger.class);

    @EventListener
    public void handleContextRefresh(ContextRefreshedEvent event) {
        final Environment env = event.getApplicationContext().getEnvironment();
        LOGGER.info("====== Configuration ======");
        LOGGER.info("Active profiles: {}", Arrays.toString(env.getActiveProfiles()));
        final MutablePropertySources sources = ((AbstractEnvironment) env).getPropertySources();
        StreamSupport.stream(sources.spliterator(), false)
            .filter(ps -> ps instanceof OriginTrackedMapPropertySource)
            .map(ps -> ((EnumerablePropertySource) ps).getPropertyNames())
            .flatMap(Arrays::stream)
            .distinct()
            .sorted()
            .forEach(prop -> {
                var censor =
                    prop.contains("credentials") || prop.contains("password") || prop.contains(
                        "token") || prop.contains("secret");
                var val = env.getProperty(prop);
                LOGGER.info("{}: {}", prop, censor ? "*".repeat(val.length()) : val);
            });
        LOGGER.info("===========================================");
    }
}