package de.hsrm.sls.subato.shared.config;

import com.giffing.bucket4j.spring.boot.starter.config.cache.SyncCacheResolver;
import com.giffing.bucket4j.spring.boot.starter.config.cache.jcache.JCacheCacheResolver;
import com.github.benmanes.caffeine.cache.Caffeine;
import com.github.benmanes.caffeine.jcache.configuration.CaffeineConfiguration;
import io.micrometer.core.instrument.Gauge;
import io.micrometer.core.instrument.MeterRegistry;
import java.util.concurrent.TimeUnit;
import javax.cache.Caching;
import javax.cache.spi.CachingProvider;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.caffeine.CaffeineCache;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Profile;

@Configuration
@EnableCaching
public class CacheConfig {

    // https://github.com/MarcGiffing/bucket4j-spring-boot-starter/issues/73
    @Bean
    @Primary
    @Profile("default")
    // skip creation in integration/unit tests because this bean conflicts with the default cache resolver
    public SyncCacheResolver bucket4jCacheResolver() {
        final CachingProvider cachingProvider = Caching.getCachingProvider();
        CaffeineConfiguration<Object, Object> configuration = new CaffeineConfiguration<>();
        //configuration.setExpireAfterWrite(OptionalLong.of(Duration.ofHours(1).toNanos()));
        //configuration.setMaximumSize(OptionalLong.of(1000000));
        var cacheManager = cachingProvider.getCacheManager();
        cacheManager.createCache("rateLimit", configuration);
        return new JCacheCacheResolver(cacheManager);
    }

    @Bean
    public CaffeineCache currentUserCache(MeterRegistry registry) {
        var cache = new CaffeineCache("currentUser",
            Caffeine.newBuilder()
                .expireAfterAccess(5, TimeUnit.MINUTES)
                .build());

        Gauge.builder("subato.user.count", () -> {
            // cleanup is required to remove evicted entries from the cache
            // this will not happen automatically because of performance reasons
            cache.getNativeCache().cleanUp();
            return cache.getNativeCache().estimatedSize();
        }).register(registry);

        return cache;
    }

    @Bean
    public CaffeineCache evaluatorsCache() {
        return new CaffeineCache("evaluators",
            Caffeine.newBuilder()
                .expireAfterAccess(5, TimeUnit.MINUTES)
                .build());
    }

}
