package de.hsrm.sls.subato.shared.error.mapping;

import de.hsrm.sls.subato.shared.error.response.ErrorResponse;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;

@Component
class AccessDeniedRemapper implements ExceptionMapper {

    @Override
    public ErrorResponse map(Throwable source) {
        var isAuthenticated = !(SecurityContextHolder.getContext()
            .getAuthentication() instanceof AnonymousAuthenticationToken);

        if (isAuthenticated) {
            return new ErrorResponse(HttpStatus.FORBIDDEN.value(), "Keine Rechte für diese Aktion.",
                null);
        } else {
            return new ErrorResponse(HttpStatus.UNAUTHORIZED.value(),
                "Nicht eingeloggt.", null);
        }
    }

    @Override
    public boolean canMap(Throwable source) {
        return source instanceof AccessDeniedException;
    }
}
