package de.hsrm.sls.subato.shared.error.mapping;

import de.hsrm.sls.subato.shared.error.response.ErrorResponse;
import jakarta.servlet.MultipartConfigElement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MaxUploadSizeExceededException;

@Component
class MaxUploadSizeExceededMapper implements ExceptionMapper {

    @Autowired
    MultipartConfigElement multipartConfigElement;

    @Override
    public ErrorResponse map(Throwable source) {
        var maxFileSizeInMb = multipartConfigElement.getMaxFileSize() / 1024 / 1024;
        return new ErrorResponse(HttpStatus.BAD_REQUEST.value(),
            "Diese Datei überschreitet das Limit von (etwa) %s MB".formatted(maxFileSizeInMb),
            null);
    }

    @Override
    public boolean canMap(Throwable source) {
        return source instanceof MaxUploadSizeExceededException;
    }
}
