package de.hsrm.sls.subato.shared.error.mapping;

import de.hsrm.sls.subato.shared.error.response.ErrorResponse;
import java.util.HashMap;
import java.util.Map;
import org.springframework.stereotype.Component;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.servlet.NoHandlerFoundException;

/**
 * maps default spring exceptions such as {@link NoHandlerFoundException} and
 * {@link HttpRequestMethodNotSupportedException}
 */
@Component
class SpringDefaultMapper implements ExceptionMapper {

    /**
     * for overriding the default message provided by spring
     */
    private Map<Class, String> messagesOverride = new HashMap<>();

    public SpringDefaultMapper() {
        messagesOverride.put(NoHandlerFoundException.class, "Hier ist nichts.");
    }


    @Override
    public ErrorResponse map(Throwable source) {
        var s = (org.springframework.web.ErrorResponse) source;

        return new ErrorResponse(s.getStatusCode().value(),
            messagesOverride.getOrDefault(source.getClass(), s.getBody().getDetail()), null);
    }

    @Override
    public boolean canMap(Throwable source) {
        return source instanceof org.springframework.web.ErrorResponse;
    }
}
