package de.hsrm.sls.subato.shared.error.rendering;

import java.util.List;
import org.springframework.util.MimeType;

/**
 * Responsible for rendering the exception to the client.
 * <p>
 * We need different representations of the exception, that depend on the context of the request. If
 * the exception occured directly by navigation in the browser, we need to show more user friendly
 * html representations or perform redirects via {@link HtmlExceptionRenderer}. Otherwise we need a
 * machine readable representation via {@link JsonExceptionRenderer}.
 */
public interface ExceptionRenderer {

    /**
     * Actual rendering. May return objects of any type supported by
     * {@link org.springframework.web.bind.annotation.ExceptionHandler}
     */
    Object render(ExceptionRendererArgs args);

    /**
     * decides if the exception renderer should be responsible for rendering the exception.
     */
    List<MimeType> supportedContentTypes();
}
