package de.hsrm.sls.subato.shared.error.rendering;

import de.hsrm.sls.subato.auth.SecurityConfig;
import de.hsrm.sls.subato.shared.config.SubatoConfig;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.security.web.savedrequest.HttpSessionRequestCache;
import org.springframework.security.web.savedrequest.RequestCache;
import org.springframework.stereotype.Component;
import org.springframework.util.MimeType;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

/**
 * see {@link ExceptionRenderer}
 */
@Component
public class HtmlExceptionRenderer implements ExceptionRenderer {

    private final RequestCache requestCache = new HttpSessionRequestCache();

    @Autowired
    private SubatoConfig subatoConfig;

    @Value("${spring.profiles.active:}")
    private String activeProfile;

    @Override
    public Object render(ExceptionRendererArgs args) {
        // if the user is not authenticated, we redirect to the login instead of rendering a message.
        if (args.getErrorResponse().getStatusCode() == HttpStatus.UNAUTHORIZED.value()) {
            // https://github.com/spring-projects/spring-security/issues/11082
            requestCache.saveRequest(args.getRequest(), args.getResponse());
            return new RedirectView(
                SecurityConfig.OAUTH_LOGIN_URL);
        }

        var status = HttpStatus.valueOf(args.getErrorResponse().getStatusCode());
        ModelAndView modelAndView = new ModelAndView();
        modelAndView.setStatus(status);
        modelAndView.addObject("errorMessage", args.getErrorResponse().getReasonPhrase());
        modelAndView.addObject("status", status.value());
        modelAndView.addObject("contactUrl", subatoConfig.getContactUrl());
        modelAndView.addObject("additionalInfo", args.getErrorResponse().getData());
        modelAndView.addObject("backgroundColor",
            activeProfile.equals("prod") ? "#623ebe" : "#006d79");
        modelAndView.setViewName("error");
        return modelAndView;
    }

    public List<MimeType> supportedContentTypes() {
        return List.of(MimeTypeUtils.TEXT_HTML);
    }
}
