package de.hsrm.sls.subato.shared.error.response;

/**
 * An immutable data structure representing HTTP error response bodies. JSON representation of this
 * class would be something like the following:
 *
 * @author Ali Dehghani
 */
public class ErrorResponse {

    /**
     * HTTP status code
     */
    private final int statusCode;

    /**
     * human-readable/application-level error message (German)
     * <p>
     * May be displayed to the user.
     */
    private final String reasonPhrase;

    /**
     * application-level error code. Null in many cases. We introduce new error codes if necessary.
     * <p>
     * Using these errors we provide more information about the actual error.
     */
    private final ErrorCode errorCode;

    /**
     * additional information for the client to process this error in more detail.
     */
    private final Object data;

    public ErrorResponse(int statusCode, String reason, ErrorCode errorCode, Object data) {
        this.statusCode = statusCode;
        this.reasonPhrase = reason;
        this.errorCode = errorCode;
        this.data = data;
    }

    public ErrorResponse(int statusCode, String reasonPhrase, ErrorCode errorCode) {
        this(statusCode, reasonPhrase, errorCode, null);
    }

    public int getStatusCode() {
        return statusCode;
    }

    public String getReasonPhrase() {
        return reasonPhrase;
    }

    public ErrorCode getErrorCode() {
        return errorCode;
    }

    public Object getData() {
        return data;
    }

    @Override
    public String toString() {
        return "ErrorResponse{" +
            "statusCode=" + statusCode +
            ", reasonPhrase='" + reasonPhrase + '\'' +
            ", errorCode=" + errorCode +
            ", data=" + data +
            '}';
    }
}