package de.hsrm.sls.subato.shared.error.response;

import de.hsrm.sls.subato.shared.exceptions.ReasonProvider;
import de.hsrm.sls.subato.shared.springext.ResponseStatusResolver;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.method.HandlerMethod;

@Component
public class ErrorResponseBuilder {

    @Autowired
    ErrorDataSerializer errorDataSerializer;

    public ErrorResponse buildForException(Throwable ex, HandlerMethod handlerMethod) {
        var responseStatus = ResponseStatusResolver.resolve(ex.getClass(), handlerMethod);
        var code = responseStatus.code();
        var reason = getReason(ex, responseStatus);
        var errorCode = getErrorCode(ex);
        var errorData = errorDataSerializer.serialize(ex);
        return new ErrorResponse(code.value(), reason, errorCode, errorData);
    }

    private String getReason(Throwable ex, ResponseStatus responseStatus) {
        var reason = responseStatus.reason();

        if (ex instanceof ReasonProvider reasonProvider) {
            var dynamicReason = reasonProvider.getReason();
            if (dynamicReason != null) {
                reason = dynamicReason;
            }
        }

        return reason;
    }

    private ErrorCode getErrorCode(Throwable ex) {
        var errorAnnotation = AnnotationUtils.findAnnotation(ex.getClass(), Error.class);
        if (errorAnnotation == null) {
            return null;
        }

        return errorAnnotation.value();
    }
}
