package de.hsrm.sls.subato.shared.exceptions;

import de.hsrm.sls.subato.shared.error.response.Error;
import de.hsrm.sls.subato.shared.error.response.ErrorCode;
import de.hsrm.sls.subato.shared.error.response.Expose;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * if the change requested by an user has dangerous consequences and should be explicity confirmed
 * by the user.
 */
@ResponseStatus(code = HttpStatus.BAD_REQUEST, reason = "Die ausgeführte Aktion hat schwere Folgen und muss explizit bestätigt werden")
@Error(ErrorCode.DANGEROUS_ACTION)
public class DangerousActionException extends RuntimeException {

    /**
     * description of the consequences
     */
    @Expose
    private String description;

    /**
     * suggestion for the user, what would be a better solution
     */
    @Expose
    private String suggestion;

    public DangerousActionException(String description, String suggestion) {
        this.description = description;
        this.suggestion = suggestion;
    }

    @Expose
    public String getDescription() {
        return description;
    }

    @Expose
    public String getSuggestion() {
        return suggestion;
    }

    @Override
    public String getMessage() {
        return "description = %s, suggestion = %s".formatted(description, suggestion);
    }
}
