package de.hsrm.sls.subato.shared.fs;

import de.hsrm.sls.subato.tasks.stef.spec.SupportedArchiveExtensions;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import org.rauschig.jarchivelib.ArchiverFactory;
import org.rauschig.jarchivelib.FileType;
import org.springframework.web.multipart.MultipartFile;

public class FileContent {

    private InputStream stream;
    private String fileName;

    public FileContent(InputStream stream, String fileName) {
        this.stream = stream;
        this.fileName = fileName;
    }

    public String getFileName() {
        return fileName;
    }

    public InputStream getStream() {
        return stream;
    }

    public static FileContent fromMultipartFile(MultipartFile file) {
        try {
            return new FileContent(file.getInputStream(), file.getOriginalFilename());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static FileContent fromFile(File file) {
        try {
            return new FileContent(new FileInputStream(file), file.getName());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static FileContent fromString(String name, String content) {
        var stream = new ByteArrayInputStream(content.getBytes(StandardCharsets.UTF_8));
        return new FileContent(stream, name);
    }

    public void saveTo(File dir) {
        saveTo(dir, false);
    }

    public void saveTo(File dir, boolean extractIfArchive) {
        try {
            var dirPath = dir.toPath();
            var path = dirPath.resolve(fileName);
            Files.createDirectories(dirPath);
            Files.copy(stream, path);

            if (extractIfArchive) {
                var fileType = FileType.get(path.toFile());
                var supportedExtensions = SupportedArchiveExtensions.getSupportedExtensions();
                var archiveFormatSupported = supportedExtensions.stream()
                    .anyMatch(e -> ".%s".formatted(e).equals(fileType.getSuffix()));

                if (fileType.isArchive() && archiveFormatSupported) {
                    extractTo(path.toFile(), dir);
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void extractTo(File file, File destination) {
        try {
            var archiver = ArchiverFactory.createArchiver(file);
            archiver.extract(file, destination);
            Files.delete(file.toPath());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
