package de.hsrm.sls.subato.shared.fs;

import com.fasterxml.jackson.annotation.JsonIgnore;
import java.io.File;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.function.Predicate;
import org.apache.commons.io.FilenameUtils;

public class FileTree {

    private UUID uuid;

    private boolean isSrc;
    private String name;

    /**
     * relative to the base path, where this file tree was created from
     */
    private String path;
    private String contentType;

    private boolean isDirectory;

    private List<FileTree> children = new ArrayList<>();

    public FileTree() {
    }

    public FileTree(String name, String path, String contentType, boolean isSrc) {
        this.name = name;
        this.path = path;
        this.contentType = contentType;
        this.uuid = UUID.randomUUID();
        this.isSrc = isSrc;
        this.children = new ArrayList<>();
    }

    public boolean isSrc() {
        return isSrc;
    }

    public void setSrc(boolean src) {
        isSrc = src;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public UUID getUuid() {
        return uuid;
    }

    public void setUuid(UUID uuid) {
        this.uuid = uuid;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public List<FileTree> getChildren() {
        return children;
    }

    public void setChildren(List<FileTree> children) {
        this.children = children;
    }

    public boolean isDirectory() {
        return isDirectory;
    }

    public void setDirectory(boolean directory) {
        isDirectory = directory;
    }

    public String getContentType() {
        return contentType;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    @JsonIgnore
    public String getAbsolutePath(Path basePath) {
        return basePath.resolve(path).toString();
    }

    @JsonIgnore
    public File getFile(Path basePath) {
        return Path.of(getAbsolutePath(basePath)).toFile();
    }

    public FileTree find(Predicate<FileTree> pred) {
        for (var item : children) {
            if (pred.test(item)) {
                return item;
            }

            var match = item.find(pred);
            if (match != null) {
                return match;
            }
        }

        return null;
    }

    public FileTree find(String path) {
        return find(i -> i.getPath().equals(path));
    }

    @JsonIgnore
    public String getExtension() {
        return FilenameUtils.getExtension(name);
    }
}
