package de.hsrm.sls.subato.shared.fs;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.mime.MimeMapping;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class FileTreeCache {

    @Autowired
    FileTreeService fileTreeService;

    private FileTree buildTree(Path path, List<MimeMapping> mimeMappingOverrides) {
        var file = path.toFile();

        if (!file.exists()) {
            throw new RuntimeException("file/directory does not exist: " + path);
        }

        if (!file.isDirectory()) {
            throw new RuntimeException("path is not a directory");
        }

        return fileTreeService.createTreeFrom(file, "", true, false, mimeMappingOverrides);
    }

    private synchronized void saveTree(FileTree tree, Path basePath) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            var serialized = objectMapper.writeValueAsString(tree);
            var treeFile = getTreeIndexPath(basePath).toFile();
            Files.write(treeFile.toPath(), serialized.getBytes(StandardCharsets.UTF_8));
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public Path getTreeIndexPath(Path basePath) {
        return basePath.resolve("tree.json");
    }

    public synchronized FileTree getTree(Path basePath, List<MimeMapping> mimeMappingOverrides) {
        var cacheFilePath = getTreeIndexPath(basePath);
        if (!cacheFilePath.toFile().exists()) {
            var tree = buildTree(basePath, mimeMappingOverrides);
            saveTree(tree, basePath);
            return tree;
        }

        var objectMapper = new ObjectMapper();
        try {
            var treeContent = FileHelper.readString(cacheFilePath.toFile());
            return objectMapper.readValue(treeContent, FileTree.class);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
