package de.hsrm.sls.subato.shared.fs;

import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.mime.MimeDetector;
import de.hsrm.sls.subato.shared.mime.MimeMapping;
import java.io.File;
import java.nio.file.Path;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class FileTreeService {

    @Autowired
    MimeDetector mimeDetector;

    public FileTree createTreeFrom(File file, String basePath, boolean recursive,
        boolean includeRoot, List<MimeMapping> mimeMappingOverrides) {
        if (!file.exists()) {
            return null;
        }

        var contentType = mimeDetector.getContentType(file, mimeMappingOverrides);
        var isSourceFile = FileHelper.isSourceFile(contentType);

        var fileName = includeRoot ? file.getName() : "";
        var item = new FileTree(fileName, Path.of(basePath).resolve(fileName).toString(),
            contentType, isSourceFile);

        if (file.isDirectory()) {
            item.setDirectory(true);

            if (recursive) {
                var relativePath = Path.of(basePath).resolve(Path.of(fileName));

                for (var f : file.listFiles()) {
                    item.getChildren().add(
                        createTreeFrom(f, relativePath.toString(), recursive, true,
                            mimeMappingOverrides));
                }
            }
        }

        return item;
    }
}
