package de.hsrm.sls.subato.shared.helper;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.MalformedInputException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

public class FileHelper {

    public static boolean isSourceFile(String contentType) {
        var contentTypes = List.of(
            "application/json",
            "application/xml",
            "application/javascript"
        );

        return contentType != null ? contentType.startsWith("text/") || contentTypes.contains(
            contentType) : false;
    }

    public static String readString(File file) {
        try {
            // https://stackoverflow.com/questions/26268132/all-inclusive-charset-to-avoid-java-nio-charset-malformedinputexception-input
            Charset[] charsets = {StandardCharsets.UTF_8, StandardCharsets.ISO_8859_1};
            for (var c : charsets) {
                try {
                    return Files.readString(file.toPath(), c);
                } catch (MalformedInputException ex) {
                }
            }

            throw new RuntimeException(
                "encoding not supported: file %s".formatted(file.toPath()));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static byte[] toBytes(File file) {
        try {
            return Files.readAllBytes(file.toPath());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
