package de.hsrm.sls.subato.shared.helper;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import org.apache.commons.lang3.StringUtils;

public class HttpServletRequestHelper {

    public static List<String> getBaseUrls(HttpServletRequest request, SubatoConfig subatoConfig) {
        var hostname = getHostname(request, subatoConfig);
        var port = getPort(request, subatoConfig);

        if (port == 80 || port == 443) {
            return List.of(
                createUrl(request.getScheme(), hostname, null),
                createUrl(request.getScheme(), hostname, port)
            );
        } else {
            return List.of(
                createUrl(request.getScheme(), hostname, port)
            );
        }
    }

    private static String createUrl(String scheme, String hostname, Integer port) {
        var sb = new StringBuilder();
        sb.append(scheme + "://");
        sb.append(hostname);
        if (port != null) {
            sb.append(":" + port);
        }
        //sb.append("/");
        return sb.toString();
    }

    private static String getHostname(HttpServletRequest request, SubatoConfig subatoConfig) {
        if (StringUtils.isBlank(subatoConfig.getPublicHostname())) {
            return request.getServerName();
        }

        return subatoConfig.getPublicHostname().split(":")[0];
    }

    private static int getPort(HttpServletRequest request, SubatoConfig subatoConfig) {
        if (StringUtils.isBlank(subatoConfig.getPublicHostname())) {
            return request.getServerPort();
        }

        var splitted = subatoConfig.getPublicHostname().split(":");
        return splitted.length >= 2 ? Integer.parseInt(splitted[1]) : request.getServerPort();
    }
}
