package de.hsrm.sls.subato.shared.helper;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import jakarta.servlet.http.HttpServletRequest;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;

public class UriHelper {

    public static String concat(String base, Path path) {
        try {
            var uri = new URL(base);
            return new URL(uri.getProtocol(), uri.getHost(), uri.getPort(),
                Path.of(uri.getPath()).resolve(path).toString(), null).toString();
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }
    }

    public static boolean isRelative(String url) {
        return !url.startsWith("/") && !url.startsWith("https://")
            && !url.startsWith("http://");
    }

    public static Map<String, String> parseQueryParameters(String url) {
        Map<String, String> paramMap = new HashMap<>();

        try {
            URI uri = new URI(url);
            String query = uri.getQuery();
            if (query != null) {
                String[] pairs = query.split("&");
                for (String pair : pairs) {
                    int idx = pair.indexOf("=");

                    if (idx == -1) {
                        // ?continue (added by Spring)
                        continue;
                    }

                    String key = URLDecoder.decode(pair.substring(0, idx), "UTF-8");
                    String value = URLDecoder.decode(pair.substring(idx + 1), "UTF-8");
                    paramMap.put(key, value);
                }
            }

            return paramMap;
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    public record UrlParseResult(String baseUrl, String absoluteUrl, String relativeUrl) {

    }

    public static UrlParseResult parseUrl(String url, HttpServletRequest request,
        SubatoConfig subatoConfig) {
        var baseUrls = HttpServletRequestHelper.getBaseUrls(request, subatoConfig);

        if (UriHelper.isRelative(url)) {
            var baseUrl = baseUrls.get(0);
            return new UrlParseResult(baseUrl, null, url);
        } else {
            for (var baseUrl : baseUrls) {
                if (url.startsWith(baseUrl)) {
                    var relativeUrl = url.replace(baseUrl, "");
                    return new UrlParseResult(baseUrl, url, relativeUrl);
                }
            }

            return null;
        }
    }


}
