package de.hsrm.sls.subato.shared.helper;

import java.io.File;
import java.nio.file.Path;
import java.util.UUID;
import net.lingala.zip4j.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import net.lingala.zip4j.model.ExcludeFileFilter;
import net.lingala.zip4j.model.ZipParameters;

public class ZipHelper {

    public static File zip(File file, File target, boolean includeRootFolder,
        ExcludeFileFilter filter) {
        try {
            if (target.exists()) {
                target.delete();
            }

            ZipFile zipFile = new ZipFile(target);
            var params = new ZipParameters();
            params.setIncludeRootFolder(includeRootFolder);
            params.setExcludeFileFilter(filter);
            zipFile.createSplitZipFileFromFolder(file, params, false, 0);

            return target;
        } catch (ZipException e) {
            throw new RuntimeException(e);
        }
    }

    public static File zip(File file, File target, boolean includeRootFolder) {
        return zip(file, target, includeRootFolder, null);
    }

    public static File zip(File file, boolean includeRootFolder, ExcludeFileFilter filter) {
        var uuid = UUID.randomUUID();
        var tempFile = Path.of(System.getProperty("java.io.tmpdir")).resolve(uuid.toString())
            .toFile();
        tempFile.deleteOnExit();

        return zip(file, tempFile, includeRootFolder, filter);
    }

    public static File zip(File file, boolean includeRootFolder) {
        return zip(file, includeRootFolder, null);
    }
}
