package de.hsrm.sls.subato.shared.markdown.latex;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import org.commonmark.node.Node;
import org.commonmark.renderer.NodeRenderer;
import org.commonmark.renderer.html.HtmlNodeRendererContext;
import org.commonmark.renderer.html.HtmlWriter;

/**
 * We insert the original math expression as is into the html doc.
 */
class LatexMathBlockHtmlNodeRenderer implements NodeRenderer {

    private final HtmlWriter html;
    private final HtmlNodeRendererContext context;

    public LatexMathBlockHtmlNodeRenderer(HtmlNodeRendererContext context) {
        this.html = context.getWriter();
        this.context = context;
    }

    @Override
    public Set<Class<? extends Node>> getNodeTypes() {
        return new HashSet<>(Arrays.asList(
            LatexMathBlock.class
        ));
    }

    @Override
    public void render(Node node) {
        if (node instanceof LatexMathBlock block) {
            this.html.text(
                "$".repeat(block.signs()) +
                    block.latex().strip() +
                    "$".repeat(block.signs())
            );
        }
    }
}
