package de.hsrm.sls.subato.shared.response;

import de.hsrm.sls.subato.shared.mime.MimeDetector;
import java.nio.charset.StandardCharsets;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.io.Resource;
import org.springframework.http.ContentDisposition;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;

public class FileResponseBuilder {

    private final Resource resource;
    private final MimeDetector mimeDetector;

    private String fileName;
    private Boolean asAttachment;

    FileResponseBuilder(Resource resource, MimeDetector mimeDetector) {
        this.resource = resource;
        this.mimeDetector = mimeDetector;
    }

    public FileResponseBuilder setFileName(String fileName) {
        this.fileName = fileName;
        return this;
    }

    public FileResponseBuilder setAsAttachment(Boolean asAttachment) {
        this.asAttachment = asAttachment;
        return this;
    }

    public ResponseEntity<Resource> build() {
        var finalFileName = fileName == null ? resource.getFilename() : fileName;
        if (StringUtils.isBlank(finalFileName)) {
            finalFileName = "unnamed";
        }

        String contentType = mimeDetector.getContentType(finalFileName);
        var contentDisposition = ContentDisposition
            .builder(asAttachment ? "attachment" : "inline")
            .filename(finalFileName, StandardCharsets.UTF_8)
            .build();

        var headers = new HttpHeaders();
        headers.setContentDisposition(contentDisposition);

        return ResponseEntity.ok().contentType(MediaType.parseMediaType(contentType))
            .headers(headers)
            .body(resource);
    }
}
