package de.hsrm.sls.subato.shared.response;

import de.hsrm.sls.subato.shared.helper.ZipHelper;
import de.hsrm.sls.subato.shared.mime.MimeDetector;
import java.io.File;
import java.io.IOException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.stereotype.Component;

@Component
public class FileResponseBuilderFactory {

    @Autowired
    private MimeDetector mimeDetector;

    public FileResponseBuilder forFile(File file) {
        if (file.isDirectory()) {
            var zipFile = ZipHelper.zip(file, true);
            var resource = load(zipFile);
            return new FileResponseBuilder(resource, mimeDetector).setFileName(
                "%s.zip".formatted(file.getName()));
        } else {
            var resource = load(file);
            return new FileResponseBuilder(resource, mimeDetector);
        }
    }

    private static Resource load(File file) {
        try {
            Resource resource = new UrlResource(file.toURI());

            if (resource.exists() || resource.isReadable()) {
                return resource;
            } else {
                throw new RuntimeException("Could not read the file!");
            }
        } catch (IOException e) {
            throw new RuntimeException("Error: " + e.getMessage());
        }
    }
}
