package de.hsrm.sls.subato.shared.springext.hm;

import jakarta.annotation.Nullable;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

/**
 * If {@link HandlerMethod} is declared as parameter on a method annotated with
 * {@link org.springframework.web.bind.annotation.ExceptionHandler}, the default resolution fails
 * with an {@link IllegalStateException} if there is no handlerMethod that threw the exception. This
 * behavior will prevent the exception handler from being executed. That could be a bug in the
 * Spring framework.
 * <p>
 * When this resolver is configured, exception handlers with a {@link HandlerMethod} parameter
 * annotated with {@link Nullable} will get executed because they get a null value in this case.
 * <p>
 */
@Component
public class NullableHandlerMethodArgumentResolver implements HandlerMethodArgumentResolver {

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.getParameterType().equals(HandlerMethod.class)
            && parameter.getParameterAnnotation(
            Nullable.class) != null;
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer,
        NativeWebRequest webRequest, WebDataBinderFactory binderFactory) {
        return null;
    }
}
