package de.hsrm.sls.subato.shared.swagger;

import de.hsrm.sls.subato.shared.error.response.Error;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * @param response
 * @param responseStatus
 */
record ResolvedResponse(SubatoApiResponse response, ResponseStatus responseStatus) {

    public String generateErrorIdentifier() {
        var errorAnnotation = AnnotationUtils.findAnnotation(response.value(),
            Error.class);
        if (errorAnnotation != null) {
            return errorAnnotation.value().toString();
        } else {
            return response.value().getSimpleName().replace("Exception", "");
        }
    }

    public String getDescription() {
        var s = responseStatus.reason();
        var errorAnnotation = AnnotationUtils.findAnnotation(response.value(),
            Error.class);
        if (errorAnnotation != null) {
            s += " (`%s`)".formatted(generateErrorIdentifier());
        }

        return s;
    }
}
