package de.hsrm.sls.subato.shared.versioning;

import com.vdurmont.semver4j.Semver;
import com.vdurmont.semver4j.Semver.VersionDiff;
import com.vdurmont.semver4j.SemverException;
import de.hsrm.sls.subato.shared.ApiConstants;
import de.hsrm.sls.subato.shared.CustomHttpHeaderConstants;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

/**
 * Responsible for checking if the specified version in the corresponding HTTP header of requests is
 * compatible with the current version. Clients SHOULD specify the version in the corresponding
 * header. If no version is specified, then this filter does nothing. If the version is specified
 * and the Major part differs, then this filter rejects the request by throwing an
 * {@link UnsupportedApiVersionException} that can be handled in clients to provide update
 * instructions.
 */
@Component
public class ApiVersionFilter extends OncePerRequestFilter {


    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response,
        FilterChain filterChain) throws ServletException, IOException {

        var requestedVersion = request.getHeader(CustomHttpHeaderConstants.API_VERSION);
        if (StringUtils.isNotBlank(requestedVersion)) {
            var api = new Semver(ApiConstants.API_VERSION);

            try {
                var specified = new Semver(requestedVersion);

                if (api.diff(specified).equals(VersionDiff.MAJOR)) {
                    throw new UnsupportedApiVersionException(requestedVersion);
                }
            } catch (SemverException ex) {
                throw new UnsupportedApiVersionException(requestedVersion);
            }
        }

        filterChain.doFilter(request, response);
    }
}
