package de.hsrm.sls.subato.shared.versioning;

import com.vdurmont.semver4j.Semver;
import com.vdurmont.semver4j.SemverException;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

/**
 * In contrast to {@link ApiVersionFilter} this filter enforces plugin upgrades, although the
 * current plugin version may be compatible with the API.
 */
@Component
public class PluginVersionFilter extends OncePerRequestFilter {

    @Autowired
    private MinPluginVersionConfig versionConfig;

    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response,
        FilterChain filterChain) throws ServletException, IOException {

        var userAgent = request.getHeader(HttpHeaders.USER_AGENT);

        if (StringUtils.isNotBlank(userAgent) &&
            StringUtils.isNotBlank(versionConfig.getIntellij()) &&
            userAgent.startsWith("SubatoIJ")) {
            var splitted = userAgent.split("/");
            var min = new Semver(versionConfig.getIntellij());

            try {
                var specified = new Semver(splitted[1]);

                if (specified.isLowerThan(min)) {
                    throw new PluginUpgradeRequiredException(splitted[1]);
                }
            } catch (SemverException ex) {
                throw new PluginUpgradeRequiredException(splitted[1]);
            }
        }

        filterChain.doFilter(request, response);
    }
}
