package de.hsrm.sls.subato.shared.websocket;

import de.hsrm.sls.subato.auth.websocket.AuthChannelInterceptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.messaging.simp.config.ChannelRegistration;
import org.springframework.messaging.simp.config.MessageBrokerRegistry;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.web.socket.config.annotation.EnableWebSocketMessageBroker;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketMessageBrokerConfigurer;

@Configuration
@EnableWebSocketMessageBroker
@Order(Ordered.HIGHEST_PRECEDENCE + 99)
// execute before spring security for AuthChannelInterceptor to work
// see https://docs.spring.io/spring-framework/reference/web/websocket/stomp/authentication-token-based.html
public class WebSocketConfig implements WebSocketMessageBrokerConfigurer {

    @Autowired
    AuthChannelInterceptor interceptor;

    @Autowired
    StompErrorHandler stompErrorHandler;

    @Override
    public void configureMessageBroker(MessageBrokerRegistry config) {
        config.enableSimpleBroker("/topic");
        config.setApplicationDestinationPrefixes("/app");
    }

    @Override
    public void registerStompEndpoints(StompEndpointRegistry registry) {
        registry.addEndpoint("/subatows").setAllowedOrigins("*");
        registry.setErrorHandler(stompErrorHandler);
    }

    // https://www.reddit.com/r/SpringBoot/comments/10gabkf/authenticate_the_user_when_using_websockets/
    @Bean(name = "csrfChannelInterceptor")
    ChannelInterceptor csrfChannelInterceptor() {
        return new ChannelInterceptor() {
        };
    }

    @Override
    public void configureClientOutboundChannel(ChannelRegistration registration) {
        //registration.interceptors(interceptor);
    }

    @Override
    public void configureClientInboundChannel(ChannelRegistration registration) {
        registration.interceptors(interceptor);
    }
}