package de.hsrm.sls.subato.solutions.cleanup;

import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import de.hsrm.sls.subato.solutions.solution.usecase.DeleteSolutionUseCase;
import java.time.LocalDateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * responsible for cleaning up anonymous solutions
 */
@Service
class CleanupSolutionsUseCase {

    private static Logger logger = LoggerFactory.getLogger(CleanupSolutionsUseCase.class);

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private DeleteSolutionUseCase deleteSolutionUseCase;

    public void deleteAnonymousSolution() {
        var now = LocalDateTime.now();
        var anonymousSolutions = solutionRepository.findAllBySubmittedByIsNull();

        var deletedSolutions = 0;
        for (var solution : anonymousSolutions) {
            if (now.isAfter(solution.getSubmittedAt().plusMinutes(5))) {
                deleteSolutionUseCase.delete(solution);
                deletedSolutions += 1;
            }
        }

        logger.info("%s/%s anonymous solutions were deleted".formatted(deletedSolutions,
            anonymousSolutions.size()));
    }
}
