package de.hsrm.sls.subato.solutions.comment;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.solutions.comment.dto.CreateSolutionCommentDto;
import de.hsrm.sls.subato.solutions.comment.dto.SolutionCommentDto;
import de.hsrm.sls.subato.solutions.comment.dto.SolutionCommentMapper;
import de.hsrm.sls.subato.solutions.comment.usecase.CreateSolutionCommentUseCase;
import de.hsrm.sls.subato.solutions.comment.usecase.DeleteSolutionCommentUseCase;
import de.hsrm.sls.subato.solutions.solution.SolutionNotFoundException;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Lösungsverwaltung")
class SolutionCommentController {

    @Autowired
    private SolutionCommentMapper solutionCommentMapper;

    @Autowired
    private CreateSolutionCommentUseCase createSolutionCommentUseCase;

    @Autowired
    private DeleteSolutionCommentUseCase deleteSolutionCommentUseCase;

    @Autowired
    private SolutionCommentRepository solutionCommentRepository;

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private PolicyEvaluator policy;

    @PostMapping("/solution_comment")
    @SubatoApiResponse(SolutionNotFoundException.class)
    @ResponseStatusMapping(exception = SolutionNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    SolutionCommentDto create(@RequestBody CreateSolutionCommentDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var solution = solutionRepository.findById(dto.getSolutionId())
            .orElseThrow(() -> new SolutionNotFoundException(dto.getSolutionId()));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(solution)
            .setPermission(SolutionsAction.SOLUTION_COMMENT_CREATE).build();
        policy.checkAndRaise(context);

        var comment = createSolutionCommentUseCase.create(dto, solution, principal);
        return solutionCommentMapper.map(comment);
    }

    @DeleteMapping("/solution_comment/{id}")
    @SubatoApiResponse(SolutionCommentNotFoundException.class)
    void delete(@PathVariable Long id, @AuthenticationPrincipal SubatoPrincipal principal) {
        var comment = solutionCommentRepository.findById(id)
            .orElseThrow(() -> new SolutionCommentNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(comment)
            .setPermission(SolutionsAction.SOLUTION_COMMENT_DELETE).build();
        policy.checkAndRaise(context);

        deleteSolutionCommentUseCase.delete(comment);
    }
}
