package de.hsrm.sls.subato.solutions.comment.usecase;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.solutions.comment.SolutionComment;
import de.hsrm.sls.subato.solutions.comment.SolutionCommentRepository;
import de.hsrm.sls.subato.solutions.comment.dto.CreateSolutionCommentDto;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.time.LocalDateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CreateSolutionCommentUseCase {

    @Autowired
    private SolutionCommentRepository solutionCommentRepository;

    @Autowired
    private UserRepository userRepository;

    public SolutionComment create(CreateSolutionCommentDto dto, Solution solution,
        SubatoPrincipal principal) {
        if (solution.getExercise() != null && solution.getExercise().getCourse().isFinished()) {
            throw new CourseFinishedException();
        }

        var user = userRepository.findByPrincipal(principal);

        var comment = new SolutionComment();
        comment.setText(dto.getText());
        comment.setFrom(user);
        comment.setSolution(solution);
        comment.setCreatedAt(LocalDateTime.now());

        return solutionCommentRepository.save(comment);
    }
}
