package de.hsrm.sls.subato.solutions.evaluation;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategies.SnakeCaseStrategy;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.SimpleAsyncTaskExecutor;

@Configuration
@ConfigurationProperties(prefix = "subato.eva")
public class EvaConfig {

    private String rabbitmqHost;
    private int rabbitmqPort;
    private String registryHost;
    private int registryPort;
    private int timeout;

    @Bean
    public SimpleAsyncTaskExecutor bulkEvalExecutor() {
        return new SimpleAsyncTaskExecutor();
    }

    public String getRegistryHost() {
        return registryHost;
    }

    public void setRegistryHost(String registryHost) {
        this.registryHost = registryHost;
    }

    public String getRabbitmqHost() {
        return rabbitmqHost;
    }

    public void setRabbitmqHost(String rabbitmqHost) {
        this.rabbitmqHost = rabbitmqHost;
    }

    public int getRabbitmqPort() {
        return rabbitmqPort;
    }

    public void setRabbitmqPort(int rabbitmqPort) {
        this.rabbitmqPort = rabbitmqPort;
    }

    public int getRegistryPort() {
        return registryPort;
    }

    public void setRegistryPort(int registryPort) {
        this.registryPort = registryPort;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    @Bean
    CachingConnectionFactory connectionFactory() {
        return new CachingConnectionFactory(rabbitmqHost, rabbitmqPort);
    }

    @Bean("eva2Template")
    public RabbitTemplate eva2Template(
        ConnectionFactory connectionFactory, @Autowired ObjectMapper objectMapper) {
        var mapper = objectMapper.copy();
        mapper.setPropertyNamingStrategy(new SnakeCaseStrategy());

        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setMessageConverter(new Jackson2JsonMessageConverter(mapper));
        rabbitTemplate.setReplyTimeout(timeout);
        rabbitTemplate.setUseDirectReplyToContainer(
            false); // prevent Reply received after timeout exception
        return rabbitTemplate;
    }

}
