package de.hsrm.sls.subato.solutions.evaluation;

import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import java.util.concurrent.atomic.AtomicInteger;
import org.springframework.stereotype.Component;

@Component
public class EvalMonitor {

    private final Counter evaluations;
    private final Counter failedEvaluations;
    private final Counter timeoutEvaluations;
    private final AtomicInteger pendingEvaluations;

    public EvalMonitor(MeterRegistry meterRegistry) {
        timeoutEvaluations = meterRegistry.counter("subato.timeout_evaluations");
        evaluations = meterRegistry.counter("subato.evaluations");
        failedEvaluations = meterRegistry.counter("subato.failed_evaluations");
        pendingEvaluations = meterRegistry.gauge("subato.pending_evaluations",
            new AtomicInteger(0));
    }

    public void start() {
        evaluations.increment();
        pendingEvaluations.incrementAndGet();
    }

    public void done() {
        pendingEvaluations.decrementAndGet();
    }

    public void failure(boolean isTimeout) {
        failedEvaluations.increment();

        if (isTimeout) {
            timeoutEvaluations.increment();
        }
    }
}
