package de.hsrm.sls.subato.solutions.evaluation;

import de.hsrm.sls.subato.solutions.solution.model.Solution;

public class EvalOptions {

    /**
     * solution to evaluate
     */
    private final Solution solution;

    /**
     * if the special acceptance tests (X)OR if the regular tests should be used for evaluation.
     */
    private final TestMode mode;

    private final boolean sendNotificationEvent;

    private EvalOptions(Solution solution, TestMode mode, boolean sendNotificationEvent) {
        this.solution = solution;
        this.mode = mode;
        this.sendNotificationEvent = sendNotificationEvent;
    }

    public Solution getSolution() {
        return solution;
    }

    public TestMode getMode() {
        return mode;
    }

    public boolean isSendNotificationEvent() {
        return sendNotificationEvent;
    }

    public static class Builder {

        private Solution solution;
        private TestMode mode = TestMode.REGULAR;
        private boolean sendNotificationEvent = true;

        public Builder setSolution(Solution solution) {
            this.solution = solution;
            return this;
        }

        public Builder setMode(TestMode mode) {
            this.mode = mode;
            return this;
        }

        public Builder setSendNotificationEvent(boolean sendNotificationEvent) {
            this.sendNotificationEvent = sendNotificationEvent;
            return this;
        }

        public EvalOptions build() {
            return new EvalOptions(solution, mode, sendNotificationEvent);
        }
    }
}
