package de.hsrm.sls.subato.solutions.evaluation.acceptance;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.solutions.evaluation.EvalOptions.Builder;
import de.hsrm.sls.subato.solutions.evaluation.TestMode;
import de.hsrm.sls.subato.solutions.evaluation.result.dto.EvalStateDetailDto;
import de.hsrm.sls.subato.solutions.evaluation.result.dto.EvalStateMapper;
import de.hsrm.sls.subato.solutions.evaluation.usecase.EvalSolutionUseCase;
import de.hsrm.sls.subato.solutions.solution.SolutionNotFoundException;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController("AcceptanceSolutionController")
@Tag(name = "Lösungsverwaltung")
public class SolutionController {

    @Autowired
    private EvalSolutionUseCase evalSolutionUseCase;

    @Autowired
    private EvalStateMapper evalStateMapper;

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private PolicyEvaluator policy;

    @PostMapping("/solution/{id}/acceptance")
    @Operation(summary = "Ausführung der Abnahmeauswertung durch Lehrende")
    @SubatoApiResponse(SolutionNotFoundException.class)
    EvalStateDetailDto testSolution(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var solution = solutionRepository.findById(id)
            .orElseThrow(() -> new SolutionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(solution)
            .setPermission(SolutionsAction.SOLUTION_RUN_ACCEPTANCE_TESTS).build();
        policy.checkAndRaise(context);

        var evalResponse = evalSolutionUseCase.evaluateAndStore(
            new Builder().setSolution(solution).setMode(TestMode.ACCEPTANCE).build());
        return evalStateMapper.mapDetail(evalResponse.getState());
    }
}
