package de.hsrm.sls.subato.solutions.evaluation.bulk;

import de.hsrm.sls.subato.solutions.evaluation.bulk.state.BulkEvalState;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.dto.BulkEvalStateMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.stereotype.Service;

/**
 * Responsible for sending state updates to the client. Currently, the full state object is send on
 * every update. This is quite inefficient in terms of network bandwidth. Maybe we should implement
 * the transmission of partial state changes.
 */
@Service
public class BulkEvalHandler {

    @Autowired
    private SimpMessagingTemplate messagingTemplate;

    @Autowired
    private BulkEvalStateMapper bulkEvalStateMapper;

    public void onStateChange(BulkEvalState state) {
        var dto = bulkEvalStateMapper.map(state);
        messagingTemplate.convertAndSend(
            "/topic/exercise/%s/acceptance".formatted(state.getExerciseId()), dto);
    }
}
