package de.hsrm.sls.subato.solutions.evaluation.bulk;

import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.solutions.evaluation.EvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.TestMode;

public class BulkEvalOptions {

    /**
     * exercise with solutions to evaluate
     */
    private final Exercise exercise;

    /**
     * see {@link EvalOptions}
     */
    private final TestMode mode;

    /**
     * whether solutions should be evaluated again, if a result already exists
     */
    private final boolean reset;

    private BulkEvalOptions(Exercise exercise, TestMode mode, boolean reset) {
        this.exercise = exercise;
        this.mode = mode;
        this.reset = reset;
    }

    public Exercise getExercise() {
        return exercise;
    }

    public TestMode getMode() {
        return mode;
    }

    public boolean isReset() {
        return reset;
    }

    public static class Builder {

        private Exercise exercise;
        private TestMode mode;
        private boolean reset;

        public Builder setExercise(Exercise exercise) {
            this.exercise = exercise;
            return this;
        }

        public Builder setMode(TestMode mode) {
            this.mode = mode;
            return this;
        }

        public Builder setReset(boolean reset) {
            this.reset = reset;
            return this;
        }

        public BulkEvalOptions build() {
            return new BulkEvalOptions(exercise, mode, reset);
        }
    }
}
