package de.hsrm.sls.subato.solutions.evaluation.registry;

import de.hsrm.sls.subato.solutions.evaluation.EvaConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException.NotFound;
import org.springframework.web.client.RestTemplate;

@Service
public class EvaRegistry {

    private static Logger logger = LoggerFactory.getLogger(EvaRegistry.class);

    @Autowired
    EvaConfig evaConfig;

    @Cacheable(value = "evaluators", key = "#ref")
    public EvaluationStack get(String ref) {
        try {
            var restTemplate = new RestTemplate();
            var url = "http://%s:%s/stacks/%s".formatted(evaConfig.getRegistryHost(),
                evaConfig.getRegistryPort(), ref);
            var response = restTemplate.exchange(url, HttpMethod.GET, null, GetStackResponse.class);
            return response.getBody().getStack();
        } catch (NotFound ex) {
            throw new EvaluatorNotFoundException(ref);
        }
    }

    @CacheEvict(value = "evaluators", allEntries = true)
    public void resetCache() {
        logger.debug("cache cleared");
    }
}
