package de.hsrm.sls.subato.solutions.solution.repository;

import static de.hsrm.sls.subato.solutions.solution.repository.SolutionSpecifications.exerciseEquals;
import static de.hsrm.sls.subato.solutions.solution.repository.SolutionSpecifications.isValid;
import static de.hsrm.sls.subato.solutions.solution.repository.SolutionSpecifications.submissionEquals;
import static de.hsrm.sls.subato.solutions.solution.repository.SolutionSpecifications.submittedByEquals;
import static de.hsrm.sls.subato.solutions.solution.repository.SolutionSpecifications.taskEquals;

import de.hsrm.sls.subato.solutions.solution.model.Solution;
import org.springframework.data.jpa.domain.Specification;

/**
 * Filter request for wrapping available filter options in
 * {@link SolutionService#findAll(SolutionFilter)}
 */
public class SolutionFilter {

    private final Long submittedById;
    private final Long exerciseId;
    private final Long taskId;
    private final Long submissionId;
    private final Boolean valid;

    private SolutionFilter(Long submittedById, Long exerciseId, Long taskId, Long submissionId,
        Boolean valid) {
        this.submittedById = submittedById;
        this.exerciseId = exerciseId;
        this.taskId = taskId;
        this.submissionId = submissionId;
        this.valid = valid;
    }

    public Long getSubmittedById() {
        return submittedById;
    }

    public Long getExerciseId() {
        return exerciseId;
    }

    public Long getTaskId() {
        return taskId;
    }

    public Long getSubmissionId() {
        return submissionId;
    }

    public Boolean getValid() {
        return valid;
    }

    public Specification<Solution> toSpecification() {
        Specification<Solution> spec = Specification.where(null);
        if (submittedById != null) {
            spec = spec.and(submittedByEquals(submittedById));
        }

        if (exerciseId != null) {
            spec = spec.and(exerciseEquals(exerciseId));
        }

        if (taskId != null) {
            spec = spec.and(taskEquals(taskId));
        }

        if (submissionId != null) {
            spec = spec.and(submissionEquals(submissionId));
        }

        if (valid != null) {
            spec = spec.and(isValid(valid));
        }

        return spec;
    }

    public static class Builder {

        private Long submittedById;
        private Long exerciseId;
        private Long taskId;
        private Long submissionId;
        private Boolean valid;

        public Builder submittedById(Long submittedById) {
            this.submittedById = submittedById;
            return this;
        }

        public Builder exerciseId(Long exerciseId) {
            this.exerciseId = exerciseId;
            return this;
        }

        public Builder taskId(Long taskId) {
            this.taskId = taskId;
            return this;
        }

        public Builder submissionId(Long submissionId) {
            this.submissionId = submissionId;
            return this;
        }

        public Builder valid(Boolean valid) {
            this.valid = valid;
            return this;
        }

        public SolutionFilter build() {
            return new SolutionFilter(
                submittedById,
                exerciseId,
                taskId,
                submissionId,
                valid
            );
        }
    }
}
