package de.hsrm.sls.subato.solutions.solution.repository;

import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.shared.exceptions.SortingNotSupportedException;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

class SolutionRepositoryImpl implements SolutionRepositoryCustom {

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    @Lazy
    private SolutionRepository solutionRepository;

    public Page<Solution> findAll(SolutionQuery query, Pageable page) {
        Specification<Solution> spec = createSpec(query);
        if (query.getSort() != null) {
            throw new SortingNotSupportedException();
        }

        return solutionRepository.findAll(spec, page);
    }

    public Long getSolutionCountForTaskInstance(TaskInstance taskInstance) {
        var spec = new SolutionFilter.Builder()
            .exerciseId(taskInstance.getExercise().getId())
            .taskId(taskInstance.getTask().getId())
            .build()
            .toSpecification();

        return solutionRepository.count(spec);
    }

    private Specification<Solution> createSpec(SolutionQuery query) {
        Specification<Solution> spec = query.getFilter().toSpecification();

        var policyContext = query.getPolicyContext();
        if (policyContext != null && policyContext.getPermission() != null) {
            var authSpec = policy.checkAsSpec(query.getPolicyContext());
            spec = spec.and(authSpec);
        }

        return spec;
    }
}
