package de.hsrm.sls.subato.solutions.submit;

import de.hsrm.sls.subato.shared.exceptions.ReasonProvider;
import java.util.List;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * Thrown if a submitted file does not meet the allowed file extensions, configured for the target
 * task.
 * <p>
 * Only applies to top level files! see STEF specification
 */
@ResponseStatus(code = HttpStatus.BAD_REQUEST, reason = "Unerlaubter Dateityp")
public class ExtensionNotAllowedException extends RuntimeException implements ReasonProvider {

    private String invalidFileName;
    private List<String> allowedExtensions;

    public ExtensionNotAllowedException(String invalidFileName, List<String> allowedExtensions) {
        this.invalidFileName = invalidFileName;
        this.allowedExtensions = allowedExtensions;
    }

    public String getReason() {
        return "Diese Datei %s wird von der Aufgabe nicht unterstützt, erlaubte Dateitypen sind: %s".formatted(
            invalidFileName, String.join(", ", allowedExtensions));
    }

    @Override
    public String getMessage() {
        return "invalidFileName = %s, allowedExtensions = %s".formatted(invalidFileName,
            allowedExtensions);
    }
}
