package de.hsrm.sls.subato.tasks.imp;

import de.hsrm.sls.subato.tasks.pool.exception.TaskPoolNotFoundException;
import java.nio.file.Path;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

/**
 * Responsible for manually importing tasks (represented as subdirs in the file system) from a
 * certain directory (root_dir) into a task pool (task_pool_id).
 * <p>
 * for starting the import, run: gradle bootRun --args='import:tasks %root_dir% %task_pool_id%'
 * <p>
 * example: gradle bootRun --args='import:tasks /home/fischer/Desktop/TaskImport/OOSE_AI 1' gradle
 * bootRun --args='import:tasks /home/fischer/Desktop/TaskImport/PMT_AI 2'
 */
@Component
class ImportTasksRunner implements CommandLineRunner {

    @Autowired
    ImportTaskUseCase importTaskUseCase;

    private static Logger logger = LoggerFactory.getLogger(ImportTasksRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("import:tasks")) {
            return;
        }
        var rootDir = Path.of(args[1]).toFile();
        var poolId = Long.parseLong(args[2]);

        try {
            importTaskUseCase.importTasks(poolId, rootDir);
        } catch (TaskPoolNotFoundException ex) {
            logger.error("No pool found with id %s".formatted(poolId));
        } catch (TaskImportException ex) {
            logger.error("Error while importing tasks", ex);
        }

        System.exit(0);
    }
}
