package de.hsrm.sls.subato.tasks.lang.usecase;

import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.tasks.lang.Language;
import de.hsrm.sls.subato.tasks.lang.LanguageNotFoundException;
import de.hsrm.sls.subato.tasks.lang.LanguageRepository;
import de.hsrm.sls.subato.tasks.lang.dto.CreateLanguageDto;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class UpdateLanguageUseCase {

    @Autowired
    private LanguageRepository languageRepository;

    /**
     * @param dto attributes for the new language
     * @return created language
     * @throws ConstraintViolationException
     */
    public Language create(CreateLanguageDto dto) {
        var language = new Language();
        updateLanguage(language, dto);
        validateLanguage(language);

        return languageRepository.save(language);
    }

    /**
     * @throws ConstraintViolationException
     * @throws LanguageNotFoundException
     */
    public Language update(String id, CreateLanguageDto dto) {
        var language = languageRepository.findById(id)
            .orElseThrow(() -> new LanguageNotFoundException(id));
        updateLanguage(language, dto);
        validateLanguage(language);

        return languageRepository.save(language);
    }

    private void updateLanguage(Language language, CreateLanguageDto dto) {
        language.setId(dto.getId());
        language.setName(dto.getName());
    }

    /**
     * @throws ConstraintViolationException if invalid
     */
    private void validateLanguage(Language language) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(language);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }
    }
}
