package de.hsrm.sls.subato.tasks.pool;

import de.hsrm.sls.subato.tasks.imp.log.TaskImportLog;
import de.hsrm.sls.subato.tasks.task.Task;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.StringUtils;

/**
 * organizational unit for tasks, covering a specific topic and/or curriculum all pools can be
 * viewed and edited by staff members but a pool is usually administrated by one person
 */
@Entity
@Table(name = "task_pool")
public class TaskPool {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    /**
     * short display name for the pool
     */
    private String name;

    /**
     * short description to be displayed primarily for indicating which topic the pool serves and
     * who is administrating it
     */
    private String description;

    /**
     * url to the git repository from where to import tasks. only repositories of gitlab.cs.hs-rm.de
     * can be used and the 'Subato' bot needs to be given access to the repository repositories used
     * should be contained in the group sls/subato-aufgaben, then the bot automatically has access
     * to it
     * <p>
     * null for manual administrated pools
     */
    private String repositoryUrl;

    /**
     * branch to checkout. if null, the default branch will be used on the next sync. this field
     * will be updated to reflect the name of the default branch after sync.
     */
    private String repositoryBranch;

    @OneToMany(mappedBy = "pool", cascade = CascadeType.REMOVE)
    private List<Task> tasks = new ArrayList<>();

    @OneToMany(mappedBy = "pool", cascade = CascadeType.ALL)
    private List<TaskImportLog> log = new ArrayList<>();

    @Column(columnDefinition = "TIMESTAMP")
    private LocalDateTime lastUpdated;

    @Column(columnDefinition = "TIMESTAMP")
    private LocalDateTime lastChecked;

    @Column(unique = true)
    private Long sourceId;

    public TaskPool() {
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<Task> getTasks() {
        return tasks;
    }

    public void setTasks(List<Task> tasks) {
        this.tasks = tasks;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getRepositoryUrl() {
        return repositoryUrl;
    }

    public void setRepositoryUrl(String repositoryUrl) {
        this.repositoryUrl = repositoryUrl;
    }

    public boolean isLinkedToRepository() {
        return StringUtils.isNotBlank(repositoryUrl);
    }

    public List<TaskImportLog> getLog() {
        return log;
    }

    public LocalDateTime getLastUpdated() {
        return lastUpdated;
    }

    public void setLastUpdated(LocalDateTime lastUpdated) {
        this.lastUpdated = lastUpdated;
    }

    public LocalDateTime getLastChecked() {
        return lastChecked;
    }

    public void setLastChecked(LocalDateTime lastChecked) {
        this.lastChecked = lastChecked;
    }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public String getRepositoryBranch() {
        return repositoryBranch;
    }

    public void setRepositoryBranch(String repositoryBranch) {
        this.repositoryBranch = repositoryBranch;
    }
}
