package de.hsrm.sls.subato.tasks.pool.repository;

import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLog;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.repository.TaskFilter;
import de.hsrm.sls.subato.tasks.task.repository.TaskRepository;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

class TaskPoolRepositoryImpl implements TaskPoolRepositoryCustom {

    @Autowired
    @Lazy
    private TaskPoolRepository taskPoolRepository;

    @Autowired
    private TaskRepository taskRepository;

    public List<TaskPool> getAllWithRepository() {
        return taskPoolRepository.findAll()
            .stream()
            .filter(pool -> pool.isLinkedToRepository())
            .collect(Collectors.toList());
    }

    public Page<Task> getTasks(Pageable page, TaskFilter filter) {
        return taskRepository.getAll(filter, page);
    }

    public List<TaskImportLog> getLogs(TaskPool pool) {
        var logs = pool.getLog();
        logs.sort(Comparator.comparing(TaskImportLog::getAt).reversed());
        return logs;
    }

    /**
     * gets all courses having at least one task from the given task pool
     *
     * @param pool
     * @return courses
     */
    public List<Course> getCourses(TaskPool pool) {
        // todo: improve performance
        var courses = pool.getTasks().stream()
            .map(t -> t.getInstances()).flatMap(Collection::stream)
            .map(ti -> ti.getExercise().getCourse())
            .distinct()
            .collect(Collectors.toList());

        courses.sort((a, b) -> a.getTerm() == null ? -1
            : b.getTerm() == null ? 1 : (int) (b.getTerm().getId() - a.getTerm().getId()));
        return courses;
    }
}
