package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.misclib.MisconceptionRepository;
import de.hsrm.sls.subato.misclib.model.Competence;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
class CompetenceResolver {

    private static Logger logger = LoggerFactory.getLogger(CompetenceResolver.class);

    @Autowired
    private MisconceptionRepository misconceptionRepository;

    public List<Competence> resolve(TaskXml doc) {
        var competences = new ArrayList<Competence>();

        if (doc.getCompetences() == null) {
            return competences;
        }

        var availableCompetences = misconceptionRepository.getCompetences();
        for (var competenceXml : doc.getCompetences().getCompetences()) {
            var matchingCompetence = availableCompetences.stream()
                .filter(c -> c.getId().equals(competenceXml.getId())).findFirst().orElse(null);
            if (matchingCompetence == null) {
                continue;
                /*
                throw new InvalidSpecException(
                    "Kompetenz %s existiert nicht im System".formatted(competenceXml.getId()));
                 */
            }

            competences.add(matchingCompetence);
        }

        return competences;
    }
}
