package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.solutions.evaluation.registry.EvaluationStack;
import de.hsrm.sls.subato.solutions.evaluation.registry.EvaluatorNotFoundException;
import de.hsrm.sls.subato.solutions.evaluation.usecase.ResolveEvaluatorUseCase;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
class EvaluatorResolver {

    @Autowired
    private ResolveEvaluatorUseCase resolveEvaluatorUseCase;

    public EvaluationStack resolve(TaskXml doc) {
        EvaluationStack stack = null;

        if (doc.getEvaluator() != null && !doc.getEvaluator().equals("unknown")) {
            try {
                var ref = doc.getEvaluator();
                stack = resolveEvaluatorUseCase.resolveReference(ref);
            } catch (EvaluatorNotFoundException ex) {
                throw new InvalidSpecException(
                    "Es wurde kein passender Evaluation Stack für %s im System gefunden".formatted(
                        ex.getRef()));
            }
        }

        return stack;
    }
}
