package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.shared.HtmlUrlRewriter;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Fix relative urls to assets in the html content of the task,
 * so that the url points to the endpoint that serves the asset
 * @see TaskController#downloadAsset
 */
@Component
public class TaskContentAssetUrlRewriter {

    @Autowired
    private HtmlUrlRewriter htmlUrlRewriter;

    public String rewiteContent(Task task, HttpServletRequest req) {
        var content = task.getSpec().getText();
        return htmlUrlRewriter.replaceRelativeUrlsInHtml(content,
            (baseUrl, path) -> "%s/task/%s/asset?path=%s".formatted(baseUrl, task.getId(), path),
            req);
    }

    public String rewiteContent(TaskInstance taskInstance, HttpServletRequest req) {
        var content = taskInstance.getTask().getSpec().getText();
        var taskId = taskInstance.getTask().getId();
        var exerciseId = taskInstance.getExercise().getId();

        return htmlUrlRewriter.replaceRelativeUrlsInHtml(content,
            (baseUrl, path) -> "%s/exercise/%s/task/%s/asset?path=%s".formatted(baseUrl, exerciseId,
                taskId, path),
            req);
    }
}