package de.hsrm.sls.subato.tasks.task.exception;

import de.hsrm.sls.subato.shared.exceptions.ReasonProvider;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * the lookup of an asset file in the {@see} STEF specification has failed because the path does not
 * point to an asset
 * <p>
 * assets are defined as images and other media files used in the task text
 */
@ResponseStatus(code = HttpStatus.NOT_FOUND, reason = "Asset existiert nicht")
public class AssetNotFoundException extends RuntimeException implements ReasonProvider {

    private long taskId;
    private String path;
    private boolean malicious;

    public AssetNotFoundException(long taskId, String path, boolean malicious) {
        this.taskId = taskId;
        this.path = path;
        this.malicious = malicious;
    }

    @Override
    public String getReason() {
        return "Unter dem Pfad %s wurde innerhalb der Aufgabe mit ID %s kein Asset gefunden".formatted(
            path, taskId);
    }

    @Override
    public String getMessage() {
        return "taskId = %s, path = %s, malicious = %s".formatted(taskId, path, malicious);
    }
}
