package de.hsrm.sls.subato.tasks.task.repository;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.exception.TaskArchivedException;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class BundleRepository {

    @Autowired
    private SubatoConfig subatoConfig;

    private File getBundleFile(Task task) {
        return Paths.get(subatoConfig.getDataDir())
            .resolve("bundle")
            .resolve(String.valueOf(task.getId()))
            .resolve("%s.zip".formatted(task.getSoftId()))
            .toFile();
    }

    public File getBundle(Task task) {
        if (task.isArchived()) {
            throw new TaskArchivedException();
        }

        return getBundleFile(task);
    }

    public File storeBundle(Task task, File file) {
        try {
            var bundleFile = getBundleFile(task);
            bundleFile.mkdirs();
            Files.delete(bundleFile.toPath());
            Files.copy(file.toPath(), bundleFile.toPath());
            return bundleFile;
        } catch (IOException exception) {
            throw new RuntimeException("Failed to store bundle for task %s".formatted(task.getId()),
                exception);
        } finally {
            try {
                FileUtils.delete(file);
            } catch (IOException e) {
                throw new RuntimeException(
                    "Failed to delete bundle tmp file for task %s".formatted(task.getId()), e);
            }
        }
    }
}
