package de.hsrm.sls.subato.tasks.task.repository;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.shared.helper.ZipHelper;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.task.Task;
import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileSystemUtils;

@Service
public class TaskContentRepository {

    @Autowired
    private SubatoConfig subatoConfig;

    public Path getTaskPath(Task task) {
        return Paths.get(subatoConfig.getDataDir())
            .resolve("task")
            .resolve(String.valueOf(task.getId()));
    }


    /**
     * returns the STEF specification of the task as zip file
     *
     * @param task
     * @return
     */
    public File getSpecAsZip(Task task) {
        var root = getTaskPath(task);
        return ZipHelper.zip(root.toFile(), false);
    }

    public File getFile(Task task, Path path) {
        var basePath = getTaskPath(task);
        var file = basePath.resolve(path).toFile();
        return file;
    }

    public File getFileByRef(Task task, FileRef ref) {
        return ref.getFile().getFile(getTaskPath(task));
    }

    public void delete(Task task) {
        var taskPath = getTaskPath(task);
        FileSystemUtils.deleteRecursively(taskPath.toFile());
    }

    /**
     * stores/replaces the spec, currently stored in the directory dir, for the given task in the
     * subato2-data directory.
     * <p>
     * when importing a spec for an existing task, a backup is created and rolled back, if the
     * import fails
     */
    public void store(Task task, File dir) {
        var taskPath = getTaskPath(task);
        // delete dir for existing task if already exists
        delete(task);

        try {
            FileUtils.copyDirectory(dir, taskPath.toFile());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
