package de.hsrm.sls.subato.tasks.task.repository;

import static de.hsrm.sls.subato.tasks.task.repository.TaskSpecifications.applyQuery;
import static de.hsrm.sls.subato.tasks.task.repository.TaskSpecifications.poolEquals;
import static de.hsrm.sls.subato.tasks.task.repository.TaskSpecifications.showArchived;

import de.hsrm.sls.subato.tasks.task.Task;
import org.springframework.data.jpa.domain.Specification;

public class TaskFilter {

    private final String query;
    private final Long poolId;
    private final Boolean showArchived;

    private TaskFilter(String query, Long poolId, Boolean showArchived) {
        this.query = query;
        this.poolId = poolId;
        this.showArchived = showArchived;
    }

    public String getQuery() {
        return query;
    }

    public Long getPoolId() {
        return poolId;
    }

    public Boolean getShowArchived() {
        return showArchived;
    }

    public Specification<Task> toSpecification() {
        Specification<Task> spec = Specification.where(null);

        if (poolId != null) {
            spec = spec.and(poolEquals(poolId));
        }

        if (query != null) {
            spec = spec.and(applyQuery(query));
        }

        if (showArchived != null) {
            spec = spec.and(showArchived(showArchived));
        }

        return spec;
    }

    public static class Builder {

        private String query;
        private Long poolId;
        private Boolean showArchived;

        public Builder query(String query) {
            this.query = query;
            return this;
        }

        public Builder poolId(Long poolId) {
            this.poolId = poolId;
            return this;
        }

        public Builder setShowArchived(Boolean showArchived) {
            this.showArchived = showArchived;
            return this;
        }

        public TaskFilter build() {
            return new TaskFilter(query, poolId, showArchived);
        }
    }
}
