package de.hsrm.sls.subato.tasks.task.usecase;

import de.hsrm.sls.subato.tasks.imp.log.LogImportUseCase;
import de.hsrm.sls.subato.tasks.imp.log.LogType;
import de.hsrm.sls.subato.tasks.stef.cache.StefSpecCache;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.SpecParser;
import de.hsrm.sls.subato.tasks.task.repository.TaskContentRepository;
import de.hsrm.sls.subato.tasks.task.repository.TaskFilter;
import de.hsrm.sls.subato.tasks.task.repository.TaskRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class VerifyTasksUseCase {

    private static Logger logger = LoggerFactory.getLogger(VerifyTasksUseCase.class);

    @Autowired
    TaskRepository taskRepository;

    @Autowired
    TaskContentRepository taskContentRepository;

    @Autowired
    LogImportUseCase logImportUseCase;

    @Autowired
    private SpecParser specParser;

    @Autowired
    private ArchiveTaskUseCase archiveTaskUseCase;

    public void verifyTasks() {
        logger.info("Started verifying tasks");

        var tasks = taskRepository.getAll(new TaskFilter.Builder().build());
        for (var task : tasks) {
            if (task.isArchived() || task.hasValidSpec()) {
                continue;
            }

            logger.debug("Rebuilding cache for task %s".formatted(task.getId()));

            try {

                var taskPath = taskContentRepository.getTaskPath(task);
                var spec = specParser.parseSpec(taskPath.toString());
                task.setSpecCache(new StefSpecCache(StefSpecCache.SERIALIZATION_VERSION, spec));
                task = taskRepository.save(task);
            } catch (InvalidSpecException e) {
                logger.debug(
                    "Spec for task %s is invalid: %s".formatted(task.getId(), e.getMessage()));

                archiveTaskUseCase.archive(task);
                logger.debug("Archived task %s".formatted(task.getId()));

                if (task.getPool() != null) {
                    logImportUseCase.log(task.getPool(), LogType.PROBLEM,
                        "Verifikation von %s fehlgeschlagen. %s.".formatted(task.getSoftId(),
                            e.getReason()));
                }
            }
        }

        logger.info("Verification of tasks finished");
    }
}
