package de.hsrm.sls.subato.user;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.attendances.attendance.model.Attendance;
import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.courses.participantprofile.model.CourseParticipantProfile;
import de.hsrm.sls.subato.courses.staff.model.CourseStaffMembership;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.modules.program.DegreeProgram;
import de.hsrm.sls.subato.solutions.comment.SolutionComment;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import jakarta.persistence.CascadeType;
import jakarta.persistence.CollectionTable;
import jakarta.persistence.Column;
import jakarta.persistence.ElementCollection;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;

@Entity
@Table(name = "users")
public class User {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @Column(unique = true)
    private String loginName;

    private String firstName;

    private String lastName;

    @Column(unique = true)
    private String mail;

    @OneToMany(mappedBy = "submittedBy")
    private List<Solution> solutions;

    @OneToMany(mappedBy = "submittedBy")
    private List<Submission> submissions;

    @OneToMany(mappedBy = "checkedBy")
    private List<Submission> checkedSubmissions;

    @ManyToMany(mappedBy = "lecturers")
    private List<CourseUnit> lecturingUnits;

    @OneToMany(mappedBy = "student")
    private List<Attendance> attendences;

    @ManyToMany(mappedBy = "participants")
    private List<CourseUnit> participatingUnits = new ArrayList<>();

    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    private List<CourseStaffMembership> staffMemberships = new ArrayList<>();

    @OneToMany(mappedBy = "from")
    private List<SolutionComment> comments;

    @ElementCollection(targetClass = Role.class)
    @CollectionTable(
        name = "user_roles",
        joinColumns = @JoinColumn(name = "user_id")
    )
    @Column(name = "role_id")
    private Set<Role> roles = new HashSet<>();

    @ManyToOne
    private DegreeProgram program;

    private boolean active;

    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    private List<CourseParticipantProfile> participantProfiles = new ArrayList<>();

    private boolean consentedToCollection;
    private int consentChangeCount;

    public User() {
    }

    public String getName() {
        return firstName + " " + lastName;
    }

    public String getLoginName() {
        return loginName;
    }

    public void setLoginName(String loginName) {
        this.loginName = loginName;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getMail() {
        return mail;
    }

    public void setMail(String mail) {
        this.mail = mail;
    }

    public List<Solution> getSolutions() {
        return solutions;
    }

    public void setSolutions(List<Solution> solutions) {
        this.solutions = solutions;
    }

    public List<CourseUnit> getLecturingCourses() {
        return lecturingUnits;
    }

    public void setLecturingCourses(List<CourseUnit> lecturingCours) {
        this.lecturingUnits = lecturingCours;
    }

    public List<Attendance> getAttendences() {
        return attendences;
    }

    public void setAttendences(List<Attendance> attendences) {
        this.attendences = attendences;
    }

    public List<CourseUnit> getLecturingUnits() {
        return lecturingUnits;
    }

    public void setLecturingUnits(List<CourseUnit> lecturingUnits) {
        this.lecturingUnits = lecturingUnits;
    }

    public List<CourseStaffMembership> getStaffMemberships() {
        return staffMemberships;
    }

    public void setStaffMemberships(List<CourseStaffMembership> groupMemberships) {
        this.staffMemberships = groupMemberships;
    }

    public List<CourseParticipantProfile> getParticipantProfiles() {
        return participantProfiles;
    }

    public Set<Role> getRoles() {
        return roles;
    }

    public DegreeProgram getProgram() {
        return program;
    }

    public void setProgram(DegreeProgram program) {
        this.program = program;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public List<Submission> getSubmissions() {
        return submissions;
    }

    public void setSubmissions(List<Submission> submissions) {
        this.submissions = submissions;
    }

    public List<Submission> getCheckedSubmissions() {
        return checkedSubmissions;
    }

    public void setCheckedSubmissions(List<Submission> checkedSubmissions) {
        this.checkedSubmissions = checkedSubmissions;
    }

    public List<CourseUnit> getParticipatingUnits() {
        return participatingUnits;
    }

    public void setParticipatingUnits(List<CourseUnit> participatingUnits) {
        this.participatingUnits = participatingUnits;
    }

    public List<SolutionComment> getComments() {
        return comments;
    }

    public void setComments(List<SolutionComment> comments) {
        this.comments = comments;
    }

    public boolean isConsentedToCollection() {
        return consentedToCollection;
    }

    public void setConsentedToCollection(boolean consentedToCollection) {
        this.consentedToCollection = consentedToCollection;
    }

    public int getConsentChangeCount() {
        return consentChangeCount;
    }

    public void setConsentChangeCount(int consentChangeCount) {
        this.consentChangeCount = consentChangeCount;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        User user = (User) o;
        return id == user.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }
}
