package de.hsrm.sls.subato.user.repository;

import static de.hsrm.sls.subato.user.repository.UserSpecifications.applyQuery;
import static de.hsrm.sls.subato.user.repository.UserSpecifications.inCourse;
import static de.hsrm.sls.subato.user.repository.UserSpecifications.inUnit;

import de.hsrm.sls.subato.user.User;
import org.springframework.data.jpa.domain.Specification;

public class UserFilter {

    private final String query;
    private final Long unitId;
    private final Long courseId;

    private UserFilter(String query, Long unitId, Long courseId) {
        this.query = query;
        this.unitId = unitId;
        this.courseId = courseId;
    }

    public String getQuery() {
        return query;
    }

    public Long getUnitId() {
        return unitId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public Specification<User> toSpecification() {
        Specification<User> spec = Specification.where(null);

        if (unitId != null) {
            spec = spec.and(inUnit(unitId));
        }

        if (courseId != null) {
            spec = spec.and(inCourse(courseId));
        }

        if (query != null) {
            spec = spec.and(applyQuery(query));
        }

        return spec;
    }

    public static class Builder {

        private String query;
        private Long unitId;
        private Long courseId;

        public Builder() {
        }


        public Builder query(String query) {
            this.query = query;
            return this;
        }

        public Builder unitId(Long unitId) {
            this.unitId = unitId;
            return this;
        }

        public Builder courseId(Long courseId) {
            this.courseId = courseId;
            return this;
        }

        public UserFilter build() {
            return new UserFilter(query, unitId, courseId);
        }
    }
}
