package de.hsrm.sls.subato.user.repository;

import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.exceptions.SortingNotSupportedException;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserNotFoundException;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;

class UserRepositoryImpl implements UserRepositoryCustom {

    @Autowired
    PolicyEvaluator policy;

    @Autowired
    @Lazy
    UserRepository userRepository;

    public User findByPrincipal(SubatoPrincipal principal) {
        if (principal == null) {
            return null;
        }

        return userRepository.findById(principal.getId())
            .orElseThrow(() -> new UserNotFoundException(principal.getId()));
    }

    public List<User> findAll(UserQuery query) {
        Specification<User> spec = createSpec(query);
        var users = userRepository.findAll(spec);
        if (query.getSort() != null) {
            users.sort(query.getSort());
        }

        return users;
    }

    public Page<User> findAll(UserQuery query, Pageable page) {
        Specification<User> spec = createSpec(query);
        if (query.getSort() != null) {
            throw new SortingNotSupportedException();
        }

        return userRepository.findAll(spec, page);
    }

    private Specification<User> createSpec(UserQuery query) {
        Specification<User> spec = query.getFilter().toSpecification();

        var policyContext = query.getPolicyContext();
        if (policyContext != null && policyContext.getPermission() != null) {
            var authSpec = policy.checkAsSpec(query.getPolicyContext());
            spec = spec.and(authSpec);
        }

        return spec;
    }
}
