package de.hsrm.sls.subato.user.repository;

import static de.hsrm.sls.subato.shared.QueryPredicates.ignoreCaseContains;

import de.hsrm.sls.subato.courses.course.model.Course_;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit_;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.User_;
import jakarta.persistence.criteria.JoinType;
import java.util.Optional;
import org.springframework.data.jpa.domain.Specification;

public class UserSpecifications {

    public static Specification<User> applyQuery(String q) {
        return (ex, cq, cb) -> cb.or(
            ignoreCaseContains(cb, Optional.of(q),
                cb.concat(cb.concat(ex.get(User_.firstName), " "), ex.get(User_.lastName))),
            ignoreCaseContains(cb, Optional.of(q),
                cb.concat(cb.concat(ex.get(User_.lastName), ", "), ex.get(User_.firstName))),
            ignoreCaseContains(cb, Optional.of(q), ex.get(User_.loginName))
        );
    }

    public static Specification<User> inUnit(Long unitId) {
        return (ex, cq, cb) -> {
            var units = ex
                .join(User_.participatingUnits, JoinType.LEFT);
            cq.distinct(true);
            return cb.equal(units.get(CourseUnit_.id), unitId);
        };
    }

    public static Specification<User> inCourse(Long courseId) {
        return (ex, cq, cb) -> {
            var units = ex
                .join(User_.participatingUnits, JoinType.LEFT);
            cq.distinct(true);
            return cb.equal(units.get(CourseUnit_.course).get(Course_.id), courseId);
        };
    }
}
