package de.hsrm.sls.subato.aor.core;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNull;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.aor.core.usecase.ImportCoreDataUseCase;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import java.nio.file.Files;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ResourceUtils;

/**
 * ad-hoc integration tests, should be refactored someday to test the individual import steps
 * isolated
 */
@SubatoTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Transactional
public class ImportCoreDataUseCaseTests {

    @Autowired
    private ImportCoreDataUseCase importCoreDataUseCase;

    @Autowired
    private ProgramModuleRepository programModuleRepository;

    @Autowired
    private DegreeProgramRepository programRepository;

    @Test
    public void shouldImport() throws Exception {
        // remember how many programs/modules were already in the fixtures
        var existingPrograms = programRepository.count();
        var existingModules = programModuleRepository.count();

        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/core-data.json");
        importCoreDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));

        // check the delta: how many have been added
        assertEquals(2, programRepository.count() - existingPrograms);
        assertEquals(4, programModuleRepository.count() - existingModules);

        // thesis should be ignored
        assertNull(programModuleRepository.findBySupplementId(1247808L));

        // should be deactivated because they are not in core-data.json
        assertFalse(programRepository.findByPosCodeAndPoYear("IAB", 2017).isActive());
        assertFalse(programRepository.findByPosCodeAndPoYear("IBB", 2016).isActive());
    }
}
