package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.dto.CreateCourseDto;
import de.hsrm.sls.subato.utils.JsonHelper;
import java.util.ArrayList;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class CreateCourseTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    private CreateCourseDto create;

    @BeforeEach
    public void setup() {
        create = new CreateCourseDto();
        create.setName("Name");
        create.setCourseId(1L);
        create.setShortName("ShortName");
        create.setTermId(null);
        create.setModuleIds(new ArrayList<>());
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaff() throws Exception {
        this.mockMvc.perform(postJson("/course")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(postJson("/course")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(postJson("/course")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
