package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.putJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.dto.UpdateFinishedDto;
import de.hsrm.sls.subato.utils.JsonHelper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class FinishCourseTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    private UpdateFinishedDto update;

    @BeforeEach
    public void setup() {
        update = new UpdateFinishedDto();
        update.setFinished(true);
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldDenyCourseTutor() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldDenyCourseLecturer() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant() throws Exception {
        this.mockMvc.perform(putJson("/course/1/finished").content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
