package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.dto.CourseDto;
import de.hsrm.sls.subato.utils.JsonHelper;
import java.util.List;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetAllCoursesTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    @Test
    public void shouldAllowGuests() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2), jsonList);
        assertCoursesAbsent(List.of(3, 4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudents() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3), jsonList);
        assertCoursesAbsent(List.of(4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaff() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3, 4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3), jsonList);
        assertCoursesAbsent(List.of(4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldAllowCourse4Admin() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3, 4), jsonList);
        assertCoursesAbsent(List.of(5), jsonList);
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldAllowCourseTutor() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3), jsonList);
        assertCoursesAbsent(List.of(4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_C4T")
    public void shouldAllowCourse4Tutor() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3, 4), jsonList);
        assertCoursesAbsent(List.of(5), jsonList);
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldAllowCourseLecturer() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3), jsonList);
        assertCoursesAbsent(List.of(4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_C4L")
    public void shouldAllowCourse4Lecturer() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3, 4), jsonList);
        assertCoursesAbsent(List.of(5), jsonList);
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldAllowCourseParticipant() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3), jsonList);
        assertCoursesAbsent(List.of(4, 5), jsonList);
    }

    @Test
    @WithUserDetails("U_STUD_C4P")
    public void shouldAllowCourse4Participant() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<CourseDto> jsonList = jsonHelper.asJsonList(result.getResponse().getContentAsString());
        assertCoursesPresent(List.of(1, 2, 3, 4), jsonList);
        assertCoursesAbsent(List.of(5), jsonList);
    }

    private void assertCoursesPresent(List<Integer> courseIds, List<CourseDto> courses) {
        for (var courseId : courseIds) {
            assertTrue(courses.stream().anyMatch(t -> t.getId() == courseId));
        }
    }

    private void assertCoursesAbsent(List<Integer> courseIds, List<CourseDto> courses) {
        for (var courseId : courseIds) {
            assertFalse(courses.stream().anyMatch(t -> t.getId() == courseId));
        }
    }
}
