package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetCourseDescriptionTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private CourseRepository courseRepository;

    @Test
    @DirtiesContext
    @WithUserDetails("U_C5A")
    public void shouldSignImageInDescription() throws Exception {
        var course = courseRepository.findById(5L).get();
        course.setDescription(
            "<img src=\"http://localhost/course_file/80/download\">");
        courseRepository.save(course);

        var response = this.mockMvc.perform(getJson("/course/5/description"))
            .andDo(print()).andExpect(status().isOk()).andReturn();
        var description = response.getResponse().getContentAsString();
        assertTrue(description.contains("/80/download?t="));
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C5A")
    public void shouldNotSignImageInDescription() throws Exception {
        var course = courseRepository.findById(5L).get();
        course.setDescription("<img src=\"http://externalsite.de/course_file/80/download\">");
        courseRepository.save(course);

        var response = this.mockMvc.perform(getJson("/course/5/description"))
            .andDo(print()).andExpect(status().isOk()).andReturn();
        var description = response.getResponse().getContentAsString();
        assertFalse(description.contains("/80/download?t="));
    }
}
