package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetCourseFilesTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    public void shouldAllowGuestsFilesFromPublicCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/1/files"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuestsFilesFromInternalCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/3/files"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    public void shouldDenyGuestsFilesFromParticipantCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/4/files"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    public void shouldDenyGuestsFilesFromPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5/files"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudentsFilesFromPublicCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/1/files"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudentsFilesFromInternalCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/3/files"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudentsFilesFromParticipantCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/4/files"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudentsFilesFromPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5/files"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldDenyCourseAdminFilesFromPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5/files"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C4P")
    public void shouldAllowCourseParticipantFilesFromParticipantCourse() throws Exception {
        this.mockMvc.perform(getJson("/course/4/files"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_STUD_C5P")
    public void shouldDenyCourseParticipantFilesFromPrivateCourse() throws Exception {
        this.mockMvc.perform(getJson("/course/5/files"))
            .andDo(print()).andExpect(status().isForbidden())
            .andReturn();
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaffFilesFromPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5/files"))
            .andDo(print()).andExpect(status().isOk());
    }
}
