package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetCourseParticipantsTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipants() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldAllowCourseLecturers() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldAllowCourseTutors() throws Exception {
        this.mockMvc.perform(getJson("/course/1/participants"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdminFromOtherCourse() throws Exception {
        this.mockMvc.perform(getJson("/course/2/participants"))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
