package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class ReimportCourseTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    @DirtiesContext
    @WithUserDetails("U_SA")
    public void shouldAllowSuperAdmin() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_SA")
    public void shouldPreventIfFinished() throws Exception {
        this.mockMvc.perform(postJson("/course/6/reimport"))
            .andDo(print()).andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldDenyCourseTutor() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldDenyCourseLecturer() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant() throws Exception {
        this.mockMvc.perform(postJson("/course/1/reimport"))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
