package de.hsrm.sls.subato.exercises.exercise;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.exercises.exercise.dto.CreateExerciseDto;
import de.hsrm.sls.subato.exercises.exercise.model.AssessmentMode;
import de.hsrm.sls.subato.utils.JsonHelper;
import java.time.LocalDateTime;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class CreateExerciseTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    private CreateExerciseDto exerciseDto;

    @BeforeEach
    public void setup() {
        exerciseDto = new CreateExerciseDto();
        exerciseDto.setCourseId(1);
        exerciseDto.setDescription("desc");
        exerciseDto.setSummary("summary");
        exerciseDto.setName("name");
        exerciseDto.setAssessmentMode(AssessmentMode.Binary);
        exerciseDto.setDueTo(LocalDateTime.MAX);
        exerciseDto.setVisibleFrom(LocalDateTime.now());
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldDenyCourseTutor() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldDenyCourseLecturer() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldDenyOtherCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C6A")
    public void shouldPreventIfCourseFinished() throws Exception {
        exerciseDto.setCourseId(6);
        this.mockMvc.perform(postJson("/exercise")
                .content(jsonHelper.asJsonString(exerciseDto)))
            .andDo(print()).andExpect(status().isBadRequest());
    }
}
