package de.hsrm.sls.subato.exercises.submission;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.assessment.submission.dto.UpdatePassedDto;
import de.hsrm.sls.subato.utils.JsonHelper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class UpdatePassedSubmissionTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    private UpdatePassedDto updatePassedDto;

    @BeforeEach
    public void setup() {
        updatePassedDto = new UpdatePassedDto();
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1L")
    public void shouldAllowCourseLecturer() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldDenyCourseTutor() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldDenyDifferentCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudent() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(postJson("/submission/1/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C6A")
    public void shouldPreventIfCourseFinished() throws Exception {
        this.mockMvc.perform(postJson("/submission/3/passed")
                .content(jsonHelper.asJsonString(updatePassedDto)))
            .andDo(print()).andExpect(status().isBadRequest());
    }
}
