package de.hsrm.sls.subato.modules.term;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.utils.JsonHelper;
import java.time.LocalDate;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class CreateNewTermTest {


    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    private CreateTermDto create;

    @BeforeEach
    public void setup() {
        create = new CreateTermDto();
        create.setName("WS69");
        create.setLecturesStartAt(LocalDate.MIN);
        create.setLecturesEndAt(LocalDate.MAX);
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_SA")
    public void shouldAllowSuperAdmin() throws Exception {
        this.mockMvc.perform(postJson("/term")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(postJson("/term")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(postJson("/term")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(postJson("/term")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/term")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant() throws Exception {
        this.mockMvc.perform(postJson("/term")
                .content(jsonHelper.asJsonString(create)))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
