package de.hsrm.sls.subato.modules.term;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class FinishTermTest {


    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private CourseRepository courseRepository;

    @Test
    @DirtiesContext
    @WithUserDetails("U_SA")
    public void shouldAllowSuperAdmin() throws Exception {
        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_SA")
    public void shouldFinishCourses() throws Exception {
        Assertions.assertFalse(courseRepository.findById(1L).get().isFinished());
        Assertions.assertFalse(courseRepository.findById(2L).get().isFinished());
        Assertions.assertFalse(courseRepository.findById(3L).get().isFinished());

        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isOk());

        Assertions.assertTrue(courseRepository.findById(1L).get().isFinished());
        Assertions.assertTrue(courseRepository.findById(2L).get().isFinished());
        Assertions.assertFalse(courseRepository.findById(3L).get().isFinished());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdmin() throws Exception {
        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant() throws Exception {
        this.mockMvc.perform(postJson("/term/1/finish"))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
