package de.hsrm.sls.subato.modules.term;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.utils.JsonHelper;
import java.util.List;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetAllTermsTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    @Test
    public void shouldAllowGuests() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1), dtoList);
        assertTermsAbsent(List.of(3, 4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudents() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3), dtoList);
        assertTermsAbsent(List.of(4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaff() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3, 4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin1() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3), dtoList);
        assertTermsAbsent(List.of(4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldAllowCourseAdmin2() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3, 4), dtoList);
        assertTermsAbsent(List.of(5), dtoList);
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldAllowCourseLecturer1() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3), dtoList);
        assertTermsAbsent(List.of(4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_C4L")
    public void shouldAllowCourseLecturer2() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3, 4), dtoList);
        assertTermsAbsent(List.of(5), dtoList);
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldAllowCourseTutor1() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3), dtoList);
        assertTermsAbsent(List.of(4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_C4T")
    public void shouldAllowCourseTutor2() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3, 4), dtoList);
        assertTermsAbsent(List.of(5), dtoList);
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldAllowCourseParticipant1() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3), dtoList);
        assertTermsAbsent(List.of(4, 5), dtoList);
    }

    @Test
    @WithUserDetails("U_STUD_C4P")
    public void shouldAllowCourseParticipant2() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        List<TermDto> dtoList = jsonHelper.termAsJsonList(
            result.getResponse().getContentAsString());
        assertTermsPresent(List.of(1, 3, 4), dtoList);
        assertTermsAbsent(List.of(5), dtoList);
    }

    private void assertTermsPresent(List<Integer> termIds, List<TermDto> terms) {
        for (var termId : termIds) {
            assertTrue(terms.stream().anyMatch(t -> t.getId() == termId));
        }
    }

    private void assertTermsAbsent(List<Integer> termIds, List<TermDto> terms) {
        for (var termId : termIds) {
            assertFalse(terms.stream().anyMatch(t -> t.getId() == termId));
        }
    }
}
