package de.hsrm.sls.subato.modules.term;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.utils.JsonHelper;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetOneTermTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    @Test
    public void shouldAllowGuests() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        TermDto termDto = jsonHelper.jsonAsTermDto(result.getResponse().getContentAsString());
        assertEquals(1, termDto.getId());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(getJson("/term/3"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudents() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        TermDto termDto = jsonHelper.jsonAsTermDto(result.getResponse().getContentAsString());
        assertEquals(1, termDto.getId());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(getJson("/term/4"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaff() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term/4"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        TermDto termDto = jsonHelper.jsonAsTermDto(result.getResponse().getContentAsString());
        assertEquals(4, termDto.getId());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdmin1() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        TermDto termDto = jsonHelper.jsonAsTermDto(result.getResponse().getContentAsString());
        assertEquals(1, termDto.getId());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdmin2() throws Exception {
        this.mockMvc.perform(getJson("/term/4"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldAllowCourseParticipant1() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/term/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        TermDto termDto = jsonHelper.jsonAsTermDto(result.getResponse().getContentAsString());
        assertEquals(1, termDto.getId());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipant2() throws Exception {
        this.mockMvc.perform(getJson("/term/5"))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
