package de.hsrm.sls.subato.shared.markdown;

import static org.junit.jupiter.api.Assertions.assertEquals;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;

@SubatoTest
@Tag(Tags.UNIT)
public class MarkdownParserTest {

    private MarkdownParser parser;

    @BeforeEach
    public void setup() {
        parser = new MarkdownParser();
    }

    @Test
    public void shouldRenderHeadingWithText() {
        var content = """
            # Überschrift
            Ein Text. Toll.
            """;

        assertEquals(parser.convertToHtml(content), """
            <h1>Überschrift</h1>
            <p>Ein Text. Toll.</p>
            """);
    }

    @Test
    public void shouldRenderLatex1() {
        var content = """
            $$
            x^2
            $$
            """;

        assertEquals(parser.convertToHtml(content), "$$x^2$$");
    }

    @Test
    public void shouldRenderLatex2() {
        var content = """
            $$x^2$$
            """;

        assertEquals(parser.convertToHtml(content), "$$x^2$$");
    }

    @Test
    public void shouldRenderLatex3() {
        var content = "$x^2$";
        assertEquals(parser.convertToHtml(content), "$x^2$");
    }

    @Test
    public void shouldRenderComplexLatex1() {
        var content = """
            $$
            \\textrm{nTimes}(f,n,x) = \\left\\{ \\begin{array}{ll}x & \\textrm{für $n\\le 0$}\\\\\\textrm{nTimes}(f,n-1,f(x)) & \\textrm{für $n>0$}\\end{array}\\right.
            $$
            """;

        assertEquals(parser.convertToHtml(content),
            "$$\\textrm{nTimes}(f,n,x) = \\left\\{ \\begin{array}{ll}x &amp; \\textrm{für $n\\le 0$}\\\\\\textrm{nTimes}(f,n-1,f(x)) &amp; \\textrm{für $n&gt;0$}\\end{array}\\right.$$");
    }

    @Test
    public void shouldRenderComplexLatex2() {
        var content = "$c = \\pm\\sqrt\\{a^2 + b^2\\}$";
        assertEquals(parser.convertToHtml(content), "$c = \\pm\\sqrt\\{a^2 + b^2\\}$");
    }

    @Test
    public void shouldRenderComplexLatex3() {
        var content = """
            $c = \\pm\\sqrt\\{a^2 + b^2\\}
            $
            """;
        assertEquals(parser.convertToHtml(content), "$c = \\pm\\sqrt\\{a^2 + b^2\\}$");
    }

    @Test
    public void shouldRenderComplexLatex4() {
        var content = """
            $$\\textrm{nTimes}(f,n,x) = \\left\\{ \\begin{array}{ll}
                         x & \\textrm{für $n\\le 0$}\\\\
                         \\textrm{nTimes}(f,n-1,f(x)) & \\textrm{für $n>0$}
                         \\end{array}\\right.$$
            """;
        assertEquals(parser.convertToHtml(content),
            """
                $$\\textrm{nTimes}(f,n,x) = \\left\\{ \\begin{array}{ll}
                             x &amp; \\textrm{für $n\\le 0$}\\\\
                             \\textrm{nTimes}(f,n-1,f(x)) &amp; \\textrm{für $n&gt;0$}
                             \\end{array}\\right.$$""");
    }

    @Test
    public void shouldRenderHeadingWithTextAndMath() {
        var content = """
            # Überschrift
            Ein Text. Toll.
            $c = \\pm\\sqrt\\{a^2 + b^2\\}$
            Und noch etwas Text.
            ```java
            int x = 17;
            ```
            """;

        assertEquals(parser.convertToHtml(content), """
            <h1>Überschrift</h1>
            <p>Ein Text. Toll.</p>
            $c = \\pm\\sqrt\\{a^2 + b^2\\}$
            <p>Und noch etwas Text.</p>
            <pre class="language-java"><code class="language-java">int x = 17;
            </code></pre>
            """);
    }
}
