package de.hsrm.sls.subato.solutions.evaluation.request;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;


@SubatoTest
@Tag(Tags.UNIT)
public class ExactMatcherTest {

    @Test
    public void shouldMatch1() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren()
            .add(new FileTree("First.java", "First.java", "text/x-java-source", true));

        var matcher = new ExactMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldMatch2() {
        var templateFile = new FileRef();
        templateFile.setPath("ABC/First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren()
            .add(new FileTree("First.java", "ABC/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new ExactMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldNotMatch1() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren()
            .add(new FileTree("First.java", "ABC/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new ExactMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        assertNull(tree);
    }

    @Test
    public void shouldNotMatch2() {
        var templateFile = new FileRef();
        templateFile.setPath("ABC/First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren()
            .add(new FileTree("First.java", "First.java", "text/x-java-source", true));

        var matcher = new ExactMatcher();
        var tree = matcher.match(solutionTree, templateFile);

        assertNull(tree);
    }
}
