package de.hsrm.sls.subato.solutions.evaluation.request;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;

@SubatoTest
@Tag(Tags.UNIT)
public class PrefixMatcherTest {

    @Test
    public void shouldMatch1() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren()
            .add(new FileTree("First.java", "ABC/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("*");
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldMatch2() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren()
            .add(new FileTree("First.java", "ABC/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("ABC");
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldMatch3() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren().add(new FileTree("src", "ABC/src", "", false));
        dir.getChildren().get(0).getChildren()
            .add(new FileTree("First.java", "ABC/src/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("ABC/src");
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldMatch4() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren().add(new FileTree("src", "ABC/src", "", false));
        dir.getChildren().get(0).getChildren()
            .add(new FileTree("First.java", "ABC/src/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("*/src");
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldMatch5() {
        var templateFile = new FileRef();
        templateFile.setPath("package/First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("src", "src", "", false);
        dir.getChildren().add(new FileTree("package", "src/package", "", false));
        dir.getChildren().get(0).getChildren()
            .add(new FileTree("First.java", "src/package/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("*");
        var tree = matcher.match(solutionTree, templateFile);

        assertNotNull(tree);
    }

    @Test
    public void shouldNotMatch1() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren()
            .add(new FileTree("First.java", "First.java", "text/x-java-source", true));

        var matcher = new PrefixMatcher("*");
        var tree = matcher.match(solutionTree, templateFile);

        assertNull(tree);
    }

    @Test
    public void shouldNotMatch2() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        solutionTree.getChildren()
            .add(new FileTree("First.java", "First.java", "text/x-java-source", true));

        var matcher = new PrefixMatcher("ABC");
        var tree = matcher.match(solutionTree, templateFile);

        assertNull(tree);
    }

    @Test
    public void shouldNotMatch3() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("ABC", "ABC", "", false);
        dir.getChildren()
            .add(new FileTree("First.java", "ABC/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("DEF");
        var tree = matcher.match(solutionTree, templateFile);

        assertNull(tree);
    }

    @Test
    public void shouldNotMatch4() {
        var templateFile = new FileRef();
        templateFile.setPath("First.java");

        var solutionTree = new FileTree();
        var dir = new FileTree("src", "src", "", false);
        dir.getChildren().add(new FileTree("package", "src/package", "", false));
        dir.getChildren().get(0).getChildren()
            .add(new FileTree("First.java", "src/package/First.java", "text/x-java-source", true));
        solutionTree.getChildren().add(dir);

        var matcher = new PrefixMatcher("*");
        var tree = matcher.match(solutionTree, templateFile);

        assertNull(tree);
    }
}
