package de.hsrm.sls.subato.solutions.solution;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import java.util.Optional;
import java.util.UUID;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;

/**
 * Integration Test für {@link SolutionController#getFile(Long, UUID, Optional)}
 */
@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetOneSolutionTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudent() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdminsFromSubmittedCourse() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldAllowCourseLecturersFromSubmittedCourse() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldAllowCourseTutorsFromSubmittedCourse() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldDenyCourseAdminFromOtherCourse() throws Exception {
        this.mockMvc.perform(getJson("/solution/1"))
            .andExpect(status().isForbidden());
    }

}
