package de.hsrm.sls.subato.tasks.pool.sync;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.tasks.pool.repository.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.repository.GitConfig;
import de.hsrm.sls.subato.tasks.pool.sync.repository.TaskPoolRepositoryManager;
import de.hsrm.sls.subato.utils.SubatoTmpDirCleanup;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration test for managing git repositories (task pools)
 * <p>
 * Remote dependencies: - GitLab (access to repository of task pool #1)
 */
@SubatoTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Tag(Tags.EXTERNAL_DEPENDENCY)
@Transactional
public class TaskPoolRepositoryManagerTests {

    @Autowired
    TaskPoolRepositoryManager repositoryManager;

    @Autowired
    TaskPoolRepository poolRepository;

    @BeforeEach
    public void beforeAll(@Autowired SubatoTmpDirCleanup tmpDirCleanup) {
        tmpDirCleanup.exec();
    }

    @Test
    public void shouldClonePublicRepo() {
        var pool = poolRepository.findById(1L).get();
        assertDoesNotThrow(() -> repositoryManager.cloneRepository(pool));
    }

    /**
     * setting {@link GitConfig#getToken()} for this test is required
     */
    @Test
    public void shouldClonePrivateRepo() {
        var pool = poolRepository.findById(2L).get();
        assertDoesNotThrow(() -> repositoryManager.cloneRepository(pool));
    }
}
